@@ -18,13 +18,13 @@
 #include <config.h>
 #include <gnome.h>
 
-void hello_cb (GtkWidget *widget, void *data);
-void about_cb (GtkWidget *widget, void *data);
-void quit_cb (GtkWidget *widget, void *data);
-void add_cb (GtkWidget *widget, void *data);
-void remove_cb (GtkWidget *widget, void *data);
+static void hello_cb (GtkWidget *widget, void *data);
+static void about_cb (GtkWidget *widget, void *data);
+static void quit_cb (GtkWidget *widget, void *data);
+static void add_cb (GtkWidget *widget, void *data);
+static void remove_cb (GtkWidget *widget, void *data);
 
-void prepare_app();
+static void prepare_app(void);
 
 GtkWidget *app;
 
@@ -58,8 +58,6 @@ GnomeUIInfo main_menu[] = {
     GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
   { GNOME_APP_UI_SUBTREE, ("Changes"), NULL, empty_menu, NULL, NULL,
     GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
-  { GNOME_APP_UI_JUSTIFY_RIGHT, NULL, NULL, NULL, NULL, NULL,
-    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
   { GNOME_APP_UI_SUBTREE, ("Help"), NULL, help_menu, NULL, NULL,
     GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
   { GNOME_APP_UI_ENDOFINFO }
@@ -68,13 +66,10 @@ GnomeUIInfo main_menu[] = {
 int
 main(int argc, char *argv[])
 {
-  argp_program_version = VERSION;
-
   /* gnome_init() is always called at the beginning of a program.  it
      takes care of initializing both Gtk and GNOME.  It also parses
      the command-line arguments.  */
-  gnome_init ("gnome-hello-6-dynamic-menus", NULL, argc, argv,
-	      0, NULL);
+  gnome_init ("gnome-hello-6-dynamic-menus", VERSION, argc, argv);
 
   /* prepare_app() makes all the gtk calls necessary to set up a
      minimal Gnome application; It's based on the hello world example
@@ -86,8 +81,8 @@ main(int argc, char *argv[])
   return 0;
 }
 
-void
-prepare_app()
+static void
+prepare_app(void)
 {
   GtkWidget *button;
   GtkMenuFactory *mf;
@@ -108,7 +103,7 @@ prepare_app()
   button = gtk_button_new_with_label ("Hello GNOME");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
      		      GTK_SIGNAL_FUNC (hello_cb), NULL);
-  gtk_container_border_width (GTK_CONTAINER (button), 60);
+  gtk_container_set_border_width (GTK_CONTAINER (button), 60);
   gnome_app_set_contents ( GNOME_APP (app), button);
 
   /* We now show the widgets, the order doesn't matter, but i suggests 
@@ -121,7 +116,7 @@ prepare_app()
 
 /* Callbacks functions */
 
-void
+static void
 hello_cb (GtkWidget *widget, void *data)
 {
   g_print ("Hello GNOME\n");
@@ -129,25 +124,25 @@ hello_cb (GtkWidget *widget, void *data)
   return;
 }
 
-void
+static void
 quit_cb (GtkWidget *widget, void *data)
 {
   gtk_main_quit ();
   return;
 }
 
-void
+static void
 added_item_callback (GtkWidget *widget, gpointer data)
 {
-  int num = (int) data;
+  gint num = GPOINTER_TO_INT (data);
   printf ("menu item %d\n", num);
 }
 
 
-void
+static void
 add_cb (GtkWidget *widget, void *data)
 {
-  static int counter = 0;
+  static gint counter = 0;
   char buf[200];
   GnomeUIInfo entry[2];
   int pos = 0;
@@ -158,7 +153,7 @@ add_cb (GtkWidget *widget, void *data)
   entry[0].label = buf;
   entry[0].hint = NULL;
   entry[0].moreinfo = added_item_callback;
-  entry[0].user_data = (gpointer) counter;
+  entry[0].user_data = GINT_TO_POINTER (counter);
   entry[0].unused_data = NULL;
   entry[0].pixmap_type = GNOME_APP_PIXMAP_NONE;
   entry[0].pixmap_info = NULL;
@@ -176,7 +171,7 @@ add_cb (GtkWidget *widget, void *data)
   return;
 }
 
-void
+static void
 remove_cb (GtkWidget *widget, void *data)
 {
   printf ("remove\n");
@@ -187,11 +182,11 @@ remove_cb (GtkWidget *widget, void *data)
 }
 
 
-void
+static void
 about_cb (GtkWidget *widget, void *data)
 {
   GtkWidget *about;
-  gchar *authors[] = {
+  const gchar *authors[] = {
 /* Here should be your names */
 	  "Mark Galassi",
 	  "Horacio J. Pea",
