@@ -1,14 +1,14 @@
 #ifndef lint
-static char rcsId[]="$Header: /cvs/gnome/gnome-libs/gtk-xmhtml/debug.c,v 1.3 1998/02/12 03:08:34 unammx Exp $";
+static char rcsId[]="$Header: /cvs/gnome/gnome-libs/gtk-xmhtml/debug.c,v 1.4 1998/12/03 06:29:09 sopwith Exp $";
 #endif
 /*****
 * debug.c : debug initialization routines.
 *
-* This file Version	$Revision: 1.3 $
+* This file Version	$Revision: 1.4 $
 *
 * Creation date:		Fri Oct 18 23:57:23 GMT+0100 1996
-* Last modification: 	$Date: 1998/02/12 03:08:34 $
-* By:					$Author: unammx $
+* Last modification: 	$Date: 1998/12/03 06:29:09 $
+* By:					$Author: sopwith $
 * Current State:		$State: Exp $
 *
 * Author:				newt
@@ -35,6 +35,16 @@ static char rcsId[]="$Header: /cvs/gnome/gnome-libs/gtk-xmhtml/debug.c,v 1.3 199
 /*****
 * ChangeLog 
 * $Log: debug.c,v $
+* Revision 1.4  1998/12/03 06:29:09  sopwith
+*
+*
+* gtk-xmhtml: fix something bacchus pointed out (initialization of global
+* variables from non-constant values).
+*
+* libgnome/gnome-exec.c: redo gnome_execute_async_with_env() to fix
+* 		       Tom's bugs so far - he will probably want to
+* 		       double-check it though.
+*
 * Revision 1.3  1998/02/12 03:08:34  unammx
 * Merge to Koen's XmHTML 1.1.2 + following fixes:
 *
@@ -117,7 +127,7 @@ static char rcsId[]="$Header: /cvs/gnome/gnome-libs/gtk-xmhtml/debug.c,v 1.3 199
 /*** Public Variable Declarations ***/
 int xmhtml_debug_levels_defined[MAX_DEBUG_LEVELS];
 int xmhtml_debug_full;
-FILE *__rsd_debug_file = stdout;
+FILE *__rsd_debug_file = NULL;
 int debug_disable_warnings = 0;
 
 /*** Private Datatype Declarations ****/
