@@ -10,18 +10,20 @@
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.
    
-   This program is distributed in the hope that it will be useful,
+   The Gnome Library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   You should have received a copy of the GNU Library General Public
+   License along with the Gnome Library; see the file COPYING.LIB.  If not,
+   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
 
 #include <config.h>
 #include <stdio.h>
 #include <string.h>
+#include <unistd.h>	/* unlink() */
 #include <stdlib.h>	/* atoi() */
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -73,15 +75,11 @@ typedef struct TProfile {
 	char *filename;
 	time_t mtime;
 	int written_to;
+	int to_be_deleted;
 	TSecHeader *section;
 	struct TProfile *link;
 } TProfile;
 
-typedef struct {
-	char *file, *section, *key, *def;
-	char *path, *opath;
-} ParsedPath;
-
 /*
  * Prefix for all the configuration operations
  * iff the path does not begin with / or with #
@@ -98,11 +96,13 @@ static TProfile *Current = 0;
  */
 static TProfile *Base = 0;
 
-/* a set handler is a function which is called every time a gnome_config_set_*
-   function is called, this can be used by the app to say guarantee a sync,
-   apps using libgnomeui should not call this, libgnomeui already provides
-   this, this would be for non-gui apps and apps that use a different toolkit
-   then gtk*/
+/*
+ * a set handler is a function which is called every time a gnome_config_set_*
+ * function is called, this can be used by the app to say guarantee a sync,
+ * apps using libgnomeui should not call this, libgnomeui already provides
+ * this, this would be for non-gui apps and apps that use a different toolkit
+ * than gtk.
+ */
 static void (*set_handler)(void *data) = NULL;
 static void *set_handler_data;
 
@@ -115,88 +115,50 @@ static void *sync_handler_data;
 #define CALL_SYNC_HANDLER() { if(sync_handler) (*sync_handler)(sync_handler_data); }
 
 
-static void
-release_path (ParsedPath *p)
-{
-	g_free (p->opath);
-	g_free (p);
-}
 
-static ParsedPath *
-parse_path (const char *path, gint priv)
-{
-	ParsedPath *p = g_malloc (sizeof (ParsedPath));
-	char *sep;
+/* The `release_path' and `parsed_path' routines are inside the
+   following file.  It is in a separate file to allow the test-suite
+   to get at it, without needing to export special symbols.
 
-	g_assert(path != NULL);
-	
-	if (*path == '/' || prefix == NULL)
-		p->opath = g_strdup (path);
-	else
-		p->opath = g_copy_strings (prefix, path, NULL);
+   typedef struct {
+	char *file, *section, *key, *def;
+	char *path, *opath;
+   } ParsedPath;
 
-	p->path = p->opath;
-
-	if (*p->path == '='){
-		/* If it is an absolute path name */
-		p->path++;
-		p->file    = strtok (p->path, "=");
-		p->section = strtok (NULL, "/=");
-		p->key     = strtok (NULL, "=");
-		p->def     = strtok (NULL, "=");
-	} else {
-		char *end;
-		sep = "/=";
-
-		p->file    = p->path;
-		p->def     = NULL;
-		p->section = NULL;
-		p->key     = NULL;
-		if ((end = strchr (p->path, '='))) {
-			*end = 0;
-			p->def = end + 1;
-		} else 
-			end = p->path + strlen (p->path);
-
-		/* Look backwards for a slash, to split key from the filename/section */
-		while (end > p->path){
-			end--;
-			if (*end == '/'){
-				*end = 0;
-				p->key = end + 1;
-				break;
-			}
-		}
-
-		/* Look backwards for the next slash, to get the section name */
-		while (end > p->path){
-			end--;
-			if (*end == '/'){
-				*end = 0;
-				p->section = end + 1;
-				break;
-			}
-		}
-		if (*p->file == '/')
-			p->file++;
-		if(priv)
-			p->file = g_concat_dir_and_file (gnome_user_private_dir,
-							 p->file);
-		else
-			p->file = g_concat_dir_and_file (gnome_user_dir,
-							 p->file);
-	}
-	return p;
-}
+   static void release_path (ParsedPath *p);
+   static ParsedPath *parse_path (const char *path, gint priv); */
+#include "parse-path.cP"
 
 static int 
 is_loaded (const char *filename, TSecHeader **section)
 {
 	TProfile *p = Base;
+	TProfile *lastp = NULL;
 	struct stat st;
 	
+	/*if the last one we accessed was this one we don't want to
+	  search*/
+	if(Current && strcasecmp (filename, Current->filename) == 0){
+		if (stat (filename, &st) == -1)
+			st.st_mtime = 0;
+		if (Current->mtime != st.st_mtime)
+			return 0;
+		*section = Current->section;
+		return 1;
+	}
+	
 	while (p){
-		if (strcasecmp (filename, p->filename) == 0){
+		/*search and destroy empty nodes*/
+		if (p->filename[0]=='\0') {
+			TProfile *next = p->link;
+			if(lastp)
+				lastp->link = next;
+			else /*the next one is the first one actually*/
+				Base = next;
+			g_free(p->filename);
+			g_free(p);
+			p = next;
+		} else if (strcasecmp (filename, p->filename) == 0){
 			if (stat (filename, &st) == -1)
 				st.st_mtime = 0;
 			if (p->mtime != st.st_mtime)
@@ -204,8 +166,10 @@ is_loaded (const char *filename, TSecHeader **section)
 			Current = p;
 			*section = p->section;
 			return 1;
+		} else {
+			lastp = p;
+			p = p->link;
 		}
-		p = p->link;
 	}
 	return 0;
 }
@@ -415,6 +379,7 @@ access_config (access_type mode, const char *section_name,
 		New->section = load (filename);
 		New->mtime = st.st_mtime;
 		New->written_to = FALSE;
+		New->to_be_deleted = FALSE;
 		Base = New;
 		section = New->section;
 		Current = New;
@@ -569,42 +534,6 @@ check_path(char *path, mode_t newmode)
 	return TRUE;
 }
 
-static void 
-dump_profile (TProfile *p)
-{
-	FILE *profile;
-    
-	if (!p)
-		return;
-	dump_profile (p->link);
-	
-	/*was this profile written to?, if not it's not necessary to dump
-	  it to disk*/
-	if(!p->written_to)
-		return;
-
-	/* .ado: p->filename can be empty, it's better to jump over */
-	if (p->filename[0] != (char) 0)
-		if (check_path(p->filename,0755) &&
-		    (profile = fopen (p->filename, "w")) != NULL){
-			dump_sections (profile, p->section);
-			fclose (profile);
-		}
-	
-	/*mark this to not be dumped any more*/
-	p->written_to = FALSE;
-}
-
-/*
- * Must be called at the end.
-*/
-void 
-gnome_config_sync (void)
-{
-	dump_profile (Base);
-	CALL_SYNC_HANDLER();
-}
-
 static void 
 free_keys (TKeys *p)
 {
@@ -634,12 +563,98 @@ free_profile (TProfile *p)
 {
 	if (!p)
 		return;
+	if(Current == p)
+		Current = NULL;
 	free_profile (p->link);
 	free_sections (p->section);
 	g_free (p->filename);
 	g_free (p);
 }
 
+
+static void 
+dump_profile (TProfile *p)
+{
+	FILE *profile;
+    
+	if (!p)
+		return;
+	dump_profile (p->link);
+	
+	/*
+	 * was this profile written to?, if not it's not necessary to dump
+	 * it to disk
+	 */
+	if (!p->to_be_deleted)
+		if(!p->written_to)
+			return;
+	
+
+	/* .ado: p->filename can be empty, it's better to jump over */
+	if (p->filename[0] != (char) 0) {
+
+		/*
+		 * this file was added to after it was cleaned so it doesn't
+		 * want to be deleted
+		 */
+		if(p->to_be_deleted && p->section)
+			p->to_be_deleted = FALSE;
+		if(p->to_be_deleted) {
+			/*remove the file and remove all it's ramaints
+			  from memory*/
+			unlink(p->filename);
+			/* this already must have been true */
+			/*p->section = 0;*/
+			p->filename [0] = 0;
+			p->written_to = TRUE;
+			p->to_be_deleted = FALSE;
+			if(p==Current)
+				Current = NULL;
+		} else if (check_path(p->filename,0755) &&
+		    (profile = fopen (p->filename, "w")) != NULL){
+			dump_sections (profile, p->section);
+			fclose (profile);
+		}
+	}
+	
+	/*mark this to not be dumped any more*/
+	p->written_to = FALSE;
+}
+
+/**
+ * gnome_config_sync:
+ *
+ * Writes all of the information modified by gnome-config to the
+ * disk.
+ *
+ * Note: the gnome_config code does not write anything to the
+ * configuration files until this routine is actually invoked.
+ */
+void 
+gnome_config_sync (void)
+{
+	dump_profile (Base);
+	CALL_SYNC_HANDLER();
+}
+
+/**
+ * gnome_config_clean_file: 
+ * @path: A gnome-config path
+ *
+ * Cleans up the configuration file specified by @path from any
+ * configuration information.
+ *
+ * Changes will take place after gnome_config_sync has been invoked.
+ */
+/**
+ * gnome_config_private_clean_file:
+ * @path: A gnome-config path
+ *
+ * Cleans up the private configuration file specified by @path from
+ * any configuration information.
+ *
+ * Changes will take place after gnome_config_sync has been invoked.
+ */
 void 
 _gnome_config_clean_file (const char *path, gint priv)
 {
@@ -653,6 +668,72 @@ _gnome_config_clean_file (const char *path, gint priv)
 	fake_path = g_copy_strings (path, "/section/key", NULL);
 	pp = parse_path (fake_path, priv);
 	g_free (fake_path);
+
+	/* this is the current so don't look no further*/
+	if (Current && strcmp (pp->file, Current->filename) == 0) {
+		free_sections (Current->section);
+		Current->section = 0;
+		Current->written_to = TRUE;
+		Current->to_be_deleted = TRUE;
+		Current = NULL;
+		release_path (pp);
+		return;
+	}
+	
+	for (p = Base; p; p = p->link){
+		if (strcmp (pp->file, p->filename) != 0)
+			continue;
+		
+		free_sections (p->section);
+		p->section = 0;
+		p->written_to = TRUE;
+		p->to_be_deleted = TRUE;
+		release_path (pp);
+		return;
+	}
+	release_path (pp);
+}
+
+/**
+ * gnome_config_drop_file: 
+ * @path: A gnome-config path
+ *
+ * Releases any memory resources that were allocated from accessing
+ * the configuration file in @path.  Changes will take place after
+ * gnome_config_sync has been invoked
+ */
+/**
+ * gnome_config_private_drop_file:
+ * @path: A gnome-config path
+ *
+ * Releases any memory resources that were allocated from accessing the
+ * private configuration file in @path.
+ */
+void 
+_gnome_config_drop_file (const char *path, gint priv)
+{
+	TProfile *p;
+	ParsedPath *pp;
+	char *fake_path;
+	
+	if (!path)
+		return;
+
+	fake_path = g_copy_strings (path, "/section/key", NULL);
+	pp = parse_path (fake_path, priv);
+	g_free (fake_path);
+
+	/*this is the current so don't look no further*/
+	if (Current && strcmp (pp->file, Current->filename) == 0) {
+		free_sections (Current->section);
+		Current->section = 0;
+		Current->filename [0] = 0;
+		Current->written_to = TRUE;
+		Current->to_be_deleted = FALSE;
+		Current = NULL;
+		release_path (pp);
+		return;
+	}
 	
 	for (p = Base; p; p = p->link){
 		if (strcmp (pp->file, p->filename) != 0)
@@ -662,12 +743,35 @@ _gnome_config_clean_file (const char *path, gint priv)
 		p->section = 0;
 		p->filename [0] = 0;
 		p->written_to = TRUE;
+		p->to_be_deleted = FALSE;
 		release_path (pp);
 		return;
 	}
 	release_path (pp);
 }
 
+/**
+ * gnome_config_init_iterator:
+ * @path: A gnome configuration path for a section.
+ *
+ * Creates an iterator handle that can be used to
+ * iterate over the keys in a section in a gnome configuration
+ * file.  @path must refer to a section.  The returned value
+ * can be used as an iterator for gnome_config_iterator_next().
+ *
+ * Returns the iterator handle.
+ */
+/**
+ * gnome_config_private_init_iterator:
+ * @path: A gnome configuration path for a section.
+ *
+ * Creates an iterator handle that can be used to
+ * iterate over the keys in a section in a private gnome configuration
+ * file.  @path must refer to a section.  The returned value
+ * can be used as an iterator for gnome_config_iterator_next().
+ *
+ * Returns the iterator handle.
+ */
 void *
 _gnome_config_init_iterator (const char *path, gint priv)
 {
@@ -692,6 +796,7 @@ _gnome_config_init_iterator (const char *path, gint priv)
 		New->section = load (pp->file);
 		New->mtime = st.st_mtime;
 		New->written_to = FALSE;
+		New->to_be_deleted = FALSE;
 		Base = New;
 		section = New->section;
 		Current = New;
@@ -709,6 +814,29 @@ _gnome_config_init_iterator (const char *path, gint priv)
 	return 0;
 }
 
+
+/**
+ * gnome_config_init_iterator_sections:
+ * @path: A gnome configuration path for a file.
+ *
+ * Creates an iterator handle that can be used to iterate over the
+ * sections in a gnome configuration file.  @path must refer to a
+ * gnome configuration file.  The returned value can be used as an
+ * iterator for gnome_config_iterator_next().
+ *
+ * Returns the iterator handle.
+ */
+/**
+ * gnome_config_private_init_iterator_sections:
+ * @path: A gnome configuration path for a file
+ *
+ * Creates an iterator handle that can be used to iterate over the
+ * sections in a private gnome configuration file.  @path must refer to a
+ * gnome configuration file.  The returned value can be used as an
+ * iterator for gnome_config_iterator_next().
+ *
+ * Returns the iterator handle.
+ */
 void *
 _gnome_config_init_iterator_sections (const char *path, gint priv)
 {
@@ -733,6 +861,7 @@ _gnome_config_init_iterator_sections (const char *path, gint priv)
 		New->section = load (pp->file);
 		New->mtime = st.st_mtime;
 		New->written_to = FALSE;
+		New->to_be_deleted = FALSE;
 		Base = New;
 		section = New->section;
 		Current = New;
@@ -744,10 +873,35 @@ _gnome_config_init_iterator_sections (const char *path, gint priv)
 	return iter;
 }
 
+/**
+ * gnome_config_iterator_next:
+ * @iterator_handle: A gnome configu iterator handle, returned from any
+ *                   iteration start routine or this routine.
+ * @key:   Address where the key gets stored.
+ * @value: Address where the value gets stored.
+ *
+ *
+ * Returns a new iterator handle.
+ *
+ * If @key is non-NULL, then @key will point to a g_malloc()ed region that
+ * holds the key.
+ *
+ * If @value is non-NULL, then @value will point to a g_malloc()ed region that
+ * holds the key.
+ *
+ */
 void *
-gnome_config_iterator_next (void *s, char **key, char **value)
+gnome_config_iterator_next (void *iterator_handle, char **key, char **value)
 {
-	iterator_type *iter = s;
+	iterator_type *iter = iterator_handle;
+
+        /*
+	 * g_return_if_fail is not appropriate since this is not
+	 * really a failure, but passing in an "empty" iterator (we
+	 * return NULL at times)
+	 */
+	if(!iterator_handle)
+		return NULL; 
 	
 	if (iter->type == 0){
 		TKeys *keys;
@@ -778,6 +932,22 @@ gnome_config_iterator_next (void *s, char **key, char **value)
 	}
 }
 
+/**
+ * gnome_config_clean_section:
+ * @path: a gnome configuration path to a section.
+ *
+ * Cleans up the section specified by @path from any
+ * configuration information.  Changes will only take place
+ * after gnome_config has been invoked.
+ */
+/**
+ * gnome_config_private_clean_section:
+ * @path: a gnome configuration path to a section.
+ *
+ * Cleans up the section specified by @path in a private file from any
+ * configuration information.  Changes will only take place after
+ * gnome_config has been invoked.
+ */
 void 
 _gnome_config_clean_section (const char *path, gint priv)
 {
@@ -800,6 +970,7 @@ _gnome_config_clean_section (const char *path, gint priv)
 		New->section = load (pp->file);
 		New->mtime = st.st_mtime;
 		New->written_to = FALSE;
+		New->to_be_deleted = FALSE;
 		Base = New;
 		section = New->section;
 		Current = New;
@@ -816,6 +987,23 @@ _gnome_config_clean_section (const char *path, gint priv)
 	release_path (pp);
 }
 
+/**
+ * gnome_config_clean_key:
+ * @path: a gnome configuration path to a key.
+ *
+ * Removes the definition for the key on a gnome configuration file.
+ *
+ * Changes will take place after gnome_config_sync has been invoked.
+ */
+/**
+ * gnome_config_private_clean_key:
+ * @path: a gnome configuration path to a key.
+ *
+ * Removes the definition for the key on a private gnome configuration
+ * file.
+ *
+ * Changes will take place after gnome_config_sync has been invoked.
+ */
 void 
 _gnome_config_clean_key (const char *path, gint priv)
 	/* *section_name, char *file */
@@ -837,6 +1025,7 @@ _gnome_config_clean_key (const char *path, gint priv)
 		New->section = load (pp->file);
 		New->mtime = st.st_mtime;
 		New->written_to = FALSE;
+		New->to_be_deleted = FALSE;
 		Base = New;
 		section = New->section;
 		Current = New;
@@ -854,6 +1043,24 @@ _gnome_config_clean_key (const char *path, gint priv)
 	release_path (pp);
 }
 
+/**
+ * gnome_config_has_section:
+ * @path: A gnome configuration path to a section
+ *
+ * Queries the gnome configuration file for the presence
+ * of the section specified in @path.
+ *
+ * Returns TRUE if the section exists.  FALSE otherwise.
+ */
+/**
+ * gnome_config_private_has_section:
+ * @path: A gnome configuration path to a section
+ *
+ * Queries the private gnome configuration file for the presence
+ * of the section specified in @path.
+ *
+ * Returns TRUE if the section exists.  FALSE otherwise.
+ */
 gboolean 
 _gnome_config_has_section (const char *path, gint priv)
 	/* char *section_name, char *profile */
@@ -877,6 +1084,7 @@ _gnome_config_has_section (const char *path, gint priv)
 		New->section = load (pp->file);
 		New->mtime = st.st_mtime;
 		New->written_to = FALSE;
+		New->to_be_deleted = FALSE;
 		Base = New;
 		section = New->section;
 		Current = New;
@@ -891,6 +1099,13 @@ _gnome_config_has_section (const char *path, gint priv)
 	return 0;
 }
 
+/**
+ * gnome_config_drop_all:
+ *
+ * Drops any information cached in memory that was fetched with
+ * gnome config.  Any pending information that has not been
+ * written to disk is discarded.
+ */
 void 
 gnome_config_drop_all (void)
 {
@@ -947,7 +1162,7 @@ _gnome_config_get_translated_string_with_default (const char *path,
 
 	char *value= NULL;
 
-	language_list= gnome_i18n_get_language_list ("LC_ALL");
+	language_list = gnome_i18n_get_language_list ("LC_ALL");
 
 	while (!value && language_list) {
 		const char *lang= language_list->data;
@@ -974,7 +1189,7 @@ _gnome_config_get_translated_string_with_default (const char *path,
 				   we want to try `pt' as a backup.  */
 				n = strcspn (lang, "@_");
 				if (lang[n]) {
-					char *copy = strndup (lang, n);
+					char *copy = g_strndup (lang, n);
 					tkey = g_copy_strings (path, "[",
 							       copy, "]",
 							       NULL);
@@ -988,7 +1203,7 @@ _gnome_config_get_translated_string_with_default (const char *path,
 				}
 			}
 		}
-		language_list= language_list->next;
+		language_list = language_list->next;
 	}
 
 	return value;
@@ -1024,8 +1239,10 @@ _gnome_config_get_bool_with_default (const char *path, gboolean *def,
 			   def);
 
 	/* It isn't an error if the key is not found.  */
-	if (r == NULL)
+	if (r == NULL){
+		release_path (pp);
 		return 0;
+	}
 
 	if (!strcasecmp (r, "true")){
 		v = 1;
@@ -1083,7 +1300,7 @@ gnome_config_make_vector (const char *rr, int *argcp, char ***argvp)
 			p++;
 		}
 
- 		(*argvp)[count++] = (char *) strndup (tmp, p - tmp);
+ 		(*argvp)[count++] = (char *) g_strndup (tmp, p - tmp);
 
 		while (*p && *p == ' ')
 			p++;
@@ -1108,6 +1325,15 @@ _gnome_config_get_vector_with_default (const char *path, int *argcp,
 	release_path (pp);
 }
 
+/**
+ * gnome_config_set_translated_string:
+ * @path: a gnome configuration path to a key
+ * @value: a string value to set.
+ *
+ * Stores the string value @new_value in the file/section/key defined
+ * by the @path on the proper section for the current language set by
+ * by the user.
+ */
 void
 _gnome_config_set_translated_string (const char *path, const char *value,
 				     gint priv)
@@ -1116,7 +1342,7 @@ _gnome_config_set_translated_string (const char *path, const char *value,
 	const char *lang;
 	char *tkey;
 
-	language_list= gnome_i18n_get_language_list("LC_ALL");
+	language_list = gnome_i18n_get_language_list("LC_ALL");
 
 	lang= language_list ? language_list->data : NULL;
 
@@ -1129,6 +1355,14 @@ _gnome_config_set_translated_string (const char *path, const char *value,
 	CALL_SET_HANDLER();
 }
 
+/**
+ * gnome_config_set_string:
+ * @path: a gnome configuration path to a key
+ * @new_value: a string value to set.
+ *
+ * Stores the string value @new_value in the file/section/key
+ * defined by the @path
+ */
 void
 _gnome_config_set_string (const char *path, const char *new_value, gint priv)
 {
@@ -1142,6 +1376,14 @@ _gnome_config_set_string (const char *path, const char *new_value, gint priv)
 	CALL_SET_HANDLER();
 }
 
+/**
+ * gnome_config_set_int:
+ * @path: a gnome configuration path to a key
+ * @new_value: a int value to set.
+ *
+ * Stores the integer value @new_value in the file/section/key
+ * defined by the @path
+ */
 void
 _gnome_config_set_int (const char *path, int new_value, gint priv)
 {
@@ -1150,13 +1392,21 @@ _gnome_config_set_int (const char *path, int new_value, gint priv)
 	const char *r;
 	
 	pp = parse_path (path, priv);
-	sprintf (intbuf, "%d", new_value);
+	g_snprintf (intbuf, sizeof(intbuf), "%d", new_value);
 	r = access_config (SET, pp->section, pp->key, intbuf, pp->file,
 			   NULL);
 	release_path (pp);
 	CALL_SET_HANDLER();
 }
 
+/**
+ * gnome_config_set_float:
+ * @path: a gnome configuration path to a key
+ * @new_value: a double value to set.
+ *
+ * Stores the double value @new_value in the file/section/key
+ * defined by the @path
+ */
 void
 _gnome_config_set_float (const char *path, gdouble new_value, gint priv)
 {
@@ -1165,13 +1415,21 @@ _gnome_config_set_float (const char *path, gdouble new_value, gint priv)
 	const char *r;
 	
 	pp = parse_path (path, priv);
-	sprintf (floatbuf, "%.17g", new_value);
+	g_snprintf (floatbuf, sizeof(floatbuf), "%.17g", new_value);
 	r = access_config (SET, pp->section, pp->key, floatbuf, pp->file,
 			   NULL);
 	release_path (pp);
 	CALL_SET_HANDLER();
 }
 
+/**
+ * gnome_config_set_bool:
+ * @path: a gnome configuration path to a key
+ * @new_value: a boolean value to set
+ *
+ * Stores boolean value @new_value in the file/section/key defined by
+ * @path.
+ */
 void
 _gnome_config_set_bool (const char *path, gboolean new_value, gint priv)
 {
@@ -1185,6 +1443,16 @@ _gnome_config_set_bool (const char *path, gboolean new_value, gint priv)
 	CALL_SET_HANDLER();
 }
 
+/**
+ * gnome_config_assemble_vector:
+ * @argc: Number of elements in the @argv string array.
+ * @argv: An array of strings.
+ *
+ * This routine returns the the strings in the array contactenated by
+ * spaces.
+ *
+ * Returns: a g_malloc()ed string with the concatenation results.
+ */
 char *
 gnome_config_assemble_vector (int argc, const char *const argv [])
 {
@@ -1192,16 +1460,17 @@ gnome_config_assemble_vector (int argc, const char *const argv [])
 	const char *s;
 	int i, len;
 
-	/* Compute length of quoted string.  We cheat and just use
-	   twice the sum of the lengths of all the strings.  Sigh.  */
+	/*
+	 * Compute length of quoted string.  We cheat and just use
+	 * twice the sum of the lengths of all the strings.  
+	 */
 	len = 1;
-	for (i = 0; i < argc; ++i) {
-		len += 2 * strlen (argv[i]) + 1;
-	}
+	for (i = 0; i < argc; ++i)
+		len += 2 * strlen (argv [i]) + 1 + argc;
 
 	p = value = g_malloc (len);
 	for (i = 0; i < argc; ++i) {
-		for (s = argv[i]; *s; ++s) {
+		for (s = argv [i]; *s; ++s) {
 			if (*s == ' ' || *s == '\\')
 				*p++ = '\\';
 			*p++ = *s;
@@ -1229,12 +1498,30 @@ _gnome_config_set_vector (const char *path, int argc,
 	CALL_SET_HANDLER();
 }
 
+/**
+ * gnome_config_push_prefix:
+ * @path: a gnome configuration path prefix
+ *
+ * @path is a prefix that will be prepended automatically to any
+ * non-absolute configuration path in gnome config.
+ *
+ * This is used to simplify application loading code.
+ *
+ * Library code will usually have to set the prefix before doing
+ * any gnome-configuration access, since the application might
+ * be using their own prefix. 
+ */
 void
 gnome_config_push_prefix (const char *path)
 {
 	prefix_list = g_slist_prepend(prefix_list, g_strdup(path));
 }
 
+/**
+ * gnome_config_pop_prefix:
+ *
+ * Call this routine to stop using the configuration prefix
+ */
 void
 gnome_config_pop_prefix (void)
 {
