@@ -28,6 +28,11 @@
 #include <unistd.h>
 #include <dirent.h>
 #include <ctype.h>
+#include <errno.h>
+
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
 
 #ifdef HAVE_DB_185_H
 # include <db_185.h>
@@ -37,6 +42,7 @@
 
 #include "libgnomeP.h"
 #include "gnome-mime.h"
+#include "gnome-regex.h"
 #ifdef NEED_GNOMESUPPORT_H
 #include "gnomesupport.h"
 #endif
@@ -149,6 +155,7 @@ static void
 unlock (void)
 {
 	if (! --lock_count) {
+		database->sync (database, 0);
 		rmdir (lock_directory);
 	}
 }
@@ -652,8 +659,9 @@ maybe_scan_app_dir (void)
 	app_type_hash = g_hash_table_new (g_str_hash, g_str_equal);
 
 	if (scandir (gnome_metadata_app_dir, &list,
-		     scan_app_file, alphasort) != -1)
-		free (list);
+		     (gpointer)scan_app_file, alphasort) != -1)
+		if  (list)
+			free (list);
 }
 
 /* This is set if we've already found a suitable regex match.  It lets
@@ -721,8 +729,11 @@ try_app_regexs (const char *file, const char *key, int *size, char **buffer)
 	short_circuit = NULL;
 	desired_key = (char *) key;
 	type_desired = ! strcmp (key, "type");
-	g_hash_table_foreach (app_rx_hash, try_one_app_regex,
-			      (gpointer) file);
+
+	if (app_rx_hash)
+		g_hash_table_foreach (app_rx_hash, try_one_app_regex,
+				      (gpointer) file);
+
 	if (! short_circuit)
 		return GNOME_METADATA_NOT_FOUND;
 
@@ -759,8 +770,17 @@ app_get_by_type (const char *type, const char *key, int *size, char **buffer)
 
 
 
-/* Set metadata associated with FILE.  Returns 0 on success, or an
-   error code.  */
+/**
+ * gnome_metadata_set:
+ * @file: File with which metadata will be associated
+ * @name: Metadata key.
+ * @size: Size in bytes of data
+ * @data: Data to be stored.
+ *
+ * Sets metadata associated with @file and @name.
+ *
+ * Returns %0 on success or an error code.
+ */
 int
 gnome_metadata_set (const char *file, const char *name,
 		    int size, const char *data)
@@ -768,19 +788,31 @@ gnome_metadata_set (const char *file, const char *name,
 	return metadata_set ("file", file, name, size, data);
 }
 
-/* Remove a piece of metadata associated with FILE.  Returns 0 on
-   success, or an error code.  */
+/**
+ * gnome_metadata_remove:
+ * @file: File name
+ * @name: Metadata key.
+ *
+ * Remove a piece of metadata associated with @file.
+ *
+ * Returns %0 on success, or an error code.
+ */
 int
 gnome_metadata_remove (const char *file, const char *name)
 {
 	return metadata_remove ("file", file, name);
 }
 
-/* Return an array of all metadata keys associated with FILE.  The
-   array is NULL terminated.  The result value can be freed with
-   gnome_string_array_free.  This only returns keys for which there is
-   a particular association with FILE.  It will not return keys for
-   which a regex or other match succeeds.  */
+/**
+ * gnome_metadata_list
+ * @file: File name.
+ *
+ * Returns an array of all metadata keys associated with @file.  The
+ * array is %NULL terminated.  The result can be freed with
+ * g_strfreev().  This only returns keys for which there
+ * is a particular association with @file.  It will not return keys
+ * for which a regex or other match succeeds.
+ */
 char **
 gnome_metadata_list (const char *file)
 {
@@ -847,8 +879,15 @@ try_regexs (const char *file, const char *key, int *size, char **buffer)
 static char *
 run_file (const char *file)
 {
-	/* FIXME: implement.  */
+#if 0
+  /* This is what the code will look like once the magic functions are
+     ready.  */
+	extern const char *gnome_mime_type_from_magic (const char *); /* FIXME */
+	const char *type = gnome_mime_type_from_magic (file);
+	return type ? strdup (type) : NULL;
+#else
 	return NULL;
+#endif
 }
 
 /* Do all the work for _get and _get_fast.  */
@@ -857,6 +896,7 @@ get_worker (const char *file, const char *name, int *size, char **buffer,
 	    int is_fast)
 {
 	int type_size, r;
+	int is_type = 0;
 	char *type;
 
 	/* Phase 1: see if data exists in database.  */
@@ -875,12 +915,16 @@ get_worker (const char *file, const char *name, int *size, char **buffer,
 
 	if (! strcmp (name, "type")) {
 		/* We're trying to fetch the type, so there's no point
-		   trying the same requests again.  */
-		if (is_fast)
-			return GNOME_METADATA_NOT_FOUND;
-		type = run_file (file);
-		if (! type)
-			return GNOME_METADATA_NOT_FOUND;
+		   trying the same requests again below.  */
+
+		type = gnome_mime_type_or_default ((char *) file, NULL);
+		if (! type) {
+			if (is_fast)
+				return GNOME_METADATA_NOT_FOUND;
+			type = run_file (file);
+			if (! type)
+				return GNOME_METADATA_NOT_FOUND;
+		}
 		*size = strlen (type) + 1;
 		*buffer = type;
 		return 0;
@@ -897,15 +941,15 @@ get_worker (const char *file, const char *name, int *size, char **buffer,
 		goto got_type;
 	}
 
-	/* See if `mime.types' has the answer.  */
-	type = gnome_mime_type_or_default (file, NULL);
-	if (type) {
-		type = strdup (type);
+	/* Try application-installed information.  */
+	if (! try_app_regexs (file, "type", &type_size, &type)) {
 		goto got_type;
 	}
 
-	/* Try application-installed information.  */
-	if (! try_app_regexs (file, "type", &type_size, &type)) {
+	/* See if `mime.types' has the answer.  */
+	type = gnome_mime_type_or_default ((char *) file, NULL);
+	if (type) {
+		type = strdup (type);
 		goto got_type;
 	}
 
@@ -928,9 +972,18 @@ got_type:
 	return r;
 }
 
-/* Get a piece of metadata associated with FILE.  SIZE and BUFFER are
-   result parameters.  *BUFFER is malloc()d.  Returns 0, or an error
-   code.  On error *BUFFER will be set to NULL.  */
+/**
+ * gnome_metadata_get:
+ * @file: File name
+ * @name: Metadata key
+ * @size: Return parameter for size of data
+ * @data: Return parameter for data
+ *
+ * Get a piece of metadata associated with @file.  @size and @buffer
+ * are result parameters.  *@buffer is malloc()d.
+ *
+ * Returns %0, or an error code.  On error *@buffer will be set to %NULL.
+ */
 int
 gnome_metadata_get (const char *file, const char *name,
 		    int *size, char **buffer)
@@ -942,8 +995,19 @@ gnome_metadata_get (const char *file, const char *name,
 	return r;
 }
 
-/* Like gnome_metadata_get, but won't run the `file' command to
-   characterize the file type.  Returns 0, or an error code.  */
+/**
+ * gnome_metadata_get_fast:
+ * @file: File name
+ * @name: Metadata key
+ * @size: Return parameter for size of data
+ * @data: Return parameter for data
+ *
+ * Like gnome_metadata_get(), but won't run the `file' command to
+ * characterize the file type.
+ * 
+ * Returns %0, or an error code.  On error *@buffer will be set to
+ * %NULL.
+ */
 int
 gnome_metadata_get_fast (const char *file, const char *name,
 			 int *size, char **buffer)
@@ -1004,29 +1068,52 @@ worker (const char *from, const char *to, int op)
 
 	unlock ();
 
-	gnome_string_array_free (keys);
+	g_strfreev (keys);
 
 	return ret;
 }
 
-/* Convenience function.  Call this when a file is renamed.  Returns 0
-   on success, or an error code.  */
+/**
+ * gnome_metadata_rename:
+ * @from: Source file name
+ * @to: Destination file name
+ *
+ * This function moves metadata associated with file @from to file
+ * @to.  It should be called after a file is renamed.
+ *
+ * Returns %0 on success, or an error code.
+ */
 int
 gnome_metadata_rename (const char *from, const char *to)
 {
 	return worker (from, to, RENAME);
 }
 
-/* Convenience function.  Call this when a file is copied.  Returns 0
-   on success, or an error code.  */
+/**
+ * gnome_metadata_copy:
+ * @from: Source file name
+ * @to: Destination file name
+ *
+ * This function copies metadata associated with file @from to file
+ * @to.  It should be called after a file is copied.
+ *
+ * Returns %0 on success, or an error code.
+ */
 int
 gnome_metadata_copy (const char *from, const char *to)
 {
 	return worker (from, to, COPY);
 }
 
-/* Convenience function.  Call this when a file is deleted.  Returns 0
-   on success, or an error code.  */
+/**
+ * gnome_metadata_copy:
+ * @file: File name
+ *
+ * This function deletes all metadata associated with @file.
+ * It should be called after a file is deleted.
+ *
+ * Returns %0 on success, or an error code.
+ */
 int
 gnome_metadata_delete (const char *file)
 {
@@ -1034,8 +1121,18 @@ gnome_metadata_delete (const char *file)
 }
 
 
-/* Add a regular expression to the internal list.  This regex is used
-   when matching requests for the metadata KEY.  */
+/**
+ * gnome_metadata_regex_add:
+ * @regex: The regular expression.
+ * @key: The metadata key.
+ * @size: Size of data in bytes.
+ * @data: The data.
+ *
+ * Add a regular expression to the internal list.  This regex is used
+ * when matching requests for the metadata @key.
+ *
+ * Returns nothing.
+ */
 void
 gnome_metadata_regex_add (const char *regex, const char *key,
 			   int size, const char *data)
@@ -1043,14 +1140,33 @@ gnome_metadata_regex_add (const char *regex, const char *key,
 	metadata_set ("regex", regex, key, size, data);
 }
 
-/* Remove a regular expression from the internal list.  */
+/**
+ * gnome_metadata_regex_remove:
+ * @regex: The regular expression.
+ * @key: The metadata key.
+ *
+ * Remove the regular expression from the internal list.
+ *
+ * Returns nothing.
+ */
 void
 gnome_metadata_regex_remove (const char *regex, const char *key)
 {
 	metadata_remove ("regex", regex, key);
 }
 
-/* Add a file type to the internal list.  */
+/**
+ * gnome_metadata_type_add:
+ * @type: File type
+ * @key: The metadata key.
+ * @size: Size of data in bytes.
+ * @data: The data.
+ *
+ * Add a file type to the internal list.  This pairing is used
+ * when matching requests for the metadata @key.
+ *
+ * Returns nothing.
+ */
 void
 gnome_metadata_type_add (const char *type, const char *key,
 			 int size, const char *data)
@@ -1058,7 +1174,15 @@ gnome_metadata_type_add (const char *type, const char *key,
 	metadata_set ("type", type, key, size, data);
 }
 
-/* Remove a file type from the internal list.  */
+/**
+ * gnome_metadata_type_remove:
+ * @type: The file type.
+ * @key: The metadata key.
+ *
+ * Remove a type/key pairing from the internal list.
+ *
+ * Returns nothing.
+ */
 void
 gnome_metadata_type_remove (const char *type, const char *key)
 {
