@@ -17,13 +17,22 @@ char *gnome_libdir_file (const char *filename);
 char *gnome_datadir_file (const char *filename);
 char *gnome_sound_file (const char *filename); 
 char *gnome_pixmap_file (const char *filename); 
+char *gnome_config_file (const char *filename);
 char *gnome_unconditional_libdir_file (const char *filename); 
 char *gnome_unconditional_datadir_file (const char *filename); 
 char *gnome_unconditional_sound_file (const char *filename); 
 char *gnome_unconditional_pixmap_file (const char *filename); 
+char *gnome_unconditional_config_file (const char *filename);
 
-/* TRUE if the file exists */
-int g_file_exists (const char *filename); 
+enum {
+	G_FILE_TEST_EXISTS=(1<<0)|(1<<1)|(1<<2), /*any type of file*/
+	G_FILE_TEST_ISFILE=1<<0,
+	G_FILE_TEST_ISLINK=1<<1,
+	G_FILE_TEST_ISDIR=1<<2
+};
+
+int g_file_test   (const char *filename, int test);
+int g_file_exists (const char *filename);
 
 /* Concatenates any number of strings; last arg should be NULL */
 char *g_copy_strings (const char *first, ...); 
@@ -48,8 +57,7 @@ const char * g_extension_pointer (const char * path);
 /* vec has to be NULL-terminated */
 char ** g_copy_vector    (char ** vec);
 /* separator can be NULL */
-char  * g_flatten_vector (const char * separator, 
-			  char ** vec);
+#define g_flatten_vector g_strjoinv
 
 /* returns the home directory of the user
  * This one is NOT to be free'd as it points into the 
