@@ -13,8 +13,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Cambridge, MA 02139, USA. 
  */
 
 #include <config.h>
@@ -67,11 +67,11 @@ static void gnome_about_display_comments (GdkWindow *win,
 					  const gchar *comments);
 
 
-/* ----------------------------------------------------------------------
-   NAME:	gnome_about_get_type
-   DESCRIPTION:	
-   ---------------------------------------------------------------------- */
-
+/**
+ * gnome_about_get_type:
+ *
+ * Returns the GtkType for the GnomeAbout widget.
+ */
 guint
 gnome_about_get_type ()
 {
@@ -96,32 +96,17 @@ gnome_about_get_type ()
 	return about_type;
 }
 
-/* ----------------------------------------------------------------------
-   NAME:	gnome_about_class_init
-   DESCRIPTION:	
-   ---------------------------------------------------------------------- */
-
 static void
 gnome_about_class_init (GnomeAboutClass *klass)
 {
 }
 
-/* ----------------------------------------------------------------------
-   NAME:	gnome_about_init
-   DESCRIPTION:	
-   ---------------------------------------------------------------------- */
-
 static void
 gnome_about_init (GnomeAbout *about)
 {
 }
 
-/* ----------------------------------------------------------------------
-   NAME:	gnome_about_repaint
-   DESCRIPTION:	
-   ---------------------------------------------------------------------- */
-
-void
+static void
 gnome_about_repaint (GtkWidget *widget, 
 		     GdkEventExpose *event,
 		     GnomeAboutInfo *gai)
@@ -227,7 +212,7 @@ gnome_about_repaint (GtkWidget *widget,
 
 }
 
-void
+static void
 gnome_about_display_comments (GdkWindow *win, 
 			      GdkFont *font,
 			      GdkGC *gc, 
@@ -324,7 +309,7 @@ gnome_about_display_comments (GdkWindow *win,
    DESCRIPTION:	Calculates size of window WITHOUT counting pixmap. 
    ---------------------------------------------------------------------- */
 
-void
+static void
 gnome_about_calc_size (GnomeAboutInfo *gai)
 {
 	GList *name;
@@ -417,7 +402,7 @@ gnome_fill_info (GtkWidget *widget,
 		 const gchar   *logo)
 {
 	GnomeAboutInfo *gai;
-	GdkColor light_green = {0, 51914, 64764, 44718};
+	static GdkColor light_green = {0, 30208, 41216, 33792};
 	GtkStyle *style;
 
 	/* alloc mem for struct */
@@ -490,7 +475,7 @@ gnome_fill_info (GtkWidget *widget,
    DESCRIPTION:	
    ---------------------------------------------------------------------- */
 
-void
+static void
 gnome_destroy_about (GtkWidget *widget, gpointer *data)
 {
 	GnomeAboutInfo *gai;
@@ -515,26 +500,24 @@ gnome_destroy_about (GtkWidget *widget, gpointer *data)
 
 }
 
-/* ----------------------------------------------------------------------
-   NAME:	gnome_about_new
-   DESCRIPTION:	Main routine that creates the widget. The arguments are:
-                  title      ... Name of app.
-		  version    ... version number
-		  copyright  ... copyright string
-		  authors    ... NULL terminated list of authors
-		  comments   ... Other comments
-		  logo       ... a logo bitmap file.
-
-		a typical call would be:
-
-   gchar *authors[] = {"author1", "author2", ..., NULL};
-
-   GtkWdiget about = gnome_about_new ("GnoApp", "1.2b",
-                           "Copyright (C) 1998",
-			   authors,
-			   "Comment line 1\nComment line 2",
-			   "/usr/local/lib/pixmaps/app_icon.xpm");
-   ---------------------------------------------------------------------- */
+/**
+ * gnome_about_new
+ * @title: Name of app.
+ * @version: version number
+ * @copyright: copyright string
+ * @authors: %NULL terminated list of authors
+ * @comments: Other comments
+ * @logo: a logo pixmap file.
+ *
+ * Description: Creates a new GNOME About dialog.  @title, @version,
+ * @copyright, and @authors are displayed first, in that order.
+ * @comments is typically the location for multiple lines of text, if
+ * necessary.  (Separate with "\n".)  @logo is the filename of a
+ * optional pixmap to be displayed in the dialog, typically a product or
+ * company logo of some sort; set to %NULL if no logo file is available.
+ *
+ * Returns: &GtkWidget pointer to new dialog
+ **/
 
 GtkWidget* 
 gnome_about_new (const gchar	*title,
@@ -546,6 +529,12 @@ gnome_about_new (const gchar	*title,
 {
 	GnomeAbout *about;
 
+	g_return_val_if_fail (title != NULL, NULL);
+	g_return_val_if_fail (version != NULL, NULL);
+	g_return_val_if_fail (copyright != NULL, NULL);
+	g_return_val_if_fail (authors != NULL, NULL);
+	g_return_val_if_fail (comments != NULL, NULL);
+	
 	about = gtk_type_new (gnome_about_get_type ());
 
 	gnome_about_construct(about, title, version, copyright,
@@ -581,9 +570,9 @@ gnome_about_construct (GnomeAbout *about,
 
 	frame = gtk_frame_new (NULL);
 	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
-	gtk_container_border_width (GTK_CONTAINER (frame), 4);
+	gtk_container_set_border_width (GTK_CONTAINER (frame), 4);
 	gtk_container_add (GTK_CONTAINER (GNOME_DIALOG(about)->vbox),
-			   frame);
+			   GTK_WIDGET(frame));
 	gtk_widget_show (frame);
 
 	drawing_area = gtk_drawing_area_new ();
