@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * GnomeApp widget (C) 1998 Red Hat Software, The Free Software Foundation,
  * Miguel de Icaza, Federico Menu, Chris Toshok.
@@ -18,34 +19,12 @@
 #include <gtk/gtkwindow.h>
 #include <libgnome/gnome-defs.h>
 
+#include "gnome-dock.h"
 
 BEGIN_GNOME_DECLS
 
-
-/* These define the possible positions for the menu bar and toolbars of the application window.
- * Note that a menu bar cannot be on the LEFT or RIGHT.
- */
-typedef enum 
-{
-	GNOME_APP_POS_TOP,
-	GNOME_APP_POS_BOTTOM,
-	GNOME_APP_POS_LEFT,
-	GNOME_APP_POS_RIGHT,
-	GNOME_APP_POS_FLOATING
-} GnomeAppWidgetPositionType;
-
-/* Everything gets put into a table that looks like:
- *
- * XXX
- * ABC
- * YYY
- *
- * There's one table element on top, three in the middle, and one on
- * the bottom.
- *
- * Obviously you can change the positions of things as needed
- * using the supplied function.
- */
+#define GNOME_APP_MENUBAR_NAME "Menubar"
+#define GNOME_APP_TOOLBAR_NAME "Toolbar"
 
 #define GNOME_APP(obj)         GTK_CHECK_CAST(obj, gnome_app_get_type(), GnomeApp)
 #define GNOME_APP_CLASS(class) GTK_CHECK_CAST_CLASS(class, gnome_app_get_type(), GnomeAppClass)
@@ -57,19 +36,37 @@ typedef struct _GnomeAppClass  GnomeAppClass;
 struct _GnomeApp {
 	GtkWindow parent_object;
 
-	char *name;			/* Application name */
-	char *prefix;			/* Prefix for gnome-config */
+	/* Application name. */
+	char *name;
 
-	GtkWidget *menubar;		/* The Menubar */
-	GtkWidget *toolbar;		/* The Toolbar */
-        GtkWidget *statusbar;		/* The Statusbar */
-	GtkWidget *contents;		/* The contents */
-	GtkWidget *table;		/* The table widget that ties them all */
+	/* Prefix for gnome-config (used to save the layout).  */
+	char *prefix;
 
-	GtkAccelGroup *accel_group;	/* Main accelerator group for this window (hotkeys live here)*/
+        /* The dock.  */
+        GtkWidget *dock;
 
-	/* Positions for the menubar and the toolbar */
-	GnomeAppWidgetPositionType pos_menubar, pos_toolbar;
+	/* The status bar.  */
+        GtkWidget *statusbar;
+
+	/* The vbox widget that ties them.  */
+	GtkWidget *vbox;
+
+	/* The menubar.  This is a pointer to a widget contained into
+           the dock.  */
+	GtkWidget *menubar;
+
+	/* The contents.  This is a pointer to dock->client_area.  */
+	GtkWidget *contents;
+
+	/* Dock layout.  */
+	GnomeDockLayout *layout;
+
+	/* Main accelerator group for this window (hotkeys live here).  */
+	GtkAccelGroup *accel_group;
+
+	/* If TRUE, the application uses gnome-config to retrieve and
+           save the docking configuration automagically.  */
+	gboolean enable_layout_config : 1;
 };
 
 struct _GnomeAppClass {
@@ -101,12 +98,37 @@ void gnome_app_set_statusbar (GnomeApp *app, GtkWidget *statusbar);
 /* Sets the content area of the application window */
 void gnome_app_set_contents (GnomeApp *app, GtkWidget *contents);
 
-/* Sets the position of the toolbar */
-void gnome_app_toolbar_set_position (GnomeApp *app, GnomeAppWidgetPositionType pos_toolbar);
+void gnome_app_add_toolbar (GnomeApp *app,
+			    GtkToolbar *toolbar,
+			    const gchar *name,
+			    GnomeDockItemBehavior behavior,
+			    GnomeDockPlacement placement,
+			    gint band_num,
+			    gint band_position,
+			    gint offset);
 
-/* Sets the position of the menu bar */
-void gnome_app_menu_set_position (GnomeApp *app, GnomeAppWidgetPositionType pos_menu);
+void gnome_app_add_docked (GnomeApp *app,
+			   GtkWidget *widget,
+			   const gchar *name,
+			   GnomeDockItemBehavior behavior,
+			   GnomeDockPlacement placement,
+			   gint band_num,
+			   gint band_position,
+			   gint offset);
 
+void gnome_app_add_dock_item (GnomeApp *app,
+			      GnomeDockItem *item,
+			      GnomeDockPlacement placement,
+			      gint band_num,
+			      gint band_position,
+			      gint offset);
+
+void gnome_app_enable_layout_config (GnomeApp *app, gboolean enable);
+
+GnomeDock *gnome_app_get_dock (GnomeApp *app);
+
+GnomeDockItem *gnome_app_get_dock_item_by_name (GnomeApp *app,
+						const gchar *name);
 
 END_GNOME_DECLS
 
