@@ -341,15 +341,16 @@ set_result(GnomeCalculator *gc)
 	gc->result = stack->d.number;
 
 	for(i=12;i>0;i--) {
-		g_snprintf(format,20,"%c .%dlg",'%',i);
-		g_snprintf(buf,80,format,gc->result);
+		g_snprintf(format, sizeof(format), "%c .%dlg", '%', i);
+		g_snprintf(buf, sizeof(buf), format, gc->result);
 		if(strlen(buf)<=12)
 			break;
 	}
 	strncpy(gc->result_string,buf,12);
 	gc->result_string[12]='\0';
 
-	put_led_font(gc);
+	if(GTK_WIDGET_REALIZED(gc))
+		put_led_font(gc);
 
 	gtk_signal_emit(GTK_OBJECT(gc),
 			gnome_calculator_signals[RESULT_CHANGED_SIGNAL],
@@ -575,13 +576,24 @@ clear_calc(GtkWidget *w, gpointer data)
 	return TRUE;
 }
 
+
+/**
+ * gnome_calculator_clear
+ * @gc: Pointer to GNOME calculator widget.
+ * @reset: %FALSE to zero, %TRUE to reset calculator completely
+ *
+ * Description:
+ * Resets the calculator back to zero.  If @reset is %TRUE, results
+ * stored in memory and the calculator mode are cleared also.
+ **/
+
 void
-gnome_calculator_clear(GnomeCalculator *gc, gint reset)
+gnome_calculator_clear(GnomeCalculator *gc, const gboolean reset)
 {
-	if(reset)
-		reset_calc(NULL,gc);
+	if (reset)
+		reset_calc(NULL, gc);
 	else
-		clear_calc(NULL,gc);
+		clear_calc(NULL, gc);
 }
 
 static gint
@@ -783,6 +795,17 @@ set_result_to(GnomeCalculator *gc, gdouble result)
 	return old;
 }
 
+
+/**
+ * gnome_calculator_set
+ * @gc: Pointer to GNOME calculator widget.
+ * @result: New value of calculator buffer.
+ *
+ * Description:
+ * Sets the value stored in the calculator's result buffer to the given
+ * @result.
+ **/
+
 void
 gnome_calculator_set(GnomeCalculator *gc, gdouble result)
 {
@@ -1044,13 +1067,12 @@ gnome_calculator_realized(GtkWidget *w, gpointer data)
 
 	gtk_widget_show(gc->display);
 
-	gc->stack = NULL;
-	reset_calc(NULL,gc);
+	put_led_font(gc);
 
 	return FALSE;
 }
 
-static CalculatorButton buttons[8][5] = {
+static const CalculatorButton buttons[8][5] = {
 	{
 		{N_("1/x"),(GtkSignalFunc)simple_func,c_inv,NULL,FALSE,0},
 		{N_("x^2"),(GtkSignalFunc)simple_func,c_pow2,sqrt,FALSE,0},
@@ -1130,12 +1152,12 @@ gnome_calculator_init (GnomeCalculator *gc)
 
 	for(x=0;x<5;x++) {
 		for(y=0;y<8;y++) {
-			CalculatorButton *but = &buttons[y][x];
+			const CalculatorButton *but = &buttons[y][x];
 			if(but->name) {
 				w=gtk_button_new_with_label(_(but->name));
 				gtk_signal_connect(GTK_OBJECT(w),"clicked",
 						   but->signal_func,
-						   but);
+						   (gpointer) but);
 
 				if(but->key) {
 				  gtk_widget_add_accelerator(w,
@@ -1176,6 +1198,18 @@ gnome_calculator_init (GnomeCalculator *gc)
 	gtk_table_attach_defaults(GTK_TABLE(table),gc->invert_button,0,1,1,2);
 }
 
+
+/**
+ * gnome_calculator_new
+ *
+ * Description:
+ * Creates a calculator widget, a window with all the common buttons and
+ * functions found on a standard pocket calculator.
+ *
+ * Returns:
+ * Pointer to newly-created calculator widget.
+ **/
+
 GtkWidget *
 gnome_calculator_new (void)
 {
@@ -1202,3 +1236,24 @@ gnome_calculator_destroy (GtkObject *object)
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
+
+/* API docs for macros in gnome-calculator.h */
+
+/**
+ * gnome_calculator_get_result
+ * @gc: Pointer to GNOME calculator widget
+ *
+ * Returns:
+ * Value currently stored in calculator buffer.
+ **/
+
+/* MACDOC
+
+gdouble
+gnome_calculator_get_result (GnomeCalculator *gc)
+{
+	I am really a macro defined in gnome-calculator.h
+}
+
+END MACDOC */
+
