@@ -59,7 +59,7 @@ struct _GnomeCanvasImage {
 	int cwidth, cheight;		/* Rendered size in pixels */
 	GdkGC *gc;			/* GC for drawing image */
 
-	int need_recalc : 1;		/* Do we need to rescale the image? */
+	unsigned int need_recalc : 1;	/* Do we need to rescale the image? */
 };
 
 struct _GnomeCanvasImageClass {
