@@ -13,6 +13,7 @@
 #include <string.h>
 #include "gnome-canvas-line.h"
 #include "gnome-canvas-util.h"
+#include "gnometypebuiltins.h"
 
 
 #define DEFAULT_SPLINE_STEPS 12		/* this is what Tk uses */
@@ -40,10 +41,12 @@ enum {
 	ARG_POINTS,
 	ARG_FILL_COLOR,
 	ARG_FILL_COLOR_GDK,
+	ARG_FILL_STIPPLE,
 	ARG_WIDTH_PIXELS,
 	ARG_WIDTH_UNITS,
 	ARG_CAP_STYLE,
 	ARG_JOIN_STYLE,
+	ARG_LINE_STYLE,
 	ARG_FIRST_ARROWHEAD,
 	ARG_LAST_ARROWHEAD,
 	ARG_SMOOTH,
@@ -64,7 +67,7 @@ static void gnome_canvas_line_get_arg    (GtkObject            *object,
 					  GtkArg               *arg,
 					  guint                 arg_id);
 
-static void   gnome_canvas_line_reconfigure (GnomeCanvasItem *item);
+static void   gnome_canvas_line_update      (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags);
 static void   gnome_canvas_line_realize     (GnomeCanvasItem *item);
 static void   gnome_canvas_line_unrealize   (GnomeCanvasItem *item);
 static void   gnome_canvas_line_draw        (GnomeCanvasItem *item, GdkDrawable *drawable,
@@ -72,6 +75,7 @@ static void   gnome_canvas_line_draw        (GnomeCanvasItem *item, GdkDrawable
 static double gnome_canvas_line_point       (GnomeCanvasItem *item, double x, double y,
 					     int cx, int cy, GnomeCanvasItem **actual_item);
 static void   gnome_canvas_line_translate   (GnomeCanvasItem *item, double dx, double dy);
+static void   gnome_canvas_line_bounds      (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2);
 
 
 static GnomeCanvasItemClass *parent_class;
@@ -111,13 +115,15 @@ gnome_canvas_line_class_init (GnomeCanvasLineClass *class)
 
 	parent_class = gtk_type_class (gnome_canvas_item_get_type ());
 
-	gtk_object_add_arg_type ("GnomeCanvasLine::points", GTK_TYPE_POINTER, GTK_ARG_READWRITE, ARG_POINTS);
+	gtk_object_add_arg_type ("GnomeCanvasLine::points", GTK_TYPE_GNOME_CANVAS_POINTS, GTK_ARG_READWRITE, ARG_POINTS);
 	gtk_object_add_arg_type ("GnomeCanvasLine::fill_color", GTK_TYPE_STRING, GTK_ARG_WRITABLE, ARG_FILL_COLOR);
 	gtk_object_add_arg_type ("GnomeCanvasLine::fill_color_gdk", GTK_TYPE_GDK_COLOR, GTK_ARG_READWRITE, ARG_FILL_COLOR_GDK);
+	gtk_object_add_arg_type ("GnomeCanvasLine::fill_stipple", GTK_TYPE_GDK_WINDOW, GTK_ARG_READWRITE, ARG_FILL_STIPPLE);
 	gtk_object_add_arg_type ("GnomeCanvasLine::width_pixels", GTK_TYPE_UINT, GTK_ARG_WRITABLE, ARG_WIDTH_PIXELS);
 	gtk_object_add_arg_type ("GnomeCanvasLine::width_units", GTK_TYPE_DOUBLE, GTK_ARG_WRITABLE, ARG_WIDTH_UNITS);
 	gtk_object_add_arg_type ("GnomeCanvasLine::cap_style", GTK_TYPE_GDK_CAP_STYLE, GTK_ARG_READWRITE, ARG_CAP_STYLE);
 	gtk_object_add_arg_type ("GnomeCanvasLine::join_style", GTK_TYPE_GDK_JOIN_STYLE, GTK_ARG_READWRITE, ARG_JOIN_STYLE);
+	gtk_object_add_arg_type ("GnomeCanvasLine::line_style", GTK_TYPE_GDK_LINE_STYLE, GTK_ARG_READWRITE, ARG_LINE_STYLE);
 	gtk_object_add_arg_type ("GnomeCanvasLine::first_arrowhead", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_FIRST_ARROWHEAD);
 	gtk_object_add_arg_type ("GnomeCanvasLine::last_arrowhead", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_LAST_ARROWHEAD);
 	gtk_object_add_arg_type ("GnomeCanvasLine::smooth", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_SMOOTH);
@@ -130,12 +136,13 @@ gnome_canvas_line_class_init (GnomeCanvasLineClass *class)
 	object_class->set_arg = gnome_canvas_line_set_arg;
 	object_class->get_arg = gnome_canvas_line_get_arg;
 
-	item_class->reconfigure = gnome_canvas_line_reconfigure;
+	item_class->update = gnome_canvas_line_update;
 	item_class->realize = gnome_canvas_line_realize;
 	item_class->unrealize = gnome_canvas_line_unrealize;
 	item_class->draw = gnome_canvas_line_draw;
 	item_class->point = gnome_canvas_line_point;
 	item_class->translate = gnome_canvas_line_translate;
+	item_class->bounds = gnome_canvas_line_bounds;
 }
 
 static void
@@ -144,7 +151,7 @@ gnome_canvas_line_init (GnomeCanvasLine *line)
 	line->width = 0.0;
 	line->cap = GDK_CAP_BUTT;
 	line->join = GDK_JOIN_MITER;
-	line->width_pixels = TRUE;
+	line->line_style = GDK_LINE_SOLID;
 	line->shape_a = 0.0;
 	line->shape_b = 0.0;
 	line->shape_c = 0.0;
@@ -170,27 +177,24 @@ gnome_canvas_line_destroy (GtkObject *object)
 	if (line->last_coords)
 		g_free (line->last_coords);
 
+	if (line->stipple)
+		gdk_bitmap_unref (line->stipple);
+
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
+/* Computes the bounding box of the line, including its arrow points.  Assumes that the number of
+ * points in the line is not zero.
+ */
 static void
-recalc_bounds (GnomeCanvasLine *line)
+get_bounds (GnomeCanvasLine *line, double *bx1, double *by1, double *bx2, double *by2)
 {
-	GnomeCanvasItem *item;
 	double *coords;
 	double x1, y1, x2, y2;
 	double width;
-	double dx, dy;
 	int i;
 
-	item = GNOME_CANVAS_ITEM (line);
-
-	if (line->num_points == 0) {
-		item->x1 = item->y1 = item->x2 = item->y2 = 0;
-		return;
-	}
-
 	/* Find bounding box of line's points */
 
 	x1 = x2 = line->coords[0];
@@ -202,7 +206,7 @@ recalc_bounds (GnomeCanvasLine *line)
 	/* Add possible over-estimate for wide lines */
 
 	if (line->width_pixels)
-		width = line->width / item->canvas->pixels_per_unit;
+		width = line->width / line->item.canvas->pixels_per_unit;
 	else
 		width = line->width;
 
@@ -239,13 +243,44 @@ recalc_bounds (GnomeCanvasLine *line)
 		for (i = 0, coords = line->last_coords; i < NUM_ARROW_POINTS; i++, coords += 2)
 			GROW_BOUNDS (x1, y1, x2, y2, coords[0], coords[1]);
 
+	/* Done */
+
+	*bx1 = x1;
+	*by1 = y1;
+	*bx2 = x2;
+	*by2 = y2;
+}
+
+static void
+recalc_bounds (GnomeCanvasLine *line)
+{
+	GnomeCanvasItem *item;
+	double x1, y1, x2, y2;
+	int cx1, cx2, cy1, cy2;
+	double dx, dy;
+
+	item = GNOME_CANVAS_ITEM (line);
+
+	if (line->num_points == 0) {
+		item->x1 = item->y1 = item->x2 = item->y2 = 0;
+		return;
+	}
+
+	/* Get bounds in world coordinates */
+
+	get_bounds (line, &x1, &y1, &x2, &y2);
+
 	/* Convert to canvas pixel coords */
 
 	dx = dy = 0.0;
 	gnome_canvas_item_i2w (item, &dx, &dy);
 
-	gnome_canvas_w2c (item->canvas, x1 + dx, y1 + dy, &item->x1, &item->y1);
-	gnome_canvas_w2c (item->canvas, x2 + dx, y2 + dy, &item->x2, &item->y2);
+	gnome_canvas_w2c (item->canvas, x1 + dx, y1 + dy, &cx1, &cy1);
+	gnome_canvas_w2c (item->canvas, x2 + dx, y2 + dy, &cx2, &cy2);
+	item->x1 = cx1;
+	item->y1 = cy1;
+	item->x2 = cx2;
+	item->y2 = cy2;
 
 	/* Some safety fudging */
 
@@ -421,16 +456,15 @@ reconfigure_arrows (GnomeCanvasLine *line)
 
 /* Convenience function to set the line's GC's foreground color */
 static void
-set_line_gcs_foreground (GnomeCanvasLine *line)
+set_line_gc_foreground (GnomeCanvasLine *line)
 {
 	GdkColor c;
 
 	if (!line->gc)
 		return;
 
-	c.pixel = line->pixel;
+	c.pixel = line->fill_pixel;
 	gdk_gc_set_foreground (line->gc, &c);
-	gdk_gc_set_foreground (line->arrow_gc, &c);
 }
 
 /* Recalculate the line's width and set it in its GC */
@@ -449,11 +483,31 @@ set_line_gc_width (GnomeCanvasLine *line)
 
 	gdk_gc_set_line_attributes (line->gc,
 				    width,
-				    GDK_LINE_SOLID,
+				    line->line_style,
 				    (line->first_arrow || line->last_arrow) ? GDK_CAP_BUTT : line->cap,
 				    line->join);
 }
 
+/* Sets the stipple pattern for the line */
+static void
+set_stipple (GnomeCanvasLine *line, GdkBitmap *stipple, int reconfigure)
+{
+	if (line->stipple && !reconfigure)
+		gdk_bitmap_unref (line->stipple);
+
+	line->stipple = stipple;
+	if (stipple && !reconfigure)
+		gdk_bitmap_ref (stipple);
+
+	if (line->gc) {
+		if (stipple) {
+			gdk_gc_set_stipple (line->gc, stipple);
+			gdk_gc_set_fill (line->gc, GDK_STIPPLED);
+		} else
+			gdk_gc_set_fill (line->gc, GDK_SOLID);
+	}
+}
+
 /* Convenience functions to recalculate the arrows and bounds of the line */
 static void
 reconfigure_arrows_and_bounds (GnomeCanvasLine *line)
@@ -511,13 +565,17 @@ gnome_canvas_line_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 
 	case ARG_FILL_COLOR:
 		gnome_canvas_get_color (item->canvas, GTK_VALUE_STRING (*arg), &color);
-		line->pixel = color.pixel;
-		set_line_gcs_foreground (line);
+		line->fill_pixel = color.pixel;
+		set_line_gc_foreground (line);
 		break;
 
 	case ARG_FILL_COLOR_GDK:
-		line->pixel = ((GdkColor *) GTK_VALUE_BOXED (*arg))->pixel;
-		set_line_gcs_foreground (line);
+		line->fill_pixel = ((GdkColor *) GTK_VALUE_BOXED (*arg))->pixel;
+		set_line_gc_foreground (line);
+		break;
+
+	case ARG_FILL_STIPPLE:
+		set_stipple (line, GTK_VALUE_BOXED (*arg), FALSE);
 		break;
 
 	case ARG_WIDTH_PIXELS:
@@ -543,6 +601,11 @@ gnome_canvas_line_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		line->join = GTK_VALUE_ENUM (*arg);
 		recalc_bounds (line);
 		break;
+	
+	case ARG_LINE_STYLE:
+		line->line_style = GTK_VALUE_ENUM (*arg);
+		set_line_gc_width (line);
+		break;
 
 	case ARG_FIRST_ARROWHEAD:
 		line->first_arrow = GTK_VALUE_BOOL (*arg);
@@ -603,11 +666,15 @@ gnome_canvas_line_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 
 	case ARG_FILL_COLOR_GDK:
 		color = g_new (GdkColor, 1);
-		color->pixel = line->pixel;
+		color->pixel = line->fill_pixel;
 		gdk_color_context_query_color (line->item.canvas->cc, color);
 		GTK_VALUE_BOXED (*arg) = color;
 		break;
 
+	case ARG_FILL_STIPPLE:
+		GTK_VALUE_BOXED (*arg) = line->stipple;
+		break;
+
 	case ARG_CAP_STYLE:
 		GTK_VALUE_ENUM (*arg) = line->cap;
 		break;
@@ -616,6 +683,10 @@ gnome_canvas_line_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		GTK_VALUE_ENUM (*arg) = line->join;
 		break;
 
+	case ARG_LINE_STYLE:
+		GTK_VALUE_ENUM (*arg) = line->line_style;
+		break;
+
 	case ARG_FIRST_ARROWHEAD:
 		GTK_VALUE_BOOL (*arg) = line->first_arrow;
 		break;
@@ -651,19 +722,18 @@ gnome_canvas_line_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 }
 
 static void
-gnome_canvas_line_reconfigure (GnomeCanvasItem *item)
+gnome_canvas_line_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags)
 {
 	GnomeCanvasLine *line;
-	GdkColor color;
-	int width;
 
 	line = GNOME_CANVAS_LINE (item);
 
-	if (parent_class->reconfigure)
-		(* parent_class->reconfigure) (item);
+	if (parent_class->update)
+		(* parent_class->update) (item, affine, clip_path, flags);
 
-	set_line_gcs_foreground (line);
+	set_line_gc_foreground (line);
 	set_line_gc_width (line);
+	set_stipple (line, line->stipple, TRUE);
 	reconfigure_arrows_and_bounds (line);
 }
 
@@ -678,9 +748,8 @@ gnome_canvas_line_realize (GnomeCanvasItem *item)
 		(* parent_class->realize) (item);
 
 	line->gc = gdk_gc_new (item->canvas->layout.bin_window);
-	line->arrow_gc = gdk_gc_new (item->canvas->layout.bin_window);
 
-	(* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->reconfigure) (item);
+	(* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->update) (item, NULL, NULL, 0);
 }
 
 static void
@@ -691,7 +760,6 @@ gnome_canvas_line_unrealize (GnomeCanvasItem *item)
 	line = GNOME_CANVAS_LINE (item);
 
 	gdk_gc_unref (line->gc);
-	gdk_gc_unref (line->arrow_gc);
 
 	if (parent_class->unrealize)
 		(* parent_class->unrealize) (item);
@@ -699,15 +767,33 @@ gnome_canvas_line_unrealize (GnomeCanvasItem *item)
 
 static void
 item_to_canvas (GnomeCanvas *canvas, double *item_coords, GdkPoint *canvas_coords, int num_points,
-		double wdx, double wdy, int cdx, int cdy)
+		int *num_drawn_points, double wdx, double wdy, int cdx, int cdy)
 {
 	int i;
 	int cx, cy;
+	int old_cx, old_cy;
 
-	for (i = 0; i < num_points; i++, item_coords += 2, canvas_coords++) {
+	/* the first point is always drawn */
+
+	gnome_canvas_w2c (canvas, wdx + item_coords[0], wdy + item_coords[1], &cx, &cy);
+	canvas_coords->x = cx - cdx;
+	canvas_coords->y = cy - cdy;
+	canvas_coords++;
+	item_coords += 2;
+	old_cx = cx;
+	old_cy = cy;
+	*num_drawn_points = 1;
+	
+	for (i = 1; i < num_points; i++, item_coords += 2 ) {
 		gnome_canvas_w2c (canvas, wdx + item_coords[0], wdy + item_coords[1], &cx, &cy);
-		canvas_coords->x = cx - cdx;
-		canvas_coords->y = cy - cdy;
+		if (old_cx != cx || old_cy!= cy) {
+			canvas_coords->x = cx - cdx;
+			canvas_coords->y = cy - cdy;
+			old_cx = cx;
+			old_cy = cy;
+			canvas_coords++; 
+			(*num_drawn_points)++;
+		}
 	}
 }
 
@@ -718,6 +804,7 @@ gnome_canvas_line_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 	GnomeCanvasLine *line;
 	GdkPoint static_points[NUM_STATIC_POINTS];
 	GdkPoint *points;
+	int actual_num_points_drawn;
 	double dx, dy;
 
 	line = GNOME_CANVAS_LINE (item);
@@ -735,8 +822,13 @@ gnome_canvas_line_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 	dx = dy = 0.0;
 	gnome_canvas_item_i2w (item, &dx, &dy);
 
-	item_to_canvas (item->canvas, line->coords, points, line->num_points, dx, dy, x, y);
-	gdk_draw_lines (drawable, line->gc, points, line->num_points);
+	item_to_canvas (item->canvas, line->coords, points, line->num_points,
+			&actual_num_points_drawn, dx, dy, x, y);
+
+	if (line->stipple)
+		gnome_canvas_set_stipple_origin (item->canvas, line->gc);
+
+	gdk_draw_lines (drawable, line->gc, points, actual_num_points_drawn);
 
 	if (points != static_points)
 		g_free (points);
@@ -746,13 +838,15 @@ gnome_canvas_line_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 	points = static_points;
 
 	if (line->first_arrow) {
-		item_to_canvas (item->canvas, line->first_coords, points, NUM_ARROW_POINTS, dx, dy, x, y);
-		gdk_draw_polygon (drawable, line->gc, TRUE, points, NUM_ARROW_POINTS);
+		item_to_canvas (item->canvas, line->first_coords, points, NUM_ARROW_POINTS,
+				&actual_num_points_drawn, dx, dy, x, y);
+		gdk_draw_polygon (drawable, line->gc, TRUE, points, actual_num_points_drawn );
 	}
 
 	if (line->last_arrow) {
-		item_to_canvas (item->canvas, line->last_coords, points, NUM_ARROW_POINTS, dx, dy, x, y);
-		gdk_draw_polygon (drawable, line->gc, TRUE, points, NUM_ARROW_POINTS);
+		item_to_canvas (item->canvas, line->last_coords, points, NUM_ARROW_POINTS,
+				&actual_num_points_drawn, dx, dy, x, y);
+		gdk_draw_polygon (drawable, line->gc, TRUE, points, actual_num_points_drawn );
 	}
 }
 
@@ -955,3 +1049,18 @@ gnome_canvas_line_translate (GnomeCanvasItem *item, double dx, double dy)
 
 	recalc_bounds (line);
 }
+
+static void
+gnome_canvas_line_bounds (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2)
+{
+	GnomeCanvasLine *line;
+
+	line = GNOME_CANVAS_LINE (item);
+
+	if (line->num_points == 0) {
+		*x1 = *y1 = *x2 = *y2 = 0.0;
+		return;
+	}
+
+	get_bounds (line, x1, y1, x2, y2);
+}
