@@ -12,6 +12,10 @@
 #include <math.h>
 #include "gnome-canvas-rect-ellipse.h"
 
+#include "libart_lgpl/art_vpath.h"
+#include "libart_lgpl/art_svp.h"
+#include "libart_lgpl/art_svp_vpath.h"
+#include "libart_lgpl/art_rgb_svp.h"
 
 /* Base class for rectangle and ellipse item types */
 
@@ -26,6 +30,8 @@ enum {
 	ARG_FILL_COLOR_GDK,
 	ARG_OUTLINE_COLOR,
 	ARG_OUTLINE_COLOR_GDK,
+	ARG_FILL_STIPPLE,
+	ARG_OUTLINE_STIPPLE,
 	ARG_WIDTH_PIXELS,
 	ARG_WIDTH_UNITS
 };
@@ -33,6 +39,7 @@ enum {
 
 static void gnome_canvas_re_class_init (GnomeCanvasREClass *class);
 static void gnome_canvas_re_init       (GnomeCanvasRE      *re);
+static void gnome_canvas_re_destroy    (GtkObject          *object);
 static void gnome_canvas_re_set_arg    (GtkObject          *object,
 					GtkArg             *arg,
 					guint               arg_id);
@@ -40,10 +47,13 @@ static void gnome_canvas_re_get_arg    (GtkObject          *object,
 					GtkArg             *arg,
 					guint               arg_id);
 
-static void gnome_canvas_re_reconfigure (GnomeCanvasItem *item);
+static void gnome_canvas_re_update      (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags);
 static void gnome_canvas_re_realize     (GnomeCanvasItem *item);
 static void gnome_canvas_re_unrealize   (GnomeCanvasItem *item);
 static void gnome_canvas_re_translate   (GnomeCanvasItem *item, double dx, double dy);
+static void gnome_canvas_re_bounds      (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2);
+
+static void gnome_canvas_re_render      (GnomeCanvasItem *item, GnomeCanvasBuf *buf);
 
 
 static GnomeCanvasItemClass *re_parent_class;
@@ -91,16 +101,22 @@ gnome_canvas_re_class_init (GnomeCanvasREClass *class)
 	gtk_object_add_arg_type ("GnomeCanvasRE::fill_color_gdk", GTK_TYPE_GDK_COLOR, GTK_ARG_READWRITE, ARG_FILL_COLOR_GDK);
 	gtk_object_add_arg_type ("GnomeCanvasRE::outline_color", GTK_TYPE_STRING, GTK_ARG_WRITABLE, ARG_OUTLINE_COLOR);
 	gtk_object_add_arg_type ("GnomeCanvasRE::outline_color_gdk", GTK_TYPE_GDK_COLOR, GTK_ARG_READWRITE, ARG_OUTLINE_COLOR_GDK);
+	gtk_object_add_arg_type ("GnomeCanvasRE::fill_stipple", GTK_TYPE_GDK_WINDOW, GTK_ARG_READWRITE, ARG_FILL_STIPPLE);
+	gtk_object_add_arg_type ("GnomeCanvasRE::outline_stipple", GTK_TYPE_GDK_WINDOW, GTK_ARG_READWRITE, ARG_OUTLINE_STIPPLE);
 	gtk_object_add_arg_type ("GnomeCanvasRE::width_pixels", GTK_TYPE_UINT, GTK_ARG_WRITABLE, ARG_WIDTH_PIXELS);
 	gtk_object_add_arg_type ("GnomeCanvasRE::width_units", GTK_TYPE_DOUBLE, GTK_ARG_WRITABLE, ARG_WIDTH_UNITS);
 
+	object_class->destroy = gnome_canvas_re_destroy;
 	object_class->set_arg = gnome_canvas_re_set_arg;
 	object_class->get_arg = gnome_canvas_re_get_arg;
 
-	item_class->reconfigure = gnome_canvas_re_reconfigure;
+	item_class->update = gnome_canvas_re_update;
 	item_class->realize = gnome_canvas_re_realize;
 	item_class->unrealize = gnome_canvas_re_unrealize;
 	item_class->translate = gnome_canvas_re_translate;
+	item_class->bounds = gnome_canvas_re_bounds;
+
+	item_class->render = gnome_canvas_re_render;
 }
 
 static void
@@ -111,6 +127,28 @@ gnome_canvas_re_init (GnomeCanvasRE *re)
 	re->x2 = 0.0;
 	re->y2 = 0.0;
 	re->width = 0.0;
+	re->fill_svp = NULL;
+	re->outline_svp = NULL;
+}
+
+static void
+gnome_canvas_re_destroy (GtkObject *object)
+{
+	GnomeCanvasRE *re;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (GNOME_IS_CANVAS_RE (object));
+
+	re = GNOME_CANVAS_RE (object);
+
+	if (re->fill_stipple)
+		gdk_bitmap_unref (re->fill_stipple);
+
+	if (re->outline_stipple)
+		gdk_bitmap_unref (re->outline_stipple);
+
+	if (GTK_OBJECT_CLASS (re_parent_class)->destroy)
+		(* GTK_OBJECT_CLASS (re_parent_class)->destroy) (object);
 }
 
 static void
@@ -118,6 +156,7 @@ recalc_bounds (GnomeCanvasRE *re)
 {
 	GnomeCanvasItem *item;
 	double x1, y1, x2, y2;
+	int cx1, cy1, cx2, cy2;
 	double hwidth;
 
 	item = GNOME_CANVAS_ITEM (re);
@@ -134,8 +173,12 @@ recalc_bounds (GnomeCanvasRE *re)
 
 	gnome_canvas_item_i2w (item, &x1, &y1);
 	gnome_canvas_item_i2w (item, &x2, &y2);
-	gnome_canvas_w2c (item->canvas, x1 - hwidth, y1 - hwidth, &item->x1, &item->y1);
-	gnome_canvas_w2c (item->canvas, x2 + hwidth, y2 + hwidth, &item->x2, &item->y2);
+	gnome_canvas_w2c (item->canvas, x1 - hwidth, y1 - hwidth, &cx1, &cy1);
+	gnome_canvas_w2c (item->canvas, x2 + hwidth, y2 + hwidth, &cx2, &cy2);
+	item->x1 = cx1;
+	item->y1 = cy1;
+	item->x2 = cx2;
+	item->y2 = cy2;
 
 	/* Some safety fudging */
 
@@ -145,6 +188,9 @@ recalc_bounds (GnomeCanvasRE *re)
 	item->y2 += 2;
 
 	gnome_canvas_group_child_bounds (GNOME_CANVAS_GROUP (item->parent), item);
+
+	if (item->canvas->aa)
+		gnome_canvas_item_request_update (item);
 }
 
 /* Convenience function to set a GC's foreground color to the specified pixel value */
@@ -160,6 +206,25 @@ set_gc_foreground (GdkGC *gc, gulong pixel)
 	gdk_gc_set_foreground (gc, &c);
 }
 
+/* Sets the stipple pattern for the specified gc */
+static void
+set_stipple (GdkGC *gc, GdkBitmap **internal_stipple, GdkBitmap *stipple, int reconfigure)
+{
+	if (*internal_stipple && !reconfigure)
+		gdk_bitmap_unref (*internal_stipple);
+
+	*internal_stipple = stipple;
+	if (stipple && !reconfigure)
+		gdk_bitmap_ref (stipple);
+
+	if (gc)
+		if (stipple) {
+			gdk_gc_set_stipple (gc, stipple);
+			gdk_gc_set_fill (gc, GDK_STIPPLED);
+		} else
+			gdk_gc_set_fill (gc, GDK_SOLID);
+}
+
 /* Recalculate the outline width of the rectangle/ellipse and set it in its GC */
 static void
 set_outline_gc_width (GnomeCanvasRE *re)
@@ -214,9 +279,21 @@ gnome_canvas_re_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		if (gnome_canvas_get_color (item->canvas, GTK_VALUE_STRING (*arg), &color)) {
 			re->fill_set = TRUE;
 			re->fill_pixel = color.pixel;
-			set_gc_foreground (re->fill_gc, re->fill_pixel);
-		} else
+			if (item->canvas->aa)
+				re->fill_color =
+					((color.red & 0xff00) << 16) |
+					((color.green & 0xff00) << 8) |
+					(color.blue & 0xff00) |
+					0xff;
+			else
+				set_gc_foreground (re->fill_gc, re->fill_pixel);
+		} else {
 			re->fill_set = FALSE;
+			re->fill_color = 0;
+		}
+
+		if (item->canvas->aa)
+			gnome_canvas_item_request_update (item);
 
 		break;
 
@@ -230,9 +307,18 @@ gnome_canvas_re_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		if (gnome_canvas_get_color (item->canvas, GTK_VALUE_STRING (*arg), &color)) {
 			re->outline_set = TRUE;
 			re->outline_pixel = color.pixel;
-			set_gc_foreground (re->outline_gc, re->outline_pixel);
-		} else
+			if (item->canvas->aa)
+				re->outline_color =
+					((color.red & 0xff00) << 16) |
+					((color.green & 0xff00) << 8) |
+					(color.blue & 0xff00) |
+					0xff;
+			else
+				set_gc_foreground (re->outline_gc, re->outline_pixel);
+		} else {
 			re->outline_set = FALSE;
+			re->outline_color = 0;
+		}
 
 		break;
 
@@ -242,6 +328,14 @@ gnome_canvas_re_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		set_gc_foreground (re->outline_gc, re->outline_pixel);
 		break;
 
+	case ARG_FILL_STIPPLE:
+		set_stipple (re->fill_gc, &re->fill_stipple, GTK_VALUE_BOXED (*arg), FALSE);
+		break;
+
+	case ARG_OUTLINE_STIPPLE:
+		set_stipple (re->outline_gc, &re->outline_stipple, GTK_VALUE_BOXED (*arg), FALSE);
+		break;
+		
 	case ARG_WIDTH_PIXELS:
 		re->width = GTK_VALUE_UINT (*arg);
 		re->width_pixels = TRUE;
@@ -279,7 +373,6 @@ static void
 gnome_canvas_re_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 {
 	GnomeCanvasRE *re;
-	GdkColor *color;
 
 	re = GNOME_CANVAS_RE (object);
 
@@ -308,6 +401,14 @@ gnome_canvas_re_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		get_color_arg (re, re->outline_pixel, arg);
 		break;
 
+	case ARG_FILL_STIPPLE:
+		GTK_VALUE_BOXED (*arg) = re->fill_stipple;
+		break;
+
+	case ARG_OUTLINE_STIPPLE:
+		GTK_VALUE_BOXED (*arg) = re->outline_stipple;
+		break;
+
 	default:
 		arg->type = GTK_TYPE_INVALID;
 		break;
@@ -315,18 +416,22 @@ gnome_canvas_re_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 }
 
 static void
-gnome_canvas_re_reconfigure (GnomeCanvasItem *item)
+gnome_canvas_re_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags)
 {
 	GnomeCanvasRE *re;
 
 	re = GNOME_CANVAS_RE (item);
 
-	if (re_parent_class->reconfigure)
-		(* re_parent_class->reconfigure) (item);
+	if (re_parent_class->update)
+		(* re_parent_class->update) (item, affine, clip_path, flags);
 
-	set_gc_foreground (re->fill_gc, re->fill_pixel);
-	set_gc_foreground (re->outline_gc, re->outline_pixel);
-	set_outline_gc_width (re);
+	if (!item->canvas->aa) {
+		set_gc_foreground (re->fill_gc, re->fill_pixel);
+		set_gc_foreground (re->outline_gc, re->outline_pixel);
+		set_stipple (re->fill_gc, &re->fill_stipple, re->fill_stipple, TRUE);
+		set_stipple (re->outline_gc, &re->outline_stipple, re->outline_stipple, TRUE);
+		set_outline_gc_width (re);
+	} 
 
 	recalc_bounds (re);
 }
@@ -344,7 +449,7 @@ gnome_canvas_re_realize (GnomeCanvasItem *item)
 	re->fill_gc = gdk_gc_new (item->canvas->layout.bin_window);
 	re->outline_gc = gdk_gc_new (item->canvas->layout.bin_window);
 
-	(* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->reconfigure) (item);
+	(* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->update) (item, NULL, NULL, 0);
 }
 
 static void
@@ -376,15 +481,81 @@ gnome_canvas_re_translate (GnomeCanvasItem *item, double dx, double dy)
 	recalc_bounds (re);
 }
 
+static void
+gnome_canvas_re_bounds (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2)
+{
+	GnomeCanvasRE *re;
+	double hwidth;
+
+	re = GNOME_CANVAS_RE (item);
+
+	if (re->width_pixels)
+		hwidth = (re->width / item->canvas->pixels_per_unit) / 2.0;
+	else
+		hwidth = re->width / 2.0;
+
+	*x1 = re->x1 - hwidth;
+	*y1 = re->y1 - hwidth;
+	*x2 = re->x2 + hwidth;
+	*y2 = re->y2 + hwidth;
+}
+
+static void
+gnome_canvas_re_render (GnomeCanvasItem *item,
+			GnomeCanvasBuf *buf)
+{
+	GnomeCanvasRE *re;
+	guint32 fg_color, bg_color;
+
+	re = GNOME_CANVAS_RE (item);
+
+	if (re->fill_svp != NULL) {
+
+		if (buf->is_bg) {
+			bg_color = buf->bg_color;
+			fg_color = re->fill_color >> 8; /* FIXME: this needs to be a composite */
+			art_rgb_svp_aa (re->fill_svp,
+					buf->rect.x0, buf->rect.y0, buf->rect.x1, buf->rect.y1,
+					fg_color, bg_color,
+					buf->buf, buf->buf_rowstride);
+			buf->is_bg = 0;
+			buf->is_buf = 1;
+		} else {
+			art_rgb_svp_alpha (re->fill_svp,
+					   buf->rect.x0, buf->rect.y0, buf->rect.x1, buf->rect.y1,
+					   re->fill_color,
+					   buf->buf, buf->buf_rowstride);
+		}
+	}
+
+	if (re->outline_svp != NULL) {
+
+		if (buf->is_bg) {
+			bg_color = buf->bg_color;
+			fg_color = re->outline_color >> 8; /* FIXME: this needs to be a composite */
+			art_rgb_svp_aa (re->outline_svp,
+					buf->rect.x0, buf->rect.y0, buf->rect.x1, buf->rect.y1,
+					fg_color, bg_color,
+					buf->buf, buf->buf_rowstride);
+			buf->is_bg = 0;
+			buf->is_buf = 1;
+		} else {
+			art_rgb_svp_alpha (re->outline_svp,
+					   buf->rect.x0, buf->rect.y0, buf->rect.x1, buf->rect.y1,
+					   re->outline_color,
+					   buf->buf, buf->buf_rowstride);
+		}
+	}
+}
 
 /* Rectangle item */
 
 
 static void gnome_canvas_rect_class_init (GnomeCanvasRectClass *class);
 
-static void   gnome_canvas_rect_draw  (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width, int height);
-static double gnome_canvas_rect_point (GnomeCanvasItem *item, double x, double y, int cx, int cy,
-				       GnomeCanvasItem **actual_item);
+static void   gnome_canvas_rect_draw   (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width, int height);
+static double gnome_canvas_rect_point  (GnomeCanvasItem *item, double x, double y, int cx, int cy,
+				        GnomeCanvasItem **actual_item);
 
 
 GtkType
@@ -440,7 +611,10 @@ gnome_canvas_rect_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int
 	gnome_canvas_w2c (item->canvas, dx + re->x1, dy + re->y1, &x1, &y1);
 	gnome_canvas_w2c (item->canvas, dx + re->x2, dy + re->y2, &x2, &y2);
 
-	if (re->fill_set)
+	if (re->fill_set) {
+		if (re->fill_stipple)
+			gnome_canvas_set_stipple_origin (item->canvas, re->fill_gc);
+
 		gdk_draw_rectangle (drawable,
 				    re->fill_gc,
 				    TRUE,
@@ -448,8 +622,12 @@ gnome_canvas_rect_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int
 				    y1 - y,
 				    x2 - x1 + 1,
 				    y2 - y1 + 1);
+	}
+
+	if (re->outline_set) {
+		if (re->outline_stipple)
+			gnome_canvas_set_stipple_origin (item->canvas, re->outline_gc);
 
-	if (re->outline_set)
 		gdk_draw_rectangle (drawable,
 				    re->outline_gc,
 				    FALSE,
@@ -457,6 +635,7 @@ gnome_canvas_rect_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int
 				    y1 - y,
 				    x2 - x1,
 				    y2 - y1);
+	}
 }
 
 static double
@@ -538,16 +717,130 @@ gnome_canvas_rect_point (GnomeCanvasItem *item, double x, double y, int cx, int
 	return sqrt (dx * dx + dy * dy);
 }
 
+#if 0
+static void
+gnome_canvas_rect_update (GnomeCanvasItem *item)
+{
+	GnomeCanvasRE *re;
+	ArtVpath vpath[11];
+	double x0, y0, x1, y1;
+	double dx, dy;
+	double halfwidth;
+	int i;
+
+	re = GNOME_CANVAS_RE (item);
+
+	x0 = re->x1 * item->canvas->pixels_per_unit;
+	y0 = re->y1 * item->canvas->pixels_per_unit;
+	x1 = re->x2 * item->canvas->pixels_per_unit;
+	y1 = re->y2 * item->canvas->pixels_per_unit;
+
+	if (re->fill_svp)
+		art_svp_free (re->fill_svp);
+
+	if (re->fill_set) {
+		vpath[0].code = ART_MOVETO;
+		vpath[0].x = x0;
+		vpath[0].y = y0;
+		vpath[1].code = ART_LINETO;
+		vpath[1].x = x0;
+		vpath[1].y = y1;
+		vpath[2].code = ART_LINETO;
+		vpath[2].x = x1;
+		vpath[2].y = y1;
+		vpath[3].code = ART_LINETO;
+		vpath[3].x = x1;
+		vpath[3].y = y0;
+		vpath[4].code = ART_LINETO;
+		vpath[4].x = x0;
+		vpath[4].y = y0;
+		vpath[5].code = ART_END;
+		vpath[5].x = 0;
+		vpath[5].y = 0;
+
+		re->fill_svp = art_svp_from_vpath (vpath);
+	} else
+		re->fill_svp = NULL;
+
+	if (re->outline_svp)
+		art_svp_free (re->outline_svp);
+
+	if (re->outline_set) {
+		/* We do the stroking by hand because it's simple enough
+		   and could save time. */
+
+		if (re->width_pixels)
+			halfwidth = re->width * 0.5;
+		else
+			halfwidth = re->width * item->canvas->pixels_per_unit * 0.5;
+
+		i = 0;
+		vpath[i].code = ART_MOVETO;
+		vpath[i].x = x0 - halfwidth;
+		vpath[i].y = y0 - halfwidth;
+		i++;
+		vpath[i].code = ART_LINETO;
+		vpath[i].x = x0 - halfwidth;
+		vpath[i].y = y1 + halfwidth;
+		i++;
+		vpath[i].code = ART_LINETO;
+		vpath[i].x = x1 + halfwidth;
+		vpath[i].y = y1 + halfwidth;
+		i++;
+		vpath[i].code = ART_LINETO;
+		vpath[i].x = x1 + halfwidth;
+		vpath[i].y = y0 - halfwidth;
+		i++;
+		vpath[i].code = ART_LINETO;
+		vpath[i].x = x0 - halfwidth;
+		vpath[i].y = y0 - halfwidth;
+		i++;
+
+		if (x1 - halfwidth > x0 + halfwidth &&
+		    y1 - halfwidth > y0 + halfwidth) {
+			vpath[i].code = ART_MOVETO;
+			vpath[i].x = x0 + halfwidth;
+			vpath[i].y = y0 + halfwidth;
+			i++;
+			vpath[i].code = ART_LINETO;
+			vpath[i].x = x1 - halfwidth;
+			vpath[i].y = y0 + halfwidth;
+			i++;
+			vpath[i].code = ART_LINETO;
+			vpath[i].x = x1 - halfwidth;
+			vpath[i].y = y1 - halfwidth;
+			i++;
+			vpath[i].code = ART_LINETO;
+			vpath[i].x = x0 + halfwidth;
+			vpath[i].y = y1 - halfwidth;
+			i++;
+			vpath[i].code = ART_LINETO;
+			vpath[i].x = x0 + halfwidth;
+			vpath[i].y = y0 + halfwidth;
+			i++;
+		}
+		vpath[i].code = ART_END;
+		vpath[i].x = 0;
+		vpath[i].y = 0;
+
+		re->outline_svp = art_svp_from_vpath (vpath);
+	} else
+		re->outline_svp = NULL;
+
+
+	/* FIXME: request repaint */
+}
+#endif
 
 /* Ellipse item */
 
 
 static void gnome_canvas_ellipse_class_init (GnomeCanvasEllipseClass *class);
 
-static void   gnome_canvas_ellipse_draw  (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y,
-					  int width, int height);
-static double gnome_canvas_ellipse_point (GnomeCanvasItem *item, double x, double y, int cx, int cy,
-					  GnomeCanvasItem **actual_item);
+static void   gnome_canvas_ellipse_draw   (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y,
+					   int width, int height);
+static double gnome_canvas_ellipse_point  (GnomeCanvasItem *item, double x, double y, int cx, int cy,
+					   GnomeCanvasItem **actual_item);
 
 
 GtkType
@@ -603,7 +896,10 @@ gnome_canvas_ellipse_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x,
 	gnome_canvas_w2c (item->canvas, dx + re->x1, dy + re->y1, &x1, &y1);
 	gnome_canvas_w2c (item->canvas, dx + re->x2, dy + re->y2, &x2, &y2);
 
-	if (re->fill_set)
+	if (re->fill_set) {
+		if (re->fill_stipple)
+			gnome_canvas_set_stipple_origin (item->canvas, re->fill_gc);
+		
 		gdk_draw_arc (drawable,
 			      re->fill_gc,
 			      TRUE,
@@ -613,8 +909,12 @@ gnome_canvas_ellipse_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x,
 			      y2 - y1,
 			      0 * 64,
 			      360 * 64);
+	}
+
+	if (re->outline_set) {
+		if (re->outline_stipple)
+			gnome_canvas_set_stipple_origin (item->canvas, re->outline_gc);
 
-	if (re->outline_set)
 		gdk_draw_arc (drawable,
 			      re->outline_gc,
 			      FALSE,
@@ -624,6 +924,7 @@ gnome_canvas_ellipse_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x,
 			      y2 - y1,
 			      0 * 64,
 			      360 * 64);
+	}
 }
 
 static double
@@ -697,3 +998,87 @@ gnome_canvas_ellipse_point (GnomeCanvasItem *item, double x, double y, int cx, i
 
 	return outline_dist;
 }
+
+#define N_PTS 126
+
+static void
+gnome_canvas_gen_ellipse (ArtVpath *vpath, double x0, double y0,
+			  double x1, double y1)
+{
+	int i;
+
+	for (i = 0; i < N_PTS + 1; i++) {
+		double th;
+
+		th = (2 * M_PI * i) / N_PTS;
+		vpath[i].code = i == 0 ? ART_MOVETO : ART_LINETO;
+		vpath[i].x = (x0 + x1) * 0.5 + (x1 - x0) * 0.5 * cos (th);
+		vpath[i].y = (y0 + y1) * 0.5 - (y1 - y0) * 0.5 * sin (th);
+	}
+}
+#if 0
+static void
+gnome_canvas_ellipse_update (GnomeCanvasItem *item)
+{
+	GnomeCanvasRE *re;
+	ArtVpath vpath[N_PTS + N_PTS + 3];
+	double x0, y0, x1, y1;
+	double dx, dy;
+	int i;
+
+	re = GNOME_CANVAS_RE (item);
+
+	dx = 0.0;
+	dy = 0.0;
+
+	gnome_canvas_item_i2w (item, &dx, &dy);
+
+	gnome_canvas_w2c_d (item->canvas, dx + re->x1, dy + re->y1, &x0, &y0);
+	gnome_canvas_w2c_d (item->canvas, dx + re->x2, dy + re->y2, &x1, &y1);
+
+	if (re->fill_svp)
+		art_svp_free (re->fill_svp);
+
+	if (re->fill_set) {
+		gnome_canvas_gen_ellipse (vpath, x0, y0, x1, y1);
+		vpath[N_PTS + 1].code = ART_END;
+		vpath[N_PTS + 1].x = 0;
+		vpath[N_PTS + 1].y = 0;
+
+		re->fill_svp = art_svp_from_vpath (vpath);
+	}
+
+	if (re->outline_svp)
+		art_svp_free (re->outline_svp);
+
+	if (re->outline_set) {
+		double halfwidth;
+
+		if (re->width_pixels)
+			halfwidth = re->width * 0.5;
+		else
+			halfwidth = re->width * item->canvas->pixels_per_unit * 0.5;
+
+		i = 0;
+		gnome_canvas_gen_ellipse (vpath + i,
+					  x0 - halfwidth, y0 - halfwidth,
+					  x1 + halfwidth, y1 + halfwidth);
+		i = N_PTS + 1;
+		if (x1 - halfwidth > x0 + halfwidth &&
+		    y1 - halfwidth > y0 + halfwidth) {
+			gnome_canvas_gen_ellipse (vpath + i,
+						  x0 + halfwidth, y1 - halfwidth,
+						  x1 - halfwidth, y0 + halfwidth);
+			i += N_PTS + 1;
+		}
+		vpath[i].code = ART_END;
+		vpath[i].x = 0;
+		vpath[i].y = 0;
+
+		re->outline_svp = art_svp_from_vpath (vpath);
+	} else
+		re->outline_svp = NULL;
+
+	/* FIXME: request repaint */
+}
+#endif
