@@ -35,7 +35,10 @@ enum {
 	ARG_X_OFFSET,
 	ARG_Y_OFFSET,
 	ARG_FILL_COLOR,
-	ARG_FILL_COLOR_GDK
+	ARG_FILL_COLOR_GDK,
+	ARG_FILL_STIPPLE,
+	ARG_TEXT_WIDTH,
+	ARG_TEXT_HEIGHT
 };
 
 
@@ -49,7 +52,7 @@ static void gnome_canvas_text_get_arg    (GtkObject            *object,
 					  GtkArg               *arg,
 					  guint                 arg_id);
 
-static void   gnome_canvas_text_reconfigure (GnomeCanvasItem *item);
+static void   gnome_canvas_text_update      (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags);
 static void   gnome_canvas_text_realize     (GnomeCanvasItem *item);
 static void   gnome_canvas_text_unrealize   (GnomeCanvasItem *item);
 static void   gnome_canvas_text_draw        (GnomeCanvasItem *item, GdkDrawable *drawable,
@@ -57,6 +60,7 @@ static void   gnome_canvas_text_draw        (GnomeCanvasItem *item, GdkDrawable
 static double gnome_canvas_text_point       (GnomeCanvasItem *item, double x, double y,
 					     int cx, int cy, GnomeCanvasItem **actual_item);
 static void   gnome_canvas_text_translate   (GnomeCanvasItem *item, double dx, double dy);
+static void   gnome_canvas_text_bounds      (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2);
 
 
 static GnomeCanvasItemClass *parent_class;
@@ -110,17 +114,21 @@ gnome_canvas_text_class_init (GnomeCanvasTextClass *class)
 	gtk_object_add_arg_type ("GnomeCanvasText::y_offset", GTK_TYPE_DOUBLE, GTK_ARG_READWRITE, ARG_Y_OFFSET);
 	gtk_object_add_arg_type ("GnomeCanvasText::fill_color", GTK_TYPE_STRING, GTK_ARG_WRITABLE, ARG_FILL_COLOR);
 	gtk_object_add_arg_type ("GnomeCanvasText::fill_color_gdk", GTK_TYPE_GDK_COLOR, GTK_ARG_READWRITE, ARG_FILL_COLOR_GDK);
+	gtk_object_add_arg_type ("GnomeCanvasText::fill_stipple", GTK_TYPE_GDK_WINDOW, GTK_ARG_READWRITE, ARG_FILL_STIPPLE);
+	gtk_object_add_arg_type ("GnomeCanvasText::text_width", GTK_TYPE_DOUBLE, GTK_ARG_READABLE, ARG_TEXT_WIDTH);
+	gtk_object_add_arg_type ("GnomeCanvasText::text_height", GTK_TYPE_DOUBLE, GTK_ARG_READABLE, ARG_TEXT_HEIGHT);
 
 	object_class->destroy = gnome_canvas_text_destroy;
 	object_class->set_arg = gnome_canvas_text_set_arg;
 	object_class->get_arg = gnome_canvas_text_get_arg;
 
-	item_class->reconfigure = gnome_canvas_text_reconfigure;
+	item_class->update = gnome_canvas_text_update;
 	item_class->realize = gnome_canvas_text_realize;
 	item_class->unrealize = gnome_canvas_text_unrealize;
 	item_class->draw = gnome_canvas_text_draw;
 	item_class->point = gnome_canvas_text_point;
 	item_class->translate = gnome_canvas_text_translate;
+	item_class->bounds = gnome_canvas_text_bounds;
 }
 
 static void
@@ -156,6 +164,9 @@ gnome_canvas_text_destroy (GtkObject *object)
 
 	gdk_font_unref (text->font);
 
+	if (text->stipple)
+		gdk_bitmap_unref (text->stipple);
+
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
@@ -340,6 +351,25 @@ set_text_gc_foreground (GnomeCanvasText *text)
 	gdk_gc_set_foreground (text->gc, &c);
 }
 
+/* Sets the stipple pattern for the text */
+static void
+set_stipple (GnomeCanvasText *text, GdkBitmap *stipple, int reconfigure)
+{
+	if (text->stipple && !reconfigure)
+		gdk_bitmap_unref (text->stipple);
+
+	text->stipple = stipple;
+	if (stipple && !reconfigure)
+		gdk_bitmap_ref (stipple);
+
+	if (text->gc)
+		if (stipple) {
+			gdk_gc_set_stipple (text->gc, stipple);
+			gdk_gc_set_fill (text->gc, GDK_STIPPLED);
+		} else
+			gdk_gc_set_fill (text->gc, GDK_SOLID);
+}
+
 static void
 gnome_canvas_text_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 {
@@ -442,6 +472,10 @@ gnome_canvas_text_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		set_text_gc_foreground (text);
 		break;
 
+	case ARG_FILL_STIPPLE:
+		set_stipple (text, GTK_VALUE_BOXED (*arg), FALSE);
+		break;
+
 	default:
 		break;
 	}
@@ -507,6 +541,18 @@ gnome_canvas_text_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		GTK_VALUE_BOXED (*arg) = color;
 		break;
 
+	case ARG_FILL_STIPPLE:
+		GTK_VALUE_BOXED (*arg) = text->stipple;
+		break;
+
+	case ARG_TEXT_WIDTH:
+		GTK_VALUE_DOUBLE (*arg) = text->max_width / text->item.canvas->pixels_per_unit;
+		break;
+
+	case ARG_TEXT_HEIGHT:
+		GTK_VALUE_DOUBLE (*arg) = text->height / text->item.canvas->pixels_per_unit;
+		break;
+
 	default:
 		arg->type = GTK_TYPE_INVALID;
 		break;
@@ -514,17 +560,17 @@ gnome_canvas_text_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 }
 
 static void
-gnome_canvas_text_reconfigure (GnomeCanvasItem *item)
+gnome_canvas_text_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags)
 {
 	GnomeCanvasText *text;
-	GdkColor color;
 
 	text = GNOME_CANVAS_TEXT (item);
 
-	if (parent_class->reconfigure)
-		(* parent_class->reconfigure) (item);
+	if (parent_class->update)
+		(* parent_class->update) (item, affine, clip_path, flags);
 
 	set_text_gc_foreground (text);
+	set_stipple (text, text->stipple, TRUE);
 
 	recalc_bounds (text);
 }
@@ -541,7 +587,7 @@ gnome_canvas_text_realize (GnomeCanvasItem *item)
 
 	text->gc = gdk_gc_new (item->canvas->layout.bin_window);
 
-	(* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->reconfigure) (item);
+	(* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->update) (item, NULL, NULL, 0);
 }
 
 static void
@@ -611,16 +657,21 @@ gnome_canvas_text_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 	lines = text->lines;
 	ypos = text->cy + text->font->ascent;
 
-	for (i = 0; i < text->num_lines; i++) {
-		xpos = get_line_xpos (text, lines);
+	if (text->stipple)
+		gnome_canvas_set_stipple_origin (item->canvas, text->gc);
 
-		gdk_draw_text (drawable,
-			       text->font,
-			       text->gc,
-			       xpos - x,
-			       ypos - y,
-			       lines->text,
-			       lines->length);
+	for (i = 0; i < text->num_lines; i++) {
+		if (lines->length != 0) {
+			xpos = get_line_xpos (text, lines);
+
+			gdk_draw_text (drawable,
+				       text->font,
+				       text->gc,
+				       xpos - x,
+				       ypos - y,
+				       lines->text,
+				       lines->length);
+		}
 
 		ypos += text->font->ascent + text->font->descent;
 		lines++;
@@ -725,3 +776,64 @@ gnome_canvas_text_translate (GnomeCanvasItem *item, double dx, double dy)
 
 	recalc_bounds (text);
 }
+
+static void
+gnome_canvas_text_bounds (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2)
+{
+	GnomeCanvasText *text;
+	double width, height;
+
+	text = GNOME_CANVAS_TEXT (item);
+
+	*x1 = text->x;
+	*y1 = text->y;
+
+	if (text->clip) {
+		width = text->clip_width;
+		height = text->clip_height;
+	} else {
+		width = text->max_width / item->canvas->pixels_per_unit;
+		height = text->height / item->canvas->pixels_per_unit;
+	}
+
+	switch (text->anchor) {
+	case GTK_ANCHOR_NW:
+	case GTK_ANCHOR_W:
+	case GTK_ANCHOR_SW:
+		break;
+
+	case GTK_ANCHOR_N:
+	case GTK_ANCHOR_CENTER:
+	case GTK_ANCHOR_S:
+		*x1 -= width / 2.0;
+		break;
+
+	case GTK_ANCHOR_NE:
+	case GTK_ANCHOR_E:
+	case GTK_ANCHOR_SE:
+		*x1 -= width;
+		break;
+	}
+
+	switch (text->anchor) {
+	case GTK_ANCHOR_NW:
+	case GTK_ANCHOR_N:
+	case GTK_ANCHOR_NE:
+		break;
+
+	case GTK_ANCHOR_W:
+	case GTK_ANCHOR_CENTER:
+	case GTK_ANCHOR_E:
+		*y1 -= height / 2.0;
+		break;
+
+	case GTK_ANCHOR_SW:
+	case GTK_ANCHOR_S:
+	case GTK_ANCHOR_SE:
+		*y1 -= height;
+		break;
+	}
+
+	*x2 = *x1 + width;
+	*y2 = *y1 + height;
+}
