@@ -1,20 +1,25 @@
 /* GnomeCanvas widget - Tk-like canvas widget for Gnome
  *
- * GnomeCanvas is basically a port of the Tk toolkit's most excellent canvas widget.  Tk is
- * copyrighted by the Regents of the University of California, Sun Microsystems, and other parties.
+ * GnomeCanvas is basically a port of the Tk toolkit's most excellent canvas
+ * widget.  Tk is copyrighted by the Regents of the University of California,
+ * Sun Microsystems, and other parties.
  *
  * Copyright (C) 1998 The Free Software Foundation
  *
- * Author: Federico Mena <federico@nuclecu.unam.mx>
+ * Authors: Federico Mena <federico@nuclecu.unam.mx>
+ *          Raph Levien <raph@gimp.org>
  */
 
 #ifndef GNOME_CANVAS_H
 #define GNOME_CANVAS_H
 
 #include <libgnome/gnome-defs.h>
-#include <gtk/gtkcontainer.h>
-#include <libgnomeui/gtklayout.h>
+#include <gtk/gtklayout.h>
 #include <stdarg.h>
+#include <libart_lgpl/art_misc.h>
+#include <libart_lgpl/art_rect.h>
+#include <libart_lgpl/art_svp.h>
+#include <libart_lgpl/art_uta.h>
 
 BEGIN_GNOME_DECLS
 
@@ -23,10 +28,24 @@ BEGIN_GNOME_DECLS
 #define GNOME_CANVAS_EPSILON 1e-10
 
 
+/* Macros for building colors.  The values are in [0, 255] */
+
+#define GNOME_CANVAS_COLOR(r, g, b) ((((int) (r) & 0xff) << 24)	\
+				     | (((int) (g) & 0xff) << 16)	\
+				     | (((int) (b) & 0xff) << 8)	\
+				     | 0xff)
+
+#define GNOME_CANVAS_COLOR_A(r, g, b, a) ((((int) (r) & 0xff) << 24)	\
+					  | (((int) (g) & 0xff) << 16)	\
+					  | (((int) (b) & 0xff) << 8)	\
+					  | ((int) (a) & 0xff))
+
+
 typedef struct _GnomeCanvas           GnomeCanvas;
 typedef struct _GnomeCanvasClass      GnomeCanvasClass;
 typedef struct _GnomeCanvasGroup      GnomeCanvasGroup;
 typedef struct _GnomeCanvasGroupClass GnomeCanvasGroupClass;
+typedef struct _GnomeCanvasBuf        GnomeCanvasBuf;
 
 
 /* GnomeCanvasItem - base item class for canvas items
@@ -47,9 +66,40 @@ typedef struct _GnomeCanvasGroupClass GnomeCanvasGroupClass;
 enum {
 	GNOME_CANVAS_ITEM_REALIZED      = 1 << 4,
 	GNOME_CANVAS_ITEM_MAPPED        = 1 << 5,
-	GNOME_CANVAS_ITEM_ALWAYS_REDRAW = 1 << 6
+	GNOME_CANVAS_ITEM_ALWAYS_REDRAW = 1 << 6,
+	GNOME_CANVAS_ITEM_VISIBLE       = 1 << 7,
+	GNOME_CANVAS_ITEM_NEED_UPDATE	= 1 << 8
 };
 
+/* Update flags for items */
+enum {
+	GNOME_CANVAS_UPDATE_REQUESTED  = 1 << 0,
+	GNOME_CANVAS_UPDATE_AFFINE     = 1 << 1,
+	GNOME_CANVAS_UPDATE_CLIP       = 1 << 2,
+	GNOME_CANVAS_UPDATE_VISIBILITY = 1 << 3
+};
+
+/* Data for rendering in antialiased mode */
+
+struct _GnomeCanvasBuf {
+	guchar *buf;			/* 24 bit RGB buffer for rendering */
+	int buf_rowstride;		/* The rowstride for buf */
+	ArtIRect rect;			/* The rectangle describing the
+					 * rendering area.
+					 */
+	guint32 bg_color;		/* The background color in 0xrrggbb */
+
+	/* Invariant: at least one of the following flags is true. */
+
+	unsigned int is_bg : 1;		/* Set when the render rectangle area is
+					 * the solid color bg_color.
+					 */
+	unsigned int is_buf : 1;	/* Set when the render rectangle area is
+					 * represented by the buf
+					 */
+};
+
+
 #define GNOME_TYPE_CANVAS_ITEM            (gnome_canvas_item_get_type ())
 #define GNOME_CANVAS_ITEM(obj)            (GTK_CHECK_CAST ((obj), GNOME_TYPE_CANVAS_ITEM, GnomeCanvasItem))
 #define GNOME_CANVAS_ITEM_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), GNOME_TYPE_CANVAS_ITEM, GnomeCanvasItemClass))
@@ -66,7 +116,7 @@ struct _GnomeCanvasItem {
 	GnomeCanvas *canvas;		/* The parent canvas for this item */
 	GnomeCanvasItem *parent;	/* The parent canvas group (of type GnomeCanvasGroup) */
 
-	int x1, y1, x2, y2;		/* Bounding box for this item, in canvas pixel coordinates.
+	double x1, y1, x2, y2;		/* Bounding box for this item, in canvas pixel coordinates.
 					 * The bounding box contains (x1, y1) but not (x2, y2).
 					 */
 };
@@ -74,8 +124,12 @@ struct _GnomeCanvasItem {
 struct _GnomeCanvasItemClass {
 	GtkObjectClass parent_class;
 
-	/* Reconfigure an item -- fixup GCs, recalc bounds, etc. */
-	void (* reconfigure) (GnomeCanvasItem *item);
+	/* Tell the item to update itself.  The flags are from the update flags
+	 * defined above.  The item should update its internal state from its
+	 * queued state, recompute and request its repaint area, etc.  The
+	 * affine, if used, is a pointer to a 6-element array of doubles.
+	 */
+	void (* update) (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags);
 
 	/* Realize an item -- create GCs, etc. */
 	void (* realize) (GnomeCanvasItem *item);
@@ -89,6 +143,9 @@ struct _GnomeCanvasItemClass {
 	/* Unmap an item */
 	void (* unmap) (GnomeCanvasItem *item);
 
+	/* Return the microtile coverage of the item */
+	ArtUta *(* coverage) (GnomeCanvasItem *item);
+
 	/* Draw an item of this type.  (x, y) are the upper-left canvas pixel coordinates of the *
 	 * drawable, a temporary pixmap, where things get drawn.  (width, height) are the dimensions
 	 * of the drawable.
@@ -96,6 +153,15 @@ struct _GnomeCanvasItemClass {
 	void (* draw) (GnomeCanvasItem *item, GdkDrawable *drawable,
 		       int x, int y, int width, int height);
 
+	/* Render the item over the buffer given.  The buf data structure
+	 * contains both a pointer to a packed 24-bit RGB array, and the
+	 * coordinates.  This method is only used for libart-based canvases.
+	 *
+	 * TODO: figure out where affine transforms and clip paths fit into the
+	 * rendering framework.
+	 */
+	void (* render) (GnomeCanvasItem *item, GnomeCanvasBuf *buf);
+
 	/* Calculate the distance from an item to the specified point.  It also returns a canvas
          * item which is the item itself in the case of the object being an actual leaf item, or a
          * child in case of the object being a canvas group.  (cx, cy) are the canvas pixel
@@ -103,9 +169,14 @@ struct _GnomeCanvasItemClass {
 	 */
 	double (* point) (GnomeCanvasItem *item, double x, double y, int cx, int cy, GnomeCanvasItem **actual_item);
 
+/* FIXME: remove ::translate and ::bounds */
+
 	/* Move an item by the specified amount */
 	void (* translate) (GnomeCanvasItem *item, double dx, double dy);
 
+	/* Fetch the item's bounding box (need not be exactly tight) */
+	void (* bounds) (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2);
+
 	/* Signal: an event ocurred for an item of this type.  The (x, y) coordinates are in the
 	 * canvas world coordinate system.
 	 */
@@ -143,6 +214,15 @@ void gnome_canvas_item_set_valist (GnomeCanvasItem *item, const gchar *first_arg
 /* Move an item by the specified amount */
 void gnome_canvas_item_move (GnomeCanvasItem *item, double dx, double dy);
 
+/* Scale an item about a point by the specified factors */
+void gnome_canvas_item_scale (GnomeCanvasItem *item, double x, double y, double scale_x, double scale_y);
+
+/* Rotate an item about a point by the specified number of degrees */
+void gnome_canvas_item_rotate (GnomeCanvasItem *item, double x, double y, double angle);
+
+/* Apply the specified transformation matrix to an item */
+void gnome_canvas_item_transform (GnomeCanvasItem *item, double *affine);
+
 /* Raise an item in the z-order of its parent group by the specified
  * number of positions.  The specified number must be larger than or
  * equal to 1.
@@ -161,6 +241,12 @@ void gnome_canvas_item_raise_to_top (GnomeCanvasItem *item);
 /* Lower an item to the bottom of its parent group's z-order */
 void gnome_canvas_item_lower_to_bottom (GnomeCanvasItem *item);
 
+/* Show an item (make it visible).  If the item is already shown, it has no effect. */
+void gnome_canvas_item_show (GnomeCanvasItem *item);
+
+/* Hide an item (make it invisible).  If the item is already invisible, it has no effect. */
+void gnome_canvas_item_hide (GnomeCanvasItem *item);
+
 /* Grab the mouse for the specified item.  Only the events in event_mask will be reported.  If
  * cursor is non-NULL, it will be used during the duration of the grab.  Time is a proper X event
  * time parameter.  Returns the same values as XGrabPointer().
@@ -187,6 +273,17 @@ void gnome_canvas_item_i2w (GnomeCanvasItem *item, double *x, double *y);
  */
 void gnome_canvas_item_reparent (GnomeCanvasItem *item, GnomeCanvasGroup *new_group);
 
+/* Used to send all of the keystroke events to a specific item as well as GDK_FOCUS_CHANGE events. */
+void gnome_canvas_item_grab_focus (GnomeCanvasItem *item);
+
+/* Fetch the bounding box of the item.  The bounding box may not be exactly tight, but the canvas
+ * items will do the best they can.
+ */
+void gnome_canvas_item_get_bounds (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2);
+
+/* Request that the update method eventually gets called. */
+void gnome_canvas_item_request_update (GnomeCanvasItem *item);
+
 
 /* GnomeCanvasGroup - a group of canvas items
  *
@@ -270,9 +367,11 @@ struct _GnomeCanvas {
 						 * but not (x2, y2) -- specified in canvas pixel units.
 						 */
 
-	int zoom_xofs, zoom_yofs; 		/* Internal pixel offsets for when zoomed out */
+	ArtUta *redraw_area;			/* Area that needs redrawing, stored as microtiles */
 
-	int width, height;			/* Size of canvas window in pixels */
+	int draw_xofs, draw_yofs;		/* Offsets of the temporary drawing pixmap */
+
+	int zoom_xofs, zoom_yofs; 		/* Internal pixel offsets for when zoomed out */
 
 	int state;				/* Last known modifier state, for deferred repick
 						 * when a button is down
@@ -285,6 +384,8 @@ struct _GnomeCanvas {
 	GnomeCanvasItem *grabbed_item;		/* Item that holds a pointer grab, or NULL if none */
 	guint grabbed_event_mask;		/* Event mask specified when grabbing an item */
 
+	GnomeCanvasItem *focused_item;		/* If non-NULL the currently focused item */
+
 	GdkEvent pick_event;			/* Event on which selection of current item is based */
 
 	int close_enough;			/* Tolerance distance for picking items */
@@ -292,10 +393,13 @@ struct _GnomeCanvas {
 	GdkColorContext *cc;			/* Color context used for color allocation */
 	GdkGC *pixmap_gc;			/* GC for temporary pixmap */
 
-	int need_redraw : 1;			/* Will redraw at next idle loop iteration */
-	int need_repick : 1;			/* Will repick current item at next idle loop iteration */
-	int left_grabbed_item : 1;		/* For use by the internal pick_event function */
-	int in_repick : 1;			/* For use by the internal pick_event function */
+	unsigned int need_update : 1;		/* Will update at next idle loop iteration */
+	unsigned int need_redraw : 1;		/* Will redraw at next idle loop iteration */
+	unsigned int need_repick : 1;		/* Will repick current item at next idle loop iteration */
+	unsigned int left_grabbed_item : 1;	/* For use by the internal pick_event function */
+	unsigned int in_repick : 1;		/* For use by the internal pick_event function */
+
+	unsigned int aa : 1;			/* antialiased rendering */
 };
 
 struct _GnomeCanvasClass {
@@ -329,9 +433,6 @@ void gnome_canvas_get_scroll_region (GnomeCanvas *canvas, double *x1, double *y1
 /* Sets the number of pixels that correspond to one unit in world coordinates */
 void gnome_canvas_set_pixels_per_unit (GnomeCanvas *canvas, double n);
 
-/* Sets the size in pixels of the canvas window */
-void gnome_canvas_set_size (GnomeCanvas *canvas, int width, int height);
-
 /* Scrolls the canvas to the specified offsets, given in canvas pixel coordinates */
 void gnome_canvas_scroll_to (GnomeCanvas *canvas, int cx, int cy);
 
@@ -343,6 +444,12 @@ void gnome_canvas_get_scroll_offsets (GnomeCanvas *canvas, int *cx, int *cy);
 /* Requests that the canvas be repainted immediately instead of in the idle loop. */
 void gnome_canvas_update_now (GnomeCanvas *canvas);
 
+/* For use only by item type implementations. Request that the canvas eventually redraw the
+ * specified region. The region is specified as a microtile array. This function takes over
+ * responsibility for freeing the uta argument.
+ */
+void gnome_canvas_request_redraw_uta (GnomeCanvas *canvas, ArtUta *uta);
+
 /* For use only by item type implementations.  Request that the canvas eventually redraw the
  * specified region.  The region contains (x1, y1) but not (x2, y2).
  */
@@ -353,13 +460,27 @@ void gnome_canvas_request_redraw (GnomeCanvas *canvas, int x1, int y1, int x2, i
  * the upper-left scrolling limit and something else for the lower-left scrolling limit).
  */
 void gnome_canvas_w2c (GnomeCanvas *canvas, double wx, double wy, int *cx, int *cy);
+void gnome_canvas_w2c_d (GnomeCanvas *canvas, double wx, double wy, double *cx, double *cy);
 void gnome_canvas_c2w (GnomeCanvas *canvas, int cx, int cy, double *wx, double *wy);
 
+/* This function takes in coordinates relative to the GTK_LAYOUT (canvas)->bin_window and converts
+ * them to world coordinates.
+ */
+void gnome_canvas_window_to_world (GnomeCanvas *canvas, double winx, double winy, double *worldx, double *worldy);
+
+/* This is the inverse of gnome_canvas_window_to_world */
+void gnome_canvas_world_to_window (GnomeCanvas *canvas, double worldx, double worldy, double *winx, double *winy);
+
 /* Takes a string specification for a color and allocates it into the specified GdkColor.  If the
  * string is null, then it returns FALSE. Otherwise, it returns TRUE.
  */
 int gnome_canvas_get_color (GnomeCanvas *canvas, char *spec, GdkColor *color);
 
+/* Sets the stipple origin of the specified gc so that it will be aligned with all the stipples used
+ * in the specified canvas.  This is intended for use only by canvas item implementations.
+ */
+void gnome_canvas_set_stipple_origin (GnomeCanvas *canvas, GdkGC *gc);
+
 
 END_GNOME_DECLS
 
