@@ -13,7 +13,6 @@
 #include <gtk/gtk.h>
 #include "gnome-dateedit.h"
 #include <libgnome/gnome-i18nP.h>
-#include <libgnomeui/gtkcalendar.h>
 #include <libgnomeui/gnome-stock.h>
 
 
@@ -33,7 +32,11 @@ static void gnome_date_edit_destroy      (GtkObject          *object);
 
 static GtkHBoxClass *parent_class;
 
-
+/**
+ * gnome_date_edit_get_type:
+ *
+ * Returns the GtkType for the GnomeDateEdit widget
+ */
 guint
 gnome_date_edit_get_type (void)
 {
@@ -74,7 +77,7 @@ day_selected (GtkCalendar *calendar, GnomeDateEdit *gde)
 
 	gtk_calendar_get_date (calendar, &year, &month, &day);
 
-	g_snprintf (buffer, 40, "%d/%d/%d", month + 1, day, year); /* FIXME: internationalize this - strftime()*/
+	g_snprintf (buffer, sizeof(buffer), "%d/%d/%d", month + 1, day, year); /* FIXME: internationalize this - strftime()*/
 	gtk_entry_set_text (GTK_ENTRY (gde->date_entry), buffer);
 	gtk_signal_emit (GTK_OBJECT (gde), date_edit_signals [DATE_CHANGED]);
 }
@@ -162,21 +165,31 @@ position_popup (GnomeDateEdit *gde)
 static void
 select_clicked (GtkWidget *widget, GnomeDateEdit *gde)
 {
-	struct tm *mtm;
+	struct tm mtm;
 	GdkCursor *cursor;
 
-	mtm = localtime (&gde->initial_time);
+        /* This code is pretty much just copied from gtk_date_edit_get_date */
+      	sscanf (gtk_entry_get_text (GTK_ENTRY (gde->date_entry)), "%d/%d/%d",
+		&mtm.tm_mon, &mtm.tm_mday, &mtm.tm_year); /* FIXME: internationalize this - strptime()*/
+        
+        mtm.tm_mon--;
 
-	/* FIXME: this would be better if we set the date from what is displayed */
-	gtk_calendar_select_month (GTK_CALENDAR (gde->calendar), mtm->tm_mon, 1900 + mtm->tm_year);
+	/* Hope the user does not actually mean years early in the A.D. days...
+	 * This date widget will obviously not work for a history program :-)
+	 */
+	if (mtm.tm_year >= 1900)
+		mtm.tm_year -= 1900;
 
-	position_popup (gde);
+	gtk_calendar_select_month (GTK_CALENDAR (gde->calendar), mtm.tm_mon, 1900 + mtm.tm_year);
+        gtk_calendar_select_day (GTK_CALENDAR (gde->calendar), mtm.tm_mday);
 
-	gtk_widget_realize (gde->cal_popup);
+        position_popup (gde);
+       
 	gtk_widget_show (gde->cal_popup);
 	gtk_widget_grab_focus (gde->cal_popup);
 	gtk_grab_add (gde->cal_popup);
 
+
 	cursor = gdk_cursor_new (GDK_ARROW);
 
 	gdk_pointer_grab (gde->cal_popup->window, TRUE,
@@ -327,6 +340,14 @@ gnome_date_edit_destroy (GtkObject *object)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
+/**
+ * gnome_date_edit_set_time:
+ * @gde: the GnomeDateEdit widget
+ * @the_time: The time and date that should be set on the widget
+ *
+ * Changes the displayed date and time in the GnomeDateEdit widget
+ * to be the one represented by @the_time.
+ */
 void
 gnome_date_edit_set_time (GnomeDateEdit *gde, time_t the_time)
 {
@@ -342,7 +363,10 @@ gnome_date_edit_set_time (GnomeDateEdit *gde, time_t the_time)
 	mytm = localtime (&the_time);
 
 	/* Set the date */
-	sprintf (buffer, "%d/%d/%d", mytm->tm_mon + 1, mytm->tm_mday, 1900 + mytm->tm_year);
+	g_snprintf (buffer, sizeof(buffer), "%d/%d/%d",
+		    mytm->tm_mon + 1,
+		    mytm->tm_mday,
+		    1900 + mytm->tm_year);
 	gtk_entry_set_text (GTK_ENTRY (gde->date_entry), buffer);
 
 	if (gde->time_entry) {
@@ -355,6 +379,15 @@ gnome_date_edit_set_time (GnomeDateEdit *gde, time_t the_time)
 	}
 }
 
+/**
+ * gnome_date_edit_set_popup_range:
+ * @gde: The GnomeDateEdit widget
+ * @low_hour: low boundary for the time-range display popup.
+ * @up_hour:  upper boundary for the time-range display popup.
+ *
+ * Sets the range of times that will be provide by the time popup
+ * selectors.
+ */
 void
 gnome_date_edit_set_popup_range (GnomeDateEdit *gde, int low_hour, int up_hour)
 {
@@ -437,16 +470,27 @@ create_children (GnomeDateEdit *gde, int show_time)
 	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
 	gtk_container_add (GTK_CONTAINER (gde->cal_popup), frame);
 	gtk_widget_show (frame);
-
+       
 	gde->calendar = gtk_calendar_new ();
 	gtk_calendar_display_options (GTK_CALENDAR (gde->calendar),
 				      GTK_CALENDAR_SHOW_DAY_NAMES | GTK_CALENDAR_SHOW_HEADING);
 	gtk_signal_connect (GTK_OBJECT (gde->calendar), "day_selected",
 			    GTK_SIGNAL_FUNC (day_selected), gde);
 	gtk_container_add (GTK_CONTAINER (frame), gde->calendar);
-	gtk_widget_show (gde->calendar);
+        gtk_widget_show (gde->calendar);
 }
 
+/**
+ * gnome_date_edit_new:
+ * @the_time: date and time to be displayed on the widget
+ * @show_time: whether time should be displayed
+ * @use_24_format: whether 24-hour format is desired for the time display.
+ *
+ * Creates a new GnomeDateEdit widget which can be used to provide
+ * an easy to use way for entering dates and times.
+ * 
+ * Returns a GnomeDateEdit widget.
+ */
 GtkWidget *
 gnome_date_edit_new (time_t the_time, int show_time, int use_24_format)
 {
@@ -461,6 +505,12 @@ gnome_date_edit_new (time_t the_time, int show_time, int use_24_format)
 	return GTK_WIDGET (gde);
 }
 
+/**
+ * gnome_date_edit_get_date:
+ * @gde: The GnomeDateEdit widget
+ *
+ * Returns the time entered in the GnomeDateEdit widget
+ */
 time_t
 gnome_date_edit_get_date (GnomeDateEdit *gde)
 {
@@ -503,3 +553,9 @@ gnome_date_edit_get_date (GnomeDateEdit *gde)
 
 	return mktime (&tm);
 }
+
+
+
+
+
+
