@@ -12,8 +12,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
  */
 
 #include <config.h>
@@ -51,8 +51,10 @@ static void gnome_dialog_marshal_signal_2 (GtkObject         *object,
 					   gpointer           func_data,
 					   GtkArg            *args);
 
-static void gnome_dialog_class_init (GnomeDialogClass *klass);
-static void gnome_dialog_init       (GnomeDialog      *messagebox);
+static void gnome_dialog_class_init       (GnomeDialogClass *klass);
+static void gnome_dialog_init             (GnomeDialog * dialog);
+static void gnome_dialog_init_action_area (GnomeDialog * dialog);
+
 
 static void gnome_dialog_button_clicked (GtkWidget   *button, 
 					 GtkWidget   *messagebox);
@@ -162,20 +164,17 @@ static void
 gnome_dialog_init (GnomeDialog *dialog)
 {
   GtkWidget * vbox;
-  GtkWidget * separator;
   GtkWidget * bf;
 
   g_return_if_fail(dialog != NULL);
   g_return_if_fail(GNOME_IS_DIALOG(dialog));
 
-  dialog->modal = FALSE;
   dialog->just_hide = FALSE;
   dialog->click_closes = FALSE;
   dialog->buttons = NULL;
-  dialog->parent = NULL;
 
   GTK_WINDOW(dialog)->type = gnome_preferences_get_dialog_type();
-  gtk_window_position(GTK_WINDOW(dialog), 
+  gtk_window_set_position(GTK_WINDOW(dialog), 
 		      gnome_preferences_get_dialog_position());
   
   dialog->accelerators = gtk_accel_group_new();
@@ -188,7 +187,7 @@ gnome_dialog_init (GnomeDialog *dialog)
   gtk_widget_show(bf);
 
   vbox = gtk_vbox_new(FALSE, GNOME_PAD);
-  gtk_container_border_width (GTK_CONTAINER (vbox), 
+  gtk_container_set_border_width (GTK_CONTAINER (vbox), 
 			      GNOME_PAD_SMALL);
   gtk_container_add(GTK_CONTAINER(bf), vbox);
   gtk_widget_show(vbox);
@@ -201,12 +200,15 @@ gnome_dialog_init (GnomeDialog *dialog)
 		      TRUE, TRUE,
 		      GNOME_PAD_SMALL);
   gtk_widget_show(dialog->vbox);
+}
 
-  separator = gtk_hseparator_new ();
-  gtk_box_pack_start (GTK_BOX (vbox), separator, 
-		      FALSE, TRUE,
-		      GNOME_PAD_SMALL);
-  gtk_widget_show (separator);
+static void
+gnome_dialog_init_action_area (GnomeDialog * dialog)
+{
+  GtkWidget * separator;
+
+  if (dialog->action_area)
+    return;
 
   dialog->action_area = gtk_hbutton_box_new ();
   gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog->action_area),
@@ -215,9 +217,15 @@ gnome_dialog_init (GnomeDialog *dialog)
   gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog->action_area), 
 			      GNOME_PAD);
 
-  gtk_box_pack_start (GTK_BOX (vbox), dialog->action_area, 
-		      FALSE, TRUE, 0);
+  gtk_box_pack_end (GTK_BOX (dialog->vbox), dialog->action_area, 
+		    FALSE, TRUE, 0);
   gtk_widget_show (dialog->action_area);
+
+  separator = gtk_hseparator_new ();
+  gtk_box_pack_end (GTK_BOX (dialog->vbox), separator, 
+		      FALSE, TRUE,
+		      GNOME_PAD_SMALL);
+  gtk_widget_show (separator);
 }
 
 void       
@@ -281,6 +289,10 @@ GtkWidget* gnome_dialog_new            (const gchar * title,
 
   va_end(ap);
 
+  /* argument list may be null if the user wants to do weird things to the
+   * dialog, but we need to make sure this is initialized */
+  gnome_dialog_init_action_area(dialog);
+  
   return GTK_WIDGET (dialog);
 }
 
@@ -304,7 +316,7 @@ void       gnome_dialog_set_parent     (GnomeDialog * dialog,
   g_return_if_fail(parent != NULL);
   g_return_if_fail(GTK_IS_WINDOW(parent));
 
-  dialog->parent = parent;
+  gtk_window_set_transient_for (GTK_WINDOW(dialog), parent);
 
   if ( gnome_preferences_get_dialog_centered() ) {
 
@@ -316,7 +328,7 @@ void       gnome_dialog_set_parent     (GnomeDialog * dialog,
 						  size/pos */
 
     /* Throw out other positioning */
-    gtk_window_position(GTK_WINDOW(dialog),GTK_WIN_POS_NONE);
+    gtk_window_set_position(GTK_WINDOW(dialog),GTK_WIN_POS_NONE);
 
     gdk_window_get_origin (GTK_WIDGET(parent)->window, &x, &y);
     gdk_window_get_size   (GTK_WIDGET(parent)->window, &w, &h);
@@ -360,11 +372,48 @@ void       gnome_dialog_append_button (GnomeDialog * dialog,
 
   if (button_name != NULL) {
     GtkWidget *button;
-    
+
+    gnome_dialog_init_action_area (dialog);    
+
     button = gnome_stock_or_ordinary_button (button_name);
-    gnome_button_can_default (GTK_BUTTON(button), TRUE);
-    gtk_container_add (GTK_CONTAINER(dialog->action_area), 
-		       button);
+    GTK_WIDGET_SET_FLAGS (GTK_WIDGET (button), GTK_CAN_DEFAULT);
+    gtk_box_pack_start (GTK_BOX (dialog->action_area), button, TRUE, TRUE, 0);
+
+    gtk_widget_grab_default (button);
+    gtk_widget_show (button);
+    
+    gtk_signal_connect_after (GTK_OBJECT (button), "clicked",
+			      (GtkSignalFunc) gnome_dialog_button_clicked,
+			      dialog);
+    
+    dialog->buttons = g_list_append (dialog->buttons, button);
+  }
+}
+
+void       gnome_dialog_append_button_with_pixmap (GnomeDialog * dialog,
+						   const gchar * button_name,
+						   const gchar * pixmap_name)
+{
+  g_return_if_fail(dialog != NULL);
+  g_return_if_fail(GNOME_IS_DIALOG(dialog));
+
+  if (button_name != NULL) {
+    GtkWidget *button;
+
+    if (pixmap_name != NULL) {
+      GtkWidget *pixmap;
+ 
+      pixmap = gnome_stock_new_with_icon (pixmap_name);
+      button = gnome_pixmap_button (pixmap, button_name);    
+    } else {
+      button = gnome_stock_or_ordinary_button (button_name);
+    }
+
+    gnome_dialog_init_action_area (dialog);    
+
+    GTK_WIDGET_SET_FLAGS (GTK_WIDGET (button), GTK_CAN_DEFAULT);
+    gtk_box_pack_start (GTK_BOX (dialog->action_area), button, TRUE, TRUE, 0);
+
     gtk_widget_grab_default (button);
     gtk_widget_show (button);
     
@@ -388,23 +437,43 @@ void       gnome_dialog_append_buttonsv (GnomeDialog * dialog,
   }
 }
 
-void
-gnome_dialog_set_modal (GnomeDialog * dialog)
+void       gnome_dialog_append_buttons_with_pixmaps (GnomeDialog * dialog,
+						     const gchar **names,
+						     const gchar **pixmaps)
 {
   g_return_if_fail(dialog != NULL);
   g_return_if_fail(GNOME_IS_DIALOG(dialog));
 
-  dialog->modal = TRUE;
-
-  /* This is needed if someone shows before calling the function. */
-  if (GTK_WIDGET_VISIBLE(GTK_WIDGET(dialog))) gtk_grab_add(GTK_WIDGET(dialog));
+  while(*names != NULL) {
+    gnome_dialog_append_button_with_pixmap (dialog, *names, *pixmaps);
+    names++; pixmaps++;
+  }
 }
 
 struct GnomeDialogRunInfo {
   gint button_number;
-  gint close_id, clicked_id;
+  gint close_id, clicked_id, destroy_id;
+  gboolean destroyed;
 };
 
+static void
+gnome_dialog_shutdown_run(GnomeDialog* dialog,
+                          struct GnomeDialogRunInfo* runinfo)
+{
+  if (!runinfo->destroyed) 
+    {
+      
+      gtk_signal_disconnect(GTK_OBJECT(dialog),
+                            runinfo->close_id);
+      gtk_signal_disconnect(GTK_OBJECT(dialog),
+                            runinfo->clicked_id);
+  
+      runinfo->close_id = runinfo->clicked_id = -1;
+    }
+
+  gtk_main_quit();
+}
+
 static void
 gnome_dialog_setbutton_callback(GnomeDialog *dialog,
 				gint button_number,
@@ -415,14 +484,7 @@ gnome_dialog_setbutton_callback(GnomeDialog *dialog,
 
   runinfo->button_number = button_number;
 
-  gtk_signal_disconnect(GTK_OBJECT(dialog),
-			runinfo->close_id);
-  gtk_signal_disconnect(GTK_OBJECT(dialog),
-			runinfo->clicked_id);
-
-  runinfo->close_id = runinfo->clicked_id = -1;
-
-  gtk_main_quit();
+  gnome_dialog_shutdown_run(dialog, runinfo);
 }
 
 static gboolean
@@ -432,40 +494,36 @@ gnome_dialog_quit_run(GnomeDialog *dialog,
   if(runinfo->close_id < 0)
     return FALSE;
 
-  gtk_signal_disconnect(GTK_OBJECT(dialog),
-			runinfo->close_id);
-  gtk_signal_disconnect(GTK_OBJECT(dialog),
-			runinfo->clicked_id);
-
-  runinfo->close_id = runinfo->clicked_id = -1;
-
-  gtk_main_quit();
+  gnome_dialog_shutdown_run(dialog, runinfo);
 
   return FALSE;
 }
 
-gint
-gnome_dialog_run_modal(GnomeDialog *dialog)
+static void
+gnome_dialog_mark_destroy(GnomeDialog* dialog,
+                          struct GnomeDialogRunInfo* runinfo)
 {
-  g_return_val_if_fail(dialog != NULL, -1);
-  g_return_val_if_fail(GNOME_IS_DIALOG(dialog), -1);
+  runinfo->destroyed = TRUE;
 
-  gnome_dialog_set_modal(dialog);
-  return gnome_dialog_run(dialog);
+  if(runinfo->close_id < 0)
+    return;
+  else gnome_dialog_shutdown_run(dialog, runinfo);
 }
 
-gint
-gnome_dialog_run(GnomeDialog *dialog)
+static gint
+gnome_dialog_run_real(GnomeDialog* dialog, gboolean close_after)
 {
-  gboolean was_modal, was_only_hidden;
-  struct GnomeDialogRunInfo ri = {-1,-1,-1};
+  gboolean was_modal;
+  struct GnomeDialogRunInfo ri = {-1,-1,-1,-1,FALSE};
 
   g_return_val_if_fail(dialog != NULL, -1);
   g_return_val_if_fail(GNOME_IS_DIALOG(dialog), -1);
 
-  was_only_hidden = dialog->just_hide || !dialog->click_closes;
-  was_modal = dialog->modal;
-  if (!was_modal) gnome_dialog_set_modal(dialog);
+  was_modal = GTK_WINDOW(dialog)->modal;
+  if (!was_modal) gtk_window_set_modal(GTK_WINDOW(dialog),TRUE);
+
+  /* There are several things that could happen to the dialog, and we
+     need to handle them all: click, delete_event, close, destroy */
 
   ri.clicked_id =
     gtk_signal_connect(GTK_OBJECT(dialog), "clicked",
@@ -477,65 +535,50 @@ gnome_dialog_run(GnomeDialog *dialog)
 		       GTK_SIGNAL_FUNC(gnome_dialog_quit_run),
 		       &ri);
 
+  ri.destroy_id = 
+    gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
+                       GTK_SIGNAL_FUNC(gnome_dialog_mark_destroy),
+                       &ri);
+
   if ( ! GTK_WIDGET_VISIBLE(GTK_WIDGET(dialog)) )
     gtk_widget_show(GTK_WIDGET(dialog));
 
   gtk_main();
 
-  if(was_only_hidden) {
+  if(!ri.destroyed) {
+
+    gtk_signal_disconnect(GTK_OBJECT(dialog), ri.destroy_id);
+
     if(!was_modal)
       {
-	/* It would be nicer if gnome_dialog_set_modal() took a
-	   bool value to indicate yes/no modal */
-	dialog->modal = FALSE;
-	gtk_grab_remove(GTK_WIDGET(dialog));
+	gtk_window_set_modal(GTK_WINDOW(dialog),FALSE);
       }
     
-    if(ri.close_id >= 0)
+    if(ri.close_id >= 0) /* We didn't shut down the run? */
       {
 	gtk_signal_disconnect(GTK_OBJECT(dialog), ri.close_id);
 	gtk_signal_disconnect(GTK_OBJECT(dialog), ri.clicked_id);
       }
+
+    if (close_after)
+      {
+        gnome_dialog_close(dialog);
+      }
   }
 
   return ri.button_number;
 }
 
 gint
-gnome_dialog_run_and_hide (GnomeDialog * dialog)
+gnome_dialog_run(GnomeDialog *dialog)
 {
-  gint r;
-
-  g_return_val_if_fail(dialog != NULL, -1);
-  g_return_val_if_fail(GNOME_IS_DIALOG(dialog), -1);  
-
-  gnome_dialog_close_hides(dialog, TRUE);
-
-  r = gnome_dialog_run(dialog);
-
-  gnome_dialog_close(dialog); /* Causes a hide. */
-  return r;
+  return gnome_dialog_run_real(dialog,FALSE);
 }
 
 gint 
-gnome_dialog_run_and_destroy (GnomeDialog * dialog)
+gnome_dialog_run_and_close(GnomeDialog* dialog)
 {
-  gint r;
-  gboolean need_close;
-
-  g_return_val_if_fail(dialog != NULL, -1);
-  g_return_val_if_fail(GNOME_IS_DIALOG(dialog), -1);  
-
-  need_close = dialog->click_closes;
-
-  gnome_dialog_close_hides(dialog, FALSE);
-
-  r = gnome_dialog_run(dialog);
-
-  if(need_close)
-    gnome_dialog_close(dialog); /* Causes a destroy. */
-
-  return r;
+  return gnome_dialog_run_real(dialog,TRUE);
 }
 
 void
@@ -568,13 +611,6 @@ void       gnome_dialog_set_close    (GnomeDialog * dialog,
   dialog->click_closes = click_closes;
 }
 
-void       gnome_dialog_set_destroy (GnomeDialog * d, gboolean self_destruct)
-{
-  g_warning("gnome_dialog_set_destroy() is deprecated.\n");
-
-  gnome_dialog_set_close(d, self_destruct);
-}
-
 void       gnome_dialog_close_hides (GnomeDialog * dialog, 
 				     gboolean just_hide)
 {
@@ -773,11 +809,6 @@ void gnome_dialog_close_real(GnomeDialog * dialog)
 
   gtk_widget_hide(GTK_WIDGET(dialog));
 
-  if ( dialog->modal ) 
-    gtk_grab_remove(GTK_WIDGET(dialog)); /* Otherwise the hidden
-					    widget would still have
-					    the grab - very weird. */
-
   if ( ! dialog->just_hide ) {
     gtk_widget_destroy (GTK_WIDGET (dialog));
   }
@@ -799,263 +830,8 @@ void gnome_dialog_close(GnomeDialog * dialog)
 }
 
 static void gnome_dialog_show (GtkWidget * d)
-{
-  /* _set_modal() could always do this on the first _show(), but
-     if the dialog is hidden then shown it wouldn't work. Thus this
-     function. */
-  
-  if (GNOME_DIALOG(d)->modal) gtk_grab_add (d);
-  
+{  
   if (GTK_WIDGET_CLASS(parent_class)->show)
     (* (GTK_WIDGET_CLASS(parent_class)->show))(d);
 }
 
-
-/****************************************************************
-  $Log: gnome-dialog.c,v $
-  Revision 1.39  1998/09/18 23:25:39  unammx
-  Popup menus work.  The API is very simple, and I think it is enough
-  for most purposes -- if language binding writers think there is
-  additional stuff that could be done to them, please feel free to add
-  it.
-
-  I am writing some DocBook documentation for GnomeAppHelper and popup
-  menus; hopefully it will be finished during the weekend.  People
-  should add underlined letters and accelerators to their menus as soon
-  as possible -- the arrangement of the menus and accelerators still
-  needs to be settled in the UI Guidelines, but that's another story :-)
-
-    Federico
-
-  1998-09-18  Federico Mena Quintero  <federico@nuclecu.unam.mx>
-
-  	* configure.in: Removed generation of list of canvas files.
-
-  1998-09-18  Federico Mena Quintero  <federico@nuclecu.unam.mx>
-
-  	* gnome-popup-menu.c (gnome_popup_menu_new): New public function
-  	to create a popup menu from a GnomeUIInfo array.
-  	(gnome_popup_menu_attach): New public function to attach a popup
-  	menu to a widget -- the popup menu can be called by pressing mouse
-  	button 3 over the widget.  The menu can be shared between multiple
-  	widgets and it will be destroyed when the last widget it is
-  	attached to is destroyed.
-  	(gnome_popup_menu_do_popup): New public function to explicitly
-  	popup a menu.
-
-  	* gnome-app-helper.c (do_ui_signal_connect): Use
-  	gtk_signal_connect_full() instead of gtk_signal_connect_interp(),
-  	since the latter is obsolete (but works anyway...).
-
-  	* stock_demo.c:
-  	* gnome-stock.c:
-  	* gnome-number-entry.c:
-  	* gnome-init.c:
-  	* gnome-dialog.h:
-  	* gnome-dialog.c:
-  	* gnome-dentry-edit.c:
-  	* gnome-client.c (gnome_client_init):
-  	* gnome-calculator.c:
-  	* gnome-app.c:
-  	* libgnomeui.h: Removed occurrences of #ifdef GTK_HAVE_FEATURES_1_1_0.
-  	We require it, anyways.
-
-  	* Makefile.am: Moved all the conditional canvas files into the
-  	main list of sources -- they are always compiled now.
-
-  1998-09-18  Federico Mena Quintero  <federico@nuclecu.unam.mx>
-
-  	* testgnome.c (create_app_helper): Added tests for popup menus.
-
-  	* canvas-fifteen.c:
-  	* canvas-arrowhead.c:
-  	* canvas-primitives.c: Removed occurrences of #ifdef GTK_HAVE_FEATURES_1_1_0.
-
-  Revision 1.38  1998/08/16 04:11:11  sopwith
-
-
-  libgnomeui:
-  	gnome-dialog.c: gnome_dialog_run_and_destroy() doesn't try to
-  			destroy the dialog if it already was destroyed by
-  			the click. (Havoc, can you review please?)
-  	gnome-preferences.c: Remove memory leaks. Some still
-  		             supposedly exist in the
-  			     gnome_config_get_boolean() calls, but I
-  			     couldn't see them.
-
-  			     Please try to do smart things with return
-  			     values :-)
-  	gnome.defs, gnometypebuiltins_evals.c:
-  		I didn't make these changes, but these files have diffs.
-  		Did someone put autogenerated files into CVS?
-
-  Revision 1.37  1998/08/05 15:17:09  owen
-  Wed Aug  5 11:19:56 1998  Owen Taylor  <otaylor@redhat.com>
-
-  	* gnome-dialog.c (gnome_dialog_destroy): Don't remove
-  	  the accelerator group - GTK+ handles it for us.
-
-  	* gnome-app.c gnome-app-helper.c: Added gtk-1.1 code to check
-  	  if AccelGroup is already attached before attaching it
-  	  again.
-
-  Revision 1.36  1998/07/27 14:37:10  pavlov
-  replaced GTK_HAVE_ACCEL_GROUP with GTK_HAVE_FEATURES_1_1_0
-
-  This should allow it to compile correctly with the new gtk 1.1.0 that timj
-  broke changed on us ;)
-
-  -pav
-
-  Revision 1.35  1998/07/25 03:07:40  hp
-
-
-  Fri Jul 24 19:54:43 1998  Havoc Pennington  <hp@pobox.com>
-
-          * gnome-scores.c (gnome_scores_new): Use construct to set
-          the title and buttons.
-
-          * gnome-app-util.c: Switch to parented dialogs.
-
-  	* gnome-dialog-util.h, gnome-dialog-util.c: Added "parented"
-  	versions of these. The old versions are not really necessary
-  	anymore because you can just pass NULL for parent, but they
-  	save typing I guess.
-
-  	* gnome-preferences.h: New functions for dialog position, whether
-  	to attempt dialog-over-parent centering, whether dialogs are
-  	TOPLEVEL or DIALOG.
-
-  	* gnome-dialog.c (gnome_dialog_newv): Remove unused variable.
-  	(gnome_dialog_set_parent): New function, allows setting a parent
-  	the dialog should be centered over.
-
-  Sat Jul 25 02:51:12 1998  Simon Kagedal  <simon@sdf.se>
-
-          * gnome-scores.c, gnome-scores.h:
-          made the GnomeScore dialog inherit from GnomeDialog
-          instead of GtkWindow
-
-  Revision 1.34  1998/07/18 17:53:31  hp
-
-
-  Um, someone broke every dialog in Gnome without so much as a change log
-  entry. However, the vector constructor is nice.
-
-  Sat Jul 18 12:42:09 1998  Havoc Pennington  <hp@pobox.com>
-
-  * gnome-dialog.c (gnome_dialog_constructv): Use append_button,
-  not append_buttons without NULL termination.
-  (gnome_dialog_construct): Same.
-  (gnome_dialog_new): Reverted last change. Use the construct
-  function rather than cut-and-pasting it.
-
-  Revision 1.33  1998/07/17 23:37:03  yacc
-  provide ...v-forms of functions using varargs.
-
-  Revision 1.32  1998/07/15 03:59:15  hp
-
-
-  Tue Jul 14 22:47:33 1998  Havoc Pennington  <hp@pobox.com>
-
-  * gnome-app.h, gnome-app.c, gnome-dialog.h, gnome-dialog.c:
-  Added "construct" functions. The gnome-dialog one takes a va_list
-  which is kind of weird; if there's a good alternative I'd like
-  to use it instead.
-
-  Revision 1.31  1998/07/10 08:53:22  timj
-  Fri Jul 10 10:19:38 1998  Tim Janik  <timj@gtk.org>
-
-          * gnome-app-helper.c (gnome_app_do_menu_creation):
-          * gnome-stock.c (gnome_stock_menu_item):
-          create GtkAccelLabel if GTK_HAVE_FEATURES_1_1_0, so accelerators are
-          visible.
-
-  Revision 1.30  1998/07/03 00:51:57  hp
-
-
-  This fixes a couple of bugs in my Sunday commits, but breaks things: namely,
-  your handlebox preferences if you've set them, and gnome_dialog_run_and_die
-  is renamed if anyone had used it. Sorry it didn't get in on Monday, all that
-  CVS confusion.
-
-  Mon Jun 29 14:10:48 1998  Havoc Pennington  <hp@pobox.com>
-
-  * gnome-preferences.h, gnome-preferences.c
-  (gnome_preferences_get_toolbar_relief,
-  gnome_preferences_set_toolbar_relief): Whether the toolbar buttons
-  have the beveled edge.
-  * gnome-app.c (gnome_app_set_toolbar): Turn off the toolbar button
-  relief if user requested it.
-  * gnome-preferences.c (gnome_preferences_load): Oops, forgot to
-  push a new prefix for the GnomeApp stuff, it was being saved under
-  Dialog prefs.
-  * gnome-dialog.h, gnome-dialog.c (gnome_dialog_run_and_die):
-  Renamed to run_and_destroy - run_and_die was cute at 2 am, but
-  probably not a good name. ;-)
-
-  Revision 1.29  1998/06/29 08:06:27  hp
-
-
-  I think Elliot and I talked about the dialog changes this time, and there
-  are now two functions to make both of us happy. The other change allows
-  turning off the handlebox for the menubar and toolbar, I think that was
-  mentioned recently.
-
-  Mon Jun 29 03:04:30 1998  Havoc Pennington  <hp@pobox.com>
-
-  * TODO: Updated a little.
-
-  Mon Jun 29 02:58:15 1998  Havoc Pennington  <hp@pobox.com>
-
-  * gnome-preferences.h, gnome-preferences.c
-  (gnome_preferences_get_toolbar_handlebox,
-  gnome_preferences_set_toolbar_handlebox,
-  gnome_preferences_get_menubar_handlebox,
-  gnome_preferences_set_menubar_handlebox): New functions.
-
-  * gnome-app.c (gnome_app_set_menus): Obey preferences for
-  handlebox.
-  (gnome_app_set_toolbar): Obey handlebox prefs.
-
-  Mon Jun 29 02:52:10 1998  Havoc Pennington  <hp@pobox.com>
-
-  * gnome-dialog.h, gnome-dialog.c (gnome_dialog_run): Take out the
-  hide stuff.
-  (gnome_dialog_run_and_hide): Do what gnome_dialog_run used to do.
-  (gnome_dialog_run_and_die):  Run and then destroy the dialog.
-  (gnome_dialog_button_connect_object): Someone requested this.
-
-  Revision 1.28  1998/06/11 02:26:50  yosh
-  changed things to use GTK_HAVE_FEATURES_1_1_0 instead of HAVE_DEVGTK...
-  installed headers depending on config.h stuff is bad.
-
-  -Yosh
-
-  Revision 1.27  1998/06/10 17:15:25  gregm
-  Wed Jun 10 13:07:09 EDT 1998 Gregory McLean <gregm@comstar.net>
-
-          * Wheee libgnomeui now compiles (with the exception of the
-            canvas stuff) under 1.0.x again. It also compiles under 1.1
-            for thoose of you that like to bleed. Please please if you
-            add code that _requires_ gtk 1.1 shield it with GTK_HAVE_FEATURES_1_1_0
-            so us boring folks can continue to get stuff done.
-
-  Revision 1.26  1998/06/07 17:58:21  pavlov
-  updates to make gnome-libs compile with new gtk1.1
-
-  i havn't tested this yet, other than knowing it compiles.  i will in a minute,
-  but my gtk is fooed at the moment, and button's arn't showing labels :)
-
-  -pav
-
-  Revision 1.25  1998/05/25 16:31:18  sopwith
-  Move log msgs to bottom
-
-  Revision 1.24  1998/05/25 16:15:57  sopwith
-
-
-  gnome_dialog_run_*(): Fixes for Havoc's bugs
-  Also added CVS log recording at the top :)
-
-*****************************************************************/
