@@ -11,8 +11,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
  */
 #ifndef __GNOME_DIALOG_H__
 #define __GNOME_DIALOG_H__
@@ -48,11 +48,9 @@ struct _GnomeDialog
   GList *buttons;
   GtkAccelGroup * accelerators;
 
-  int modal : 1;
-  int click_closes : 1;
-  int just_hide : 1;
+  unsigned int click_closes : 1;
+  unsigned int just_hide : 1;
 
-  GtkWindow * parent;
 };
 
 struct _GnomeDialogClass
@@ -97,19 +95,12 @@ void       gnome_dialog_button_connect_object (GnomeDialog *dialog,
 					       GtkSignalFunc callback,
 					       GtkObject * obj);
 
-/* Make the dialog modal */
-void       gnome_dialog_set_modal      (GnomeDialog *dialog);
-
 /* Run the dialog, return the button # that was pressed or -1 if none.
+   (this sets the dialog modal while it blocks)
  */
 gint       gnome_dialog_run	       (GnomeDialog *dialog);
+gint       gnome_dialog_run_and_close  (GnomeDialog *dialog);
 
-/* Run the dialog with gnome_dialog_run, and also call set_modal */
-gint       gnome_dialog_run_modal      (GnomeDialog *dialog);
-/* Run the dialog, and hide it afterwards */
-gint       gnome_dialog_run_and_hide   (GnomeDialog * dialog);
-/* Run the dialog, and destroy it afterwards */
-gint       gnome_dialog_run_and_destroy(GnomeDialog * dialog);
 
 /* Set the default button. - it will have a little highlight, 
    and pressing return will activate it. */
@@ -151,10 +142,6 @@ void       gnome_dialog_set_close      (GnomeDialog * dialog,
 void       gnome_dialog_editable_enters   (GnomeDialog * dialog,
 					   GtkEditable * editable);
 
-/* *** Deprecated. Is a set_close wrapper now. */
-void       gnome_dialog_set_destroy (GnomeDialog * d, gboolean self_destruct);
-
-
 /* Use of append_buttons is discouraged, it's really
    meant for subclasses. */
 void       gnome_dialog_append_buttons  (GnomeDialog * dialog,
@@ -165,6 +152,14 @@ void       gnome_dialog_append_button   (GnomeDialog * dialog,
 void       gnome_dialog_append_buttonsv (GnomeDialog * dialog,
 					 const gchar **buttons);
 
+/* Add button with arbitrary text and pixmap. */
+void       gnome_dialog_append_button_with_pixmap (GnomeDialog * dialog,
+						   const gchar * name,
+						   const gchar * pixmap);
+void       gnome_dialog_append_buttons_with_pixmaps (GnomeDialog * dialog,
+						     const gchar **names,
+						     const gchar **pixmaps);
+
 /* Don't use this either; it's for bindings to languages other 
    than C (which makes the varargs kind of lame... feel free to fix)
    You want _new, see above. */
