@@ -13,8 +13,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
  */
 
 #include <config.h>
@@ -22,27 +22,6 @@
 #include "libgnome/gnome-config.h"
 #include <string.h>
 
-typedef struct _GnomePreferences GnomePreferences;
-
-struct _GnomePreferences {
-  GtkButtonBoxStyle dialog_buttons_style;
-  int property_box_buttons_ok : 1;
-  int property_box_buttons_apply : 1;
-  int property_box_buttons_close : 1;
-  int property_box_buttons_help : 1;
-  int statusbar_not_dialog : 1;
-  int statusbar_is_interactive : 1;
-  int toolbar_handlebox : 1;
-  int menubar_handlebox : 1;
-  int toolbar_relief : 1;
-  int dialog_centered : 1;
-  int menus_have_icons : 1;
-  GtkWindowType dialog_type;
-  GtkWindowPosition dialog_position;
-  GnomeMDIMode mdi_mode;
-  GtkPositionType mdi_tab_pos;
-};
-
 /* 
  * Variable holds current preferences.  
  */
@@ -57,13 +36,18 @@ static GnomePreferences prefs =
   TRUE,               /* PropertyBox has Help */
   FALSE,              /* Use dialogs, not the statusbar */
   FALSE,              /* Statusbar isn't interactive */
-  TRUE,               /* Toolbar has handlebox */
-  TRUE,               /* Menubar has handlebox */
-  TRUE,               /* Toolbar buttons are relieved */
+  TRUE,               /* Menubars are detachable */
+  TRUE,               /* Menubars are relieved */
+  TRUE,               /* Toolbars are detachable */
+  TRUE,               /* Toolbars are relieved */
+  FALSE,              /* Toolbar buttons are relieved */
+  TRUE,               /* Toolbars show lines for separators */
+  TRUE,               /* Toolbars show labels */
   TRUE,               /* Center dialogs over apps when possible */
   TRUE,               /* Menu items have icons in them */
+  TRUE,               /* Disable the Imlib cache */
   GTK_WINDOW_DIALOG,  /* Dialogs are treated specially */
-  GTK_WIN_POS_MOUSE,  /* Put dialogs at mouse pointer. */
+  GTK_WIN_POS_CENTER, /* Put dialogs in center of screen. */
   GNOME_MDI_NOTEBOOK, /* Use notebook MDI mode. */
   GTK_POS_TOP         /* Show tabs on top of MDI notebooks. */
 };
@@ -79,6 +63,7 @@ static GnomePreferences prefs =
 #define STATUSBAR "/Gnome/UI_StatusBar/"
 #define APP       "/Gnome/UI_GnomeApp/"
 #define MDI       "/Gnome/UI_MDI/"
+#define CACHE     "/Gnome/Cache/"
 
 /* ==================== GnomeDialog ===================== */
 
@@ -131,12 +116,17 @@ static const gchar * const dialog_positions [] = {
 #define STATUSBAR_DIALOG_KEY       "StatusBar_not_Dialog"
 #define STATUSBAR_INTERACTIVE_KEY  "StatusBar_is_Interactive"
 
-#define TOOLBAR_HANDLEBOX_KEY      "Toolbar_has_Handlebox"
-#define MENUBAR_HANDLEBOX_KEY      "Menubar_has_Handlebox"
+#define MENUBAR_DETACHABLE_KEY     "Menubar_detachable"
+#define MENUBAR_RELIEF_KEY         "Menubar_relieved"
 
-#define TOOLBAR_RELIEF_KEY         "Toolbar_relieved_buttons"
+#define TOOLBAR_DETACHABLE_KEY     "Toolbar_detachable"
+#define TOOLBAR_RELIEF_KEY         "Toolbar_relieved"
+#define TOOLBAR_RELIEF_BTN_KEY     "Toolbar_relieved_buttons"
+#define TOOLBAR_LINES_KEY          "Toolbar_lines"
+#define TOOLBAR_LABELS_KEY         "Toolbar_labels"
 
 #define MENUS_HAVE_ICONS_KEY       "Menus_have_icons"
+#define DISABLE_IMLIB_CACHE_KEY    "Disable_Imlib_cache"
 
 /* =========== MDI ================================= */
 
@@ -184,7 +174,8 @@ enum_from_strings(gint * setme, gchar * findme,
   return retval;
 }
 
-void gnome_preferences_load(void)
+void
+gnome_preferences_load_custom(GnomePreferences *settings)
 {
   /* Probably this function should be rewritten to use the 
    *  _preferences_get functions
@@ -197,7 +188,7 @@ void gnome_preferences_load(void)
   s = gnome_config_get_string(DIALOG_BUTTONS_STYLE_KEY);
 
 
-  if ( ! enum_from_strings((int*) &prefs.dialog_buttons_style, s,
+  if ( ! enum_from_strings((int*) &settings->dialog_buttons_style, s,
 			   dialog_button_styles, NUM_BUTTON_STYLES) ) {
     g_warning("Didn't recognize buttonbox style in libgnomeui config");
   }
@@ -206,7 +197,7 @@ void gnome_preferences_load(void)
   
   s = gnome_config_get_string(DIALOG_TYPE_KEY);
 
-  if ( ! enum_from_strings((int*) &prefs.dialog_type, s,
+  if ( ! enum_from_strings((int*) &settings->dialog_type, s,
 			   dialog_types, NUM_DIALOG_TYPES) ) {
     g_warning("Didn't recognize dialog type in libgnomeui config");
   }
@@ -215,7 +206,7 @@ void gnome_preferences_load(void)
   
   s = gnome_config_get_string(DIALOG_POSITION_KEY);
 
-  if ( ! enum_from_strings((int*) &prefs.dialog_position, s,
+  if ( ! enum_from_strings((int*) &settings->dialog_position, s,
 			   dialog_positions, NUM_DIALOG_POSITIONS) ) {
     g_warning("Didn't recognize dialog position in libgnomeui config");
   }  
@@ -229,63 +220,81 @@ void gnome_preferences_load(void)
 
   b = gnome_config_get_bool_with_default(DIALOG_CENTERED_KEY"=true",
 					 NULL);
-  prefs.dialog_centered = b;
+  settings->dialog_centered = b;
 
   /* This is unused for now */
   b = gnome_config_get_bool_with_default(PROPERTY_BOX_BUTTONS_OK_KEY"=true",
 					 NULL);
-  prefs.property_box_buttons_ok = b;
+  settings->property_box_buttons_ok = b;
 
   b = gnome_config_get_bool_with_default(PROPERTY_BOX_BUTTONS_APPLY_KEY"=true",
 					 NULL);
-  prefs.property_box_buttons_apply = b;
+  settings->property_box_buttons_apply = b;
 
   b = gnome_config_get_bool_with_default(PROPERTY_BOX_BUTTONS_CLOSE_KEY"=true",
 					 NULL);
-  prefs.property_box_buttons_close = b;
+  settings->property_box_buttons_close = b;
 
   b = gnome_config_get_bool_with_default(PROPERTY_BOX_BUTTONS_HELP_KEY"=true",
 					 NULL);
-  prefs.property_box_buttons_help = b;
+  settings->property_box_buttons_help = b;
 
   gnome_config_pop_prefix();
   gnome_config_push_prefix(STATUSBAR);
 
   b = gnome_config_get_bool_with_default(STATUSBAR_DIALOG_KEY"=false",
 					 NULL);
-  prefs.statusbar_not_dialog = b;
+  settings->statusbar_not_dialog = b;
   
   b = gnome_config_get_bool_with_default(STATUSBAR_INTERACTIVE_KEY"=false",
 					 NULL);
-  prefs.statusbar_is_interactive = b;
+  settings->statusbar_is_interactive = b;
 
   gnome_config_pop_prefix();
   gnome_config_push_prefix(APP);
 
-  b = gnome_config_get_bool_with_default(TOOLBAR_HANDLEBOX_KEY"=true",
+  b = gnome_config_get_bool_with_default(MENUBAR_DETACHABLE_KEY"=true",
 					 NULL);
-  
-  prefs.toolbar_handlebox = b;
+  settings->menubar_detachable = b;
 
-  b = gnome_config_get_bool_with_default(MENUBAR_HANDLEBOX_KEY"=true",
+  b = gnome_config_get_bool_with_default(MENUBAR_RELIEF_KEY"=true",
 					 NULL);
-  
-  prefs.menubar_handlebox = b;
+  settings->menubar_relief = b;
+
+  b = gnome_config_get_bool_with_default(TOOLBAR_DETACHABLE_KEY"=true",
+					 NULL);
+  settings->toolbar_detachable = b;
 
   b = gnome_config_get_bool_with_default(TOOLBAR_RELIEF_KEY"=true",
 					 NULL);
-  prefs.toolbar_relief = b;
+  settings->toolbar_relief = b;
+
+  b = gnome_config_get_bool_with_default(TOOLBAR_RELIEF_BTN_KEY"=false",
+					 NULL);
+  settings->toolbar_relief_btn = b;
+
+  b = gnome_config_get_bool_with_default(TOOLBAR_LINES_KEY"=true",
+					 NULL);
+  settings->toolbar_lines = b;
+
+  b = gnome_config_get_bool_with_default(TOOLBAR_LABELS_KEY"=true",
+					 NULL);
+  settings->toolbar_labels = b;
 
   b = gnome_config_get_bool_with_default (MENUS_HAVE_ICONS_KEY"=true",
 					  NULL);
-  prefs.menus_have_icons = b;
+  settings->menus_have_icons = b;
 
+  b = gnome_config_get_bool_with_default (DISABLE_IMLIB_CACHE_KEY"=true",
+					  NULL);
+  settings->disable_imlib_cache = b;
+  
   gnome_config_pop_prefix();
   gnome_config_push_prefix(MDI);
 
   s = gnome_config_get_string(MDI_MODE_KEY);
 
-  if ( ! enum_from_strings((int*) &prefs.mdi_mode, s,
+  if ( ! enum_from_strings((int*) &settings->mdi_mode, s,
 			   mdi_modes, NUM_MDI_MODES) ) {
     g_warning("Didn't recognize MDI mode in libgnomeui config");
   }
@@ -294,7 +303,7 @@ void gnome_preferences_load(void)
   
   s = gnome_config_get_string(MDI_TAB_POS_KEY);
 
-  if ( ! enum_from_strings((int*) &prefs.mdi_tab_pos, s,
+  if ( ! enum_from_strings((int*) &settings->mdi_tab_pos, s,
 			   tab_positions, NUM_TAB_POSITIONS) ) {
     g_warning("Didn't recognize MDI notebook tab position in libgnomeui config");
   }
@@ -304,55 +313,75 @@ void gnome_preferences_load(void)
   gnome_config_pop_prefix();
 }
 
-void gnome_preferences_save(void)
+void
+gnome_preferences_save_custom(GnomePreferences *settings)
 {
   gnome_config_push_prefix(DIALOGS);
   
   gnome_config_set_string(DIALOG_BUTTONS_STYLE_KEY, 
-			  dialog_button_styles[prefs.dialog_buttons_style]);
+			  dialog_button_styles[settings->dialog_buttons_style]);
 
   gnome_config_set_string(DIALOG_TYPE_KEY, 
-			  dialog_types[prefs.dialog_type]);
+			  dialog_types[settings->dialog_type]);
 
   gnome_config_set_string(DIALOG_POSITION_KEY, 
-			  dialog_positions[prefs.dialog_position]);
+			  dialog_positions[settings->dialog_position]);
   
   gnome_config_set_bool  (DIALOG_CENTERED_KEY,
-			  prefs.dialog_centered);
+			  settings->dialog_centered);
 
   gnome_config_pop_prefix();
 
   gnome_config_push_prefix(STATUSBAR);
 
   gnome_config_set_bool(STATUSBAR_DIALOG_KEY,
-			prefs.statusbar_not_dialog);
+			settings->statusbar_not_dialog);
   gnome_config_set_bool(STATUSBAR_INTERACTIVE_KEY,
-			prefs.statusbar_is_interactive);
+			settings->statusbar_is_interactive);
 
 
   gnome_config_pop_prefix();
   gnome_config_push_prefix(APP);
 
-  gnome_config_set_bool(TOOLBAR_HANDLEBOX_KEY,
-			prefs.toolbar_handlebox);
-  gnome_config_set_bool(MENUBAR_HANDLEBOX_KEY,
-			prefs.menubar_handlebox);
+  gnome_config_set_bool(MENUBAR_DETACHABLE_KEY,
+			settings->menubar_detachable);
+  gnome_config_set_bool(MENUBAR_RELIEF_KEY,
+			settings->menubar_relief);
+  gnome_config_set_bool(TOOLBAR_DETACHABLE_KEY,
+			settings->toolbar_detachable);
   gnome_config_set_bool(TOOLBAR_RELIEF_KEY,
-			prefs.toolbar_relief);
+			settings->toolbar_relief);
+  gnome_config_set_bool(TOOLBAR_RELIEF_BTN_KEY,
+			settings->toolbar_relief_btn);
+  gnome_config_set_bool(TOOLBAR_LINES_KEY,
+			settings->toolbar_lines);
+  gnome_config_set_bool(TOOLBAR_LABELS_KEY,
+			settings->toolbar_labels);
   gnome_config_set_bool(MENUS_HAVE_ICONS_KEY,
-			prefs.menus_have_icons);
-
+			settings->menus_have_icons);
+  gnome_config_set_bool(DISABLE_IMLIB_CACHE_KEY,
+			settings->disable_imlib_cache);
   gnome_config_pop_prefix();
   gnome_config_push_prefix(MDI);
 
-  gnome_config_set_string(MDI_MODE_KEY, mdi_modes[prefs.mdi_mode]);
-  gnome_config_set_string(MDI_TAB_POS_KEY, tab_positions[prefs.mdi_tab_pos]);
+  gnome_config_set_string(MDI_MODE_KEY, mdi_modes[settings->mdi_mode]);
+  gnome_config_set_string(MDI_TAB_POS_KEY, tab_positions[settings->mdi_tab_pos]);
 
   gnome_config_pop_prefix();
 
   gnome_config_sync();
 }
 
+void gnome_preferences_load(void)
+{
+  gnome_preferences_load_custom(&prefs);
+}
+
+void gnome_preferences_save(void)
+{
+  gnome_preferences_save_custom(&prefs);
+}
+
 GtkButtonBoxStyle gnome_preferences_get_button_layout (void)
 {
   return prefs.dialog_buttons_style;
@@ -360,7 +389,7 @@ GtkButtonBoxStyle gnome_preferences_get_button_layout (void)
 
 void              gnome_preferences_set_button_layout (GtkButtonBoxStyle style)
 {
-  g_return_if_fail( (style >= 0) && (style <= GTK_BUTTONBOX_END) );
+  g_return_if_fail ((style <= GTK_BUTTONBOX_END));
   prefs.dialog_buttons_style = style;
 }
 
@@ -384,24 +413,34 @@ void              gnome_preferences_set_statusbar_interactive(gboolean b)
   prefs.statusbar_is_interactive = b;
 }
 
-gboolean          gnome_preferences_get_toolbar_handlebox    (void)
+gboolean          gnome_preferences_get_menubar_detachable   (void)
 {
-  return prefs.toolbar_handlebox;
+  return prefs.menubar_detachable;
 }
 
-void              gnome_preferences_set_toolbar_handlebox    (gboolean b)
+void              gnome_preferences_set_menubar_detachable   (gboolean b)
 {
-  prefs.toolbar_handlebox = b;
+  prefs.menubar_detachable = b;
 }
 
-gboolean          gnome_preferences_get_menubar_handlebox    (void)
+gboolean          gnome_preferences_get_menubar_relief    (void)
 {
-  return prefs.menubar_handlebox;
+  return prefs.menubar_relief;
 }
 
-void              gnome_preferences_set_menubar_handlebox    (gboolean b)
+void              gnome_preferences_set_menubar_relief    (gboolean b)
 {
-  prefs.menubar_handlebox = b;
+  prefs.menubar_relief = b;
+}
+
+gboolean          gnome_preferences_get_toolbar_detachable   (void)
+{
+  return prefs.toolbar_detachable;
+}
+
+void              gnome_preferences_set_toolbar_detachable   (gboolean b)
+{
+  prefs.toolbar_detachable = b;
 }
 
 gboolean          gnome_preferences_get_toolbar_relief    (void)
@@ -414,6 +453,36 @@ void              gnome_preferences_set_toolbar_relief    (gboolean b)
   prefs.toolbar_relief = b;
 }
 
+gboolean          gnome_preferences_get_toolbar_relief_btn (void)
+{
+  return prefs.toolbar_relief_btn;
+}
+
+void              gnome_preferences_set_toolbar_relief_btn (gboolean b)
+{
+  prefs.toolbar_relief_btn = b;
+}
+
+gboolean          gnome_preferences_get_toolbar_lines     (void)
+{
+  return prefs.toolbar_lines;
+}
+
+void              gnome_preferences_set_toolbar_lines     (gboolean b)
+{
+  prefs.toolbar_lines = b;
+}
+
+gboolean          gnome_preferences_get_toolbar_labels    (void)
+{
+  return prefs.toolbar_labels;
+}
+
+void              gnome_preferences_set_toolbar_labels    (gboolean b)
+{
+  prefs.toolbar_labels = b;
+}
+
 gboolean          gnome_preferences_get_dialog_centered      ()
 {
   return prefs.dialog_centered;
@@ -489,3 +558,15 @@ gnome_preferences_set_menus_have_icons (int have_icons)
 {
 	prefs.menus_have_icons = have_icons ? TRUE : FALSE;
 }
+
+int
+gnome_preferences_get_disable_imlib_cache (void)
+{
+	return prefs.disable_imlib_cache;
+}
+
+void
+gnome_preferences_set_disable_imlib_cache (int disable_imlib_cache)
+{
+	prefs.disable_imlib_cache = disable_imlib_cache ? TRUE : FALSE;
+}
