@@ -36,31 +36,8 @@
 #include "gnome-pixmap.h"
 #include "gnome-uidefs.h"
 
-
-/*
- * BUTTON_DEFBRD_WIDTH/HEIGHT is the value that is added to GNOME_BUTTON_WIDTH
- * and GNOME_BUTTON_HEIGHT to emulate the real visual width and height of
- * buttons without the GTK_CAN_DEFAULT flag.
- */
-#define BUTTON_DEFBRD_WIDTH 3
-#define BUTTON_DEFBRD_HEIGHT (-11)
-
-
-
-/*
- * Hmmm, I'm not sure, if using color context can consume to much colors,
- * so one can define this here. If defined, all color context modes other
- * then true/pseudo color and gray lead to the old shading method for
- * insensitive pixmaps. If undefined, the newer method will always be used
- */
-#define NOT_ALWAYS_SHADE
-
-
-
 #include "pixmaps/gnome-stock-imlib.h"
 
-
-
 #define STOCK_SEP '.'
 #define STOCK_SEP_STR "."
 
@@ -303,7 +280,7 @@ gnome_stock_pixmap_widget_set_icon(GnomeStockPixmapWidget *widget,
 	widget->icon = g_strdup(icon);
 	gnome_stock_pixmap_widget_state_changed(GTK_WIDGET(widget), 0);
 }
-#endif /* !USE_NBEW_GNOME_STOCK */
+#endif /* !USE_NEW_GNOME_STOCK */
 
 
 
@@ -364,7 +341,8 @@ gnome_stock_paint(GnomeStock *stock, GnomePixmap *pixmap)
 		if ((width != req.width) || (height != req.height)) {
 			gdk_pixmap_unref(gpixmap->pixmap);
 			gpixmap->pixmap = NULL;
-			gdk_pixmap_unref(gpixmap->mask);
+			if (gpixmap->mask)
+				gdk_pixmap_unref(gpixmap->mask);
 			gpixmap->mask = NULL;
 		}
 	}
@@ -372,22 +350,23 @@ gnome_stock_paint(GnomeStock *stock, GnomePixmap *pixmap)
 		gpixmap->pixmap = gdk_pixmap_new(pixmap->pixmap,
 						 req.width, req.height,
 						 visual->depth);
-		gpixmap->mask = gdk_pixmap_new(pixmap->mask,
-					       req.width, req.height, 1);
 	}
 	gc = gdk_gc_new(gpixmap->pixmap);
 	gdk_draw_pixmap(gpixmap->pixmap, gc, pixmap->pixmap, 0, 0, 0, 0,
 			req.width, req.height);
 	gdk_gc_destroy(gc);
 	if (pixmap->mask) {
+		gpixmap->mask = gdk_pixmap_new(pixmap->mask,
+					       req.width, req.height, 1);
 		gc = gdk_gc_new(gpixmap->mask);
 		gdk_draw_pixmap(gpixmap->mask, gc, pixmap->mask, 0, 0, 0, 0,
 				req.width, req.height);
 		gdk_gc_destroy(gc);
+		if (!(GTK_WIDGET_FLAGS(gpixmap)&GTK_NO_WINDOW) &&
+			GTK_WIDGET(gpixmap)->window)
+			gtk_widget_shape_combine_mask(GTK_WIDGET(gpixmap),
+						      gpixmap->mask, 0, 0);
 	}
-	if (GTK_WIDGET(gpixmap)->window)
-		gdk_window_shape_combine_mask(GTK_WIDGET(gpixmap)->window,
-					      gpixmap->mask, 0, 0);
 	/* XXX: this isn't needed always, but I found that in some
 	 * circumstances it is. */
 	gtk_widget_queue_draw(GTK_WIDGET(stock));
@@ -404,14 +383,8 @@ gnome_stock_state_changed(GtkWidget *widget, guint prev_state)
 	GnomeStock *w = GNOME_STOCK(widget);
 	GnomePixmap *pixmap;
 	GtkWidget *tmp;
-	static GHashTable *hash = NULL;
-	struct style_hash {
-		GtkWidget *widget;
-	};
+	guint32 c;
 
-	if (!hash) {
-		hash = g_hash_table_new(g_direct_hash, g_direct_equal);
-	}
 	if (widget->parent) {
 		GdkWindow *w;
 		tmp = widget->parent;
@@ -423,38 +396,43 @@ gnome_stock_state_changed(GtkWidget *widget, guint prev_state)
 	} else {
 		tmp = widget;
 	}
-	if (g_hash_table_lookup(hash, widget)) {
-		if (tmp != g_hash_table_lookup(hash, widget)) {
-			if (w->regular) {
-				gtk_widget_unref(GTK_WIDGET(w->regular));
-				w->regular = NULL;
-			}
-			if (w->disabled) {
-				gtk_widget_unref(GTK_WIDGET(w->disabled));
-				w->disabled = NULL;
-			}
-			if (w->focused) {
-				gtk_widget_unref(GTK_WIDGET(w->focused));
-				w->focused = NULL;
-			}
-			w->current = NULL;
-		}
-		/* FIXME: do I really need to remove the entry? */
-		g_hash_table_remove(hash, widget);
-	}
-	g_hash_table_insert(hash, widget, tmp);
+	c = (tmp->style->bg[tmp->state].red >> 8) |
+		(tmp->style->bg[tmp->state].green & 0xff00) |
+		((tmp->style->bg[tmp->state].blue & 0xff00) << 8);
 	pixmap = NULL;
-	if (GTK_WIDGET_HAS_FOCUS(widget)) {
+	/* if (GTK_WIDGET_HAS_FOCUS(widget)) { */
+	if (tmp->state == GTK_STATE_PRELIGHT) {
+		if ((c != w->c_focused) && (w->focused)) {
+			if (w->current == w->focused)
+				w->current = NULL;
+			gtk_widget_unref(GTK_WIDGET(w->focused));
+			w->focused = NULL;
+		}
+		w->c_focused = c;
 		if (!w->focused) {
 			w->focused = gnome_stock_pixmap(tmp, w->icon, GNOME_STOCK_PIXMAP_FOCUSED);
 		}
 		pixmap = w->focused;
 	} else if (!GTK_WIDGET_IS_SENSITIVE(widget)) {
+		if ((c != w->c_disabled) && (w->disabled)) {
+			if (w->current == w->disabled)
+				w->current = NULL;
+			gtk_widget_unref(GTK_WIDGET(w->disabled));
+			w->disabled = NULL;
+		}
+		w->c_disabled = c;
 		if (!w->disabled) {
 			w->disabled = gnome_stock_pixmap(tmp, w->icon, GNOME_STOCK_PIXMAP_DISABLED);
 		}
 		pixmap = w->disabled;
 	} else {
+		if ((c != w->c_regular) && (w->regular)) {
+			if (w->current == w->regular)
+				w->current = NULL;
+			gtk_widget_unref(GTK_WIDGET(w->regular));
+			w->regular = NULL;
+		}
+		w->c_regular = c;
 		if (!w->regular) {
 			w->regular = gnome_stock_pixmap(tmp, w->icon, GNOME_STOCK_PIXMAP_REGULAR);
 		}
@@ -470,18 +448,22 @@ static gint
 gnome_stock_expose(GtkWidget *widget, GdkEventExpose *event)
 {
 	gnome_stock_state_changed(widget, 0);
-	return (*parent_expose)(widget, event);
+	if (parent_expose)
+		return (*parent_expose)(widget, event);
+	else
+		return 0;
 }
 
 static void
 gnome_stock_draw(GtkWidget *widget, GdkRectangle *area)
 {
 	gnome_stock_state_changed(widget, 0);
-	(*parent_draw)(widget, area);
+	if (parent_draw)
+		(*parent_draw)(widget, area);
 }
 
 static void
-gnome_stock_class_init(GtkObjectClass *klass)
+gnome_stock_class_init(GtkObjectClass * klass)
 {
 	klass->destroy = gnome_stock_destroy;
 	((GtkWidgetClass *)klass)->state_changed =
@@ -552,7 +534,7 @@ gnome_stock_set_icon(GnomeStock *stock, const char *icon)
 	g_return_val_if_fail(icon != NULL, 0);
 
 	entry = lookup(icon, GNOME_STOCK_PIXMAP_REGULAR, 0);
-	g_return_val_if_fail(entry != NULL, 0);
+	if (!entry) return 0;
 
 	if (stock->icon) {
 		if (0 == strcmp(icon, stock->icon))
@@ -606,7 +588,8 @@ static struct scale_color scale_base = { 0xd6, 0xd6, 0xd6 };
 static struct scale_color scale_trans = { 0xff, 0x00, 0xff };
 
 static char *
-scale_down(GtkWidget *window, unsigned char *datao, gint wo, gint ho, gint w, gint h)
+scale_down(GtkWidget *window, GtkStateType state, unsigned char *datao,
+	   gint wo, gint ho, gint w, gint h)
 {
 	unsigned char *data, *p, *p2, *p3;
 	long x, y, w2, h2, xo, yo, wo3, x2, y2, i, x3, y3;
@@ -615,9 +598,9 @@ scale_down(GtkWidget *window, unsigned char *datao, gint wo, gint ho, gint w, gi
 
 	if (window) {
 		GtkStyle *style = gtk_widget_get_style(window);
-		scale_base.r = style->bg[0].red >> 8;
-		scale_base.g = style->bg[0].green >> 8;
-		scale_base.b = style->bg[0].blue >> 8;
+		scale_base.r = style->bg[state].red >> 8;
+		scale_base.g = style->bg[state].green >> 8;
+		scale_base.b = style->bg[state].blue >> 8;
 	}
 	data = g_malloc(w * h * 3);
 	p = data;
@@ -701,8 +684,8 @@ scale_down(GtkWidget *window, unsigned char *datao, gint wo, gint ho, gint w, gi
 }
 
 struct _default_entries_data {
-	char *icon, *subtype;
-	char *label;
+	const char *icon, *subtype;
+	const char *label;
 	const gchar *rgb_data;
 	int width, height;
 	int scaled_width, scaled_height;
@@ -715,7 +698,7 @@ struct _default_entries_data {
 #define MENU_W 16
 #define MENU_H 16
 
-struct _default_entries_data entries_data[] = {
+static const struct _default_entries_data entries_data[] = {
 	{GNOME_STOCK_PIXMAP_NEW, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_new, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
 	{GNOME_STOCK_PIXMAP_SAVE, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_save, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
 	{GNOME_STOCK_PIXMAP_SAVE_AS, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_save_as, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
@@ -767,6 +750,22 @@ struct _default_entries_data entries_data[] = {
 	{GNOME_STOCK_PIXMAP_MULTIPLE, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_multiple_file, 32, 32, 32, 32},
 	{GNOME_STOCK_PIXMAP_EXIT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_exit, TB_W, TB_H, TB_W, TB_H},
 	{GNOME_STOCK_PIXMAP_ABOUT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_menu_about, MENU_W, MENU_H, MENU_W, MENU_H},
+	{GNOME_STOCK_PIXMAP_UP, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_up_arrow, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_DOWN, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_down_arrow, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_TOP, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_top, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_BOTTOM, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_bottom, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_ATTACH, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_attach, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_FONT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_font, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_INDEX, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_index, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_EXEC, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_exec, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_ALIGN_LEFT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_align_left, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_ALIGN_RIGHT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_align_right, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_ALIGN_CENTER, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_align_center, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_ALIGN_JUSTIFY, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_align_justify, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_TEXT_BOLD, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_text_bold, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_TEXT_ITALIC, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_text_italic, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_TEXT_UNDERLINE, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_text_underline, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
+	{GNOME_STOCK_PIXMAP_TEXT_STRIKEOUT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_text_strikeout, TIGERT_W, TIGERT_H, TIGERT_W, TIGERT_H},
 	{GNOME_STOCK_BUTTON_OK, GNOME_STOCK_PIXMAP_REGULAR, N_("OK"), imlib_button_ok, TB_W, TB_H, TB_W, TB_H},
 	{GNOME_STOCK_BUTTON_APPLY, GNOME_STOCK_PIXMAP_REGULAR, N_("Apply"), imlib_button_apply, TB_W, TB_H, TB_W, TB_H},
 	{GNOME_STOCK_BUTTON_CANCEL, GNOME_STOCK_PIXMAP_REGULAR, N_("Cancel"), imlib_button_cancel, TB_W, TB_H, TB_W, TB_H},
@@ -776,6 +775,9 @@ struct _default_entries_data entries_data[] = {
 	{GNOME_STOCK_BUTTON_HELP, GNOME_STOCK_PIXMAP_REGULAR, N_("Help"), imlib_help, TIGERT_W, TIGERT_H, TB_W, TB_H},
 	{GNOME_STOCK_BUTTON_NEXT, GNOME_STOCK_PIXMAP_REGULAR, N_("Next"), imlib_right_arrow, TIGERT_W, TIGERT_H, TB_W, TB_H},
 	{GNOME_STOCK_BUTTON_PREV, GNOME_STOCK_PIXMAP_REGULAR, N_("Prev"), imlib_left_arrow, TIGERT_W, TIGERT_H, TB_W, TB_H},
+	{GNOME_STOCK_BUTTON_UP, GNOME_STOCK_PIXMAP_REGULAR, N_("Up"), imlib_up_arrow, TIGERT_W, TIGERT_H, TB_W, TB_H},
+	{GNOME_STOCK_BUTTON_DOWN, GNOME_STOCK_PIXMAP_REGULAR, N_("Down"), imlib_down_arrow, TIGERT_W, TIGERT_H, TB_W, TB_H},
+	{GNOME_STOCK_BUTTON_FONT, GNOME_STOCK_PIXMAP_REGULAR, N_("Font"), imlib_font, TIGERT_W, TIGERT_H, TB_W, TB_H},
 	{GNOME_STOCK_MENU_NEW, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_new, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
 	{GNOME_STOCK_MENU_SAVE, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_save, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
 	{GNOME_STOCK_MENU_SAVE_AS, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_save_as, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
@@ -826,9 +828,26 @@ struct _default_entries_data entries_data[] = {
 	/* TODO: I shouldn't waste a pixmap for that */
 	{GNOME_STOCK_MENU_BLANK, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_menu_blank, MENU_W, MENU_H, MENU_W, MENU_H}, 
 	{GNOME_STOCK_MENU_SCORES, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_menu_scores, 20, 20, 20, 20},
+	{GNOME_STOCK_MENU_UP, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_up_arrow, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_DOWN, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_down_arrow, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_TOP, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_top, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_BOTTOM, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_bottom, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_ATTACH, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_attach, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_INDEX, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_index, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_FONT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_font, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_EXEC, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_exec, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_ALIGN_LEFT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_align_left, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_ALIGN_RIGHT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_align_right, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_ALIGN_CENTER, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_align_center, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_ALIGN_JUSTIFY, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_align_justify, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_TEXT_BOLD, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_text_bold, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_TEXT_ITALIC, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_text_italic, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_TEXT_UNDERLINE, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_text_underline, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+	{GNOME_STOCK_MENU_TEXT_STRIKEOUT, GNOME_STOCK_PIXMAP_REGULAR, NULL, imlib_text_strikeout, TIGERT_W, TIGERT_H, MENU_W, MENU_H},
+
 };
 
-static int entries_data_num = sizeof(entries_data) / sizeof(entries_data[0]);
+static const int entries_data_num = sizeof(entries_data) / sizeof(entries_data[0]);
 
 
 static char *
@@ -854,7 +873,7 @@ stock_pixmaps(void)
 		entry = g_malloc(sizeof(GnomeStockPixmapEntry));
 		entry->any.width = entries_data[i].width;
 		entry->any.height = entries_data[i].height;
-		entry->any.label = entries_data[i].label;
+		entry->any.label = (char *) entries_data[i].label;
 		entry->type = GNOME_STOCK_PIXMAP_TYPE_IMLIB_SCALED;
 		entry->imlib_s.scaled_width = entries_data[i].scaled_width;
 		entry->imlib_s.scaled_height = entries_data[i].scaled_height;
@@ -871,6 +890,7 @@ stock_pixmaps(void)
 	
 #if defined(DEBUG) && 0
 	{
+		time_t time(time_t *);
 		time_t t;
 		int i;
 		unsigned char *bla;
@@ -881,7 +901,7 @@ stock_pixmaps(void)
 		t++;
 		i = 0;
 		while ((t == time(NULL)) && (i < 0x1000000)) {
-			g_free(scale_down(NULL, bla, 24, 24, 16, 16));
+			g_free(scale_down(NULL, 0, bla, 24, 24, 16, 16));
 			i++;
 		}
 		g_print("%x %d\n", i, i);
@@ -939,25 +959,30 @@ create_pixmap_from_imlib(GtkWidget *window, GnomeStockPixmapEntryImlib *data)
 
 
 static GnomePixmap *
-create_pixmap_from_imlib_scaled(GtkWidget *window,
+create_pixmap_from_imlib_scaled(GtkWidget *window, GtkStateType state,
 				GnomeStockPixmapEntryImlibScaled *data)
 {
 	static GdkImlibColor shape_color = { 0xff, 0, 0xff, 0 };
+	GnomePixmap *ret;
 	gchar *d;
 
 	if ((data->width != data->scaled_width) ||
 	    (data->height != data->scaled_height)) {
-		d = scale_down(window, (gchar *)data->rgb_data, data->width,
-			       data->height, data->scaled_width,
-			       data->scaled_height);
+		if (!gnome_config_get_bool("/Gnome/Icons/ImlibResize=false")) {
+			d = scale_down(window, state, (gchar *)data->rgb_data,
+				       data->width, data->height,
+				       data->scaled_width,
+				       data->scaled_height);
+			ret = (GnomePixmap *)gnome_pixmap_new_from_rgb_d_shaped(d, NULL, data->scaled_width, data->scaled_height, &shape_color);
+			g_free(d);
+			return ret;
+		} else {
+			return (GnomePixmap *)gnome_pixmap_new_from_rgb_d_shaped_at_size((gchar *)data->rgb_data, NULL, data->width, data->height, data->scaled_width, data->scaled_height, &shape_color);
+		}
 	} else {
-		d = (gchar *)data->rgb_data;
+		return (GnomePixmap *)gnome_pixmap_new_from_rgb_d_shaped((gchar *)data->rgb_data, NULL, data->scaled_width, data->scaled_height, &shape_color);
 	}
 
-	return (GnomePixmap *)gnome_pixmap_new_from_rgb_d_shaped(d, NULL,
-								 data->scaled_width,
-								 data->scaled_height,
-								 &shape_color);
 }
 
 
@@ -971,7 +996,8 @@ create_pixmap_from_path(GnomeStockPixmapEntryPath *data)
 
 
 static void
-build_disabled_pixmap(GtkWidget *window, GnomePixmap **inout_pixmap)
+build_disabled_pixmap(GtkWidget *window, GtkStateType state,
+		      GnomePixmap **inout_pixmap)
 {
 	GdkGC *gc;
 	GdkWindow *pixmap = (*inout_pixmap)->pixmap;
@@ -980,28 +1006,32 @@ build_disabled_pixmap(GtkWidget *window, GnomePixmap **inout_pixmap)
 	GdkVisual *visual;
 	GdkImage *image;
 	GdkColorContext *cc;
-	GdkColor color;
 	GdkColormap *cmap;
 	gint32 red, green, blue;
-#ifdef NOT_ALWAYS_SHADE
 	GtkStyle *style;
-#endif
+	int use_stripple = -1;
 
 	g_return_if_fail(window != NULL);
 
+	/* values for use_stripple:
+	 * 0 - decide upon the value of cc->mode
+	 * 1 - always stripple
+	 * 2 - always shade */
+	use_stripple = gnome_config_get_int("/Gnome/Icons/Shade=0");
+
 	gdk_window_get_size(pixmap, &w, &h);
 	visual = gtk_widget_get_visual(GTK_WIDGET(*inout_pixmap));
 	cmap = gtk_widget_get_colormap(GTK_WIDGET(*inout_pixmap));
 	gc = gdk_gc_new (pixmap);
 
 	cc = gdk_color_context_new(visual, cmap);
-#ifdef NOT_ALWAYS_SHADE
-	if ((cc->mode != GDK_CC_MODE_TRUE) &&
-	    (cc->mode != GDK_CC_MODE_MY_GRAY)) {
+	if (((cc->mode != GDK_CC_MODE_TRUE) &&
+	     (cc->mode != GDK_CC_MODE_MY_GRAY) &&
+	     (use_stripple != 2)) || 
+	    (use_stripple == 1)) {
 		gdk_color_context_free(cc);
 		style = gtk_widget_get_style(window);
-		color = style->bg[0];
-		gdk_gc_set_foreground (gc, &color);
+		gdk_gc_set_foreground (gc, &style->bg[state]);
 		for (y = 0; y < h; y ++) {
 			for (x = y % 2; x < w; x += 2) {
 				gdk_draw_point(pixmap, gc, x, y);
@@ -1010,15 +1040,13 @@ build_disabled_pixmap(GtkWidget *window, GnomePixmap **inout_pixmap)
 		gdk_gc_destroy(gc);
 		return;
 	}
-#endif
 
 	image = gdk_image_get(pixmap, 0, 0, w, h);
 	gdk_gc_get_values(gc, &vals);
 	style = gtk_widget_get_style(window);
-	color = style->bg[0];
-	red = color.red;
-	green = color.green;
-	blue = color.blue;
+	red = style->bg[state].red;
+	green = style->bg[state].green;
+	blue = style->bg[state].blue;
 	for (y = 0; y < h; y++) {
 		for (x = 0; x < w; x++) {
 			GdkColor c;
@@ -1053,7 +1081,7 @@ build_disabled_pixmap(GtkWidget *window, GnomePixmap **inout_pixmap)
 
 
 static GnomePixmap *
-gnome_stock_pixmap(GtkWidget *window, const char *icon, const char *subtype)
+gnome_stock_pixmap(GtkWidget * window, const char *icon, const char *subtype)
 {
 	GnomeStockPixmapEntry *entry;
 	GnomePixmap *pixmap;
@@ -1061,32 +1089,34 @@ gnome_stock_pixmap(GtkWidget *window, const char *icon, const char *subtype)
 	g_return_val_if_fail(icon != NULL, NULL);
 	/* subtype can be NULL, so not checked */
 	entry = lookup(icon, subtype, TRUE);
-	if (!entry) return NULL;
+	if (!entry)
+		return NULL;
 
 	if ((entry->type != GNOME_STOCK_PIXMAP_TYPE_IMLIB) &&
 	    (entry->type != GNOME_STOCK_PIXMAP_TYPE_IMLIB_SCALED)) {
 		g_return_val_if_fail(window != NULL, NULL);
 		g_return_val_if_fail(GTK_IS_WIDGET(window), NULL);
 	}
-	
+
 	pixmap = NULL;
 	switch (entry->type) {
-	 case GNOME_STOCK_PIXMAP_TYPE_DATA:
+	case GNOME_STOCK_PIXMAP_TYPE_DATA:
 		pixmap = create_pixmap_from_data(window, &entry->data);
 		break;
-	 case GNOME_STOCK_PIXMAP_TYPE_PATH:
+	case GNOME_STOCK_PIXMAP_TYPE_PATH:
 		pixmap = create_pixmap_from_path(&entry->path);
 		break;
-	 case GNOME_STOCK_PIXMAP_TYPE_IMLIB:
+	case GNOME_STOCK_PIXMAP_TYPE_IMLIB:
 		pixmap = create_pixmap_from_imlib(window, &entry->imlib);
 		break;
-	 case GNOME_STOCK_PIXMAP_TYPE_IMLIB_SCALED:
-		pixmap = create_pixmap_from_imlib_scaled(window, &entry->imlib_s);
+	case GNOME_STOCK_PIXMAP_TYPE_IMLIB_SCALED:
+		pixmap = create_pixmap_from_imlib_scaled(window, window->state,
+							 &entry->imlib_s);
 		break;
-	 case GNOME_STOCK_PIXMAP_TYPE_GPIXMAP:
+	case GNOME_STOCK_PIXMAP_TYPE_GPIXMAP:
 		pixmap = GNOME_PIXMAP(gnome_pixmap_new_from_gnome_pixmap(entry->gpixmap.pixmap));
 		break;
-	 default:
+	default:
 		g_assert_not_reached();
 		break;
 	}
@@ -1094,18 +1124,97 @@ gnome_stock_pixmap(GtkWidget *window, const char *icon, const char *subtype)
 	/* TODO: should be optimized a bit */
 	if ((NULL == lookup(icon, subtype, 0)) && (pixmap) &&
 	    (0 == strcmp(subtype, GNOME_STOCK_PIXMAP_DISABLED))) {
-		build_disabled_pixmap(window, &pixmap);
+		build_disabled_pixmap(window, window->state, &pixmap);
 	}
 	return pixmap;
 }
 
 
 
+GtkWidget *
+gnome_stock_pixmap_widget_at_size(GtkWidget *window, const char *icon,
+				  guint width, guint height)
+{
+	GnomeStockPixmapEntry *entry;
+	GnomeStockPixmapEntryImlibScaled *new_entry;
+	GdkImlibImage *im;
+	char name[512];
+
+	g_return_val_if_fail(icon != NULL, NULL);
+	g_snprintf(name, sizeof(name), "%s%ux%u", icon, width, height);
+	entry = lookup(name, GNOME_STOCK_PIXMAP_REGULAR, 0);
+	if (entry)
+		return gnome_stock_new_with_icon(name);
+	entry = lookup(icon, GNOME_STOCK_PIXMAP_REGULAR, 0);
+	if ((entry) &&
+	    ((entry->type == GNOME_STOCK_PIXMAP_TYPE_IMLIB) ||
+	     (entry->type == GNOME_STOCK_PIXMAP_TYPE_IMLIB_SCALED))) {
+		new_entry = g_malloc(sizeof(GnomeStockPixmapEntryImlibScaled));
+		new_entry->type = GNOME_STOCK_PIXMAP_TYPE_IMLIB_SCALED;
+		new_entry->label = NULL;
+		new_entry->scaled_width = width;
+		new_entry->scaled_height = height;
+		new_entry->width = entry->imlib.width;
+		new_entry->height = entry->imlib.height;
+		new_entry->rgb_data = entry->imlib.rgb_data;
+		new_entry->shape = entry->imlib.shape;
+	} else {
+		char *filename;
+		if (g_file_exists(icon))
+			filename = g_strdup(icon);
+		else
+			filename = gnome_pixmap_file(icon);
+		if (!filename) return NULL;
+		im = gdk_imlib_load_image(filename);
+		g_free(filename);
+		g_return_val_if_fail(im != NULL, NULL);
+		new_entry = g_malloc(sizeof(GnomeStockPixmapEntryImlibScaled));
+		new_entry->type = GNOME_STOCK_PIXMAP_TYPE_IMLIB_SCALED;
+		new_entry->label = NULL;
+		new_entry->scaled_width = width;
+		new_entry->scaled_height = height;
+		new_entry->width = im->rgb_width;
+		new_entry->height = im->rgb_height;
+		new_entry->rgb_data = g_malloc(im->rgb_width * im->rgb_height * 3);
+		memcpy((char *)new_entry->rgb_data, im->rgb_data,
+		       im->rgb_width * im->rgb_height * 3);
+		new_entry->shape = im->shape_color;
+		gdk_imlib_destroy_image(im);
+	}
+	gnome_stock_pixmap_register(name, GNOME_STOCK_PIXMAP_REGULAR,
+				    (GnomeStockPixmapEntry *)new_entry);
+	return gnome_stock_new_with_icon(name);
+}
+
+
+
 GtkWidget *
 gnome_stock_pixmap_widget(GtkWidget *window, const char *icon)
 {
 #if USE_NEW_GNOME_STOCK
-	return gnome_stock_new_with_icon(icon);
+	GtkWidget *w;
+
+	w = gnome_stock_new_with_icon(icon);
+	if (!w) {
+		GnomeStockPixmapEntryPath *new_entry;
+		char *filename;
+		if (g_file_exists(icon))
+			filename = g_strdup(icon);
+		else
+			filename = gnome_pixmap_file(icon);
+		if (!filename) return NULL;
+		g_assert(!gnome_stock_pixmap_checkfor(icon, GNOME_STOCK_PIXMAP_REGULAR));
+		new_entry = g_malloc(sizeof(GnomeStockPixmapEntryPath));
+		new_entry->type = GNOME_STOCK_PIXMAP_TYPE_PATH;
+		new_entry->label = NULL;
+		new_entry->pathname = filename;
+		new_entry->width = 0;
+		new_entry->height = 0;
+		gnome_stock_pixmap_register(icon, GNOME_STOCK_PIXMAP_REGULAR,
+					    (GnomeStockPixmapEntry *)new_entry);
+		return gnome_stock_new_with_icon(icon);
+	}
+	return w;
 #else
 	GtkWidget *w;
 
@@ -1163,27 +1272,7 @@ gnome_stock_pixmap_checkfor(const char *icon, const char *subtype)
 void
 gnome_button_can_default(GtkButton *button, gboolean can_default)
 {
-	GtkRequisition req;
-
-	g_return_if_fail(button != NULL);
-	g_return_if_fail(GTK_IS_BUTTON(button));
-
-	if (can_default) {
-		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-		gtk_widget_size_request(GTK_WIDGET(button), &req);
-		if (req.width < GNOME_BUTTON_WIDTH)
-			req.width = GNOME_BUTTON_WIDTH;
-		if (req.height < GNOME_BUTTON_HEIGHT)
-			req.height = GNOME_BUTTON_HEIGHT;
-	} else {
-		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-		gtk_widget_size_request(GTK_WIDGET(button), &req);
-		if (req.width < GNOME_BUTTON_WIDTH + BUTTON_DEFBRD_WIDTH)
-			req.width = GNOME_BUTTON_WIDTH + BUTTON_DEFBRD_WIDTH;
-		if (req.height < GNOME_BUTTON_HEIGHT + BUTTON_DEFBRD_HEIGHT)
-			req.height = GNOME_BUTTON_HEIGHT + BUTTON_DEFBRD_HEIGHT;
-	}
-	gtk_widget_set_usize(GTK_WIDGET(button), req.width, req.height);
+        g_warning ("gnome_button_can_default() is now deprecated.\n");
 }
 
 
@@ -1235,7 +1324,7 @@ gnome_pixmap_button(GtkWidget *pixmap, const char *text)
 			entry->any.height = req.height;
 			entry->any.label = NULL;
 			entry->gpixmap.pixmap = GNOME_PIXMAP(pixmap);
-			g_snprintf(s, 32, "%lx", (long)pixmap);
+			g_snprintf(s, sizeof(s), "%lx", (long)pixmap);
 			gnome_stock_pixmap_register(s, GNOME_STOCK_PIXMAP_REGULAR, entry);
 			pixmap = gnome_stock_pixmap_widget(button, s);
 		}
@@ -1245,13 +1334,6 @@ gnome_pixmap_button(GtkWidget *pixmap, const char *text)
 				   FALSE, FALSE, 0);
 	}
 
-	gtk_widget_size_request(button, &req);
-	if (req.width < GNOME_BUTTON_WIDTH + BUTTON_DEFBRD_WIDTH)
-		req.width = GNOME_BUTTON_WIDTH + BUTTON_DEFBRD_WIDTH;
-	if (req.height < GNOME_BUTTON_HEIGHT + BUTTON_DEFBRD_HEIGHT)
-		req.height = GNOME_BUTTON_HEIGHT + BUTTON_DEFBRD_HEIGHT;
-	gtk_widget_set_usize(GTK_WIDGET(button), req.width, req.height);
-
 	return button;
 }
 
@@ -1300,21 +1382,17 @@ gnome_stock_or_ordinary_button (const char *type)
 /*  menus  */
 /***********/
 
-static int use_icons = -1;
-
 GtkWidget *
 gnome_stock_menu_item(const char *type, const char *text)
 {
 	GtkWidget *hbox, *w, *menu_item;
+	int use_icons;
 
 	g_return_val_if_fail(type != NULL, NULL);
 	g_return_val_if_fail(gnome_stock_pixmap_checkfor(type, GNOME_STOCK_PIXMAP_REGULAR), NULL);
 	g_return_val_if_fail(text != NULL, NULL);
 
-	if (use_icons == -1) {
-		use_icons =
-			gnome_config_get_bool("Gnome/Icons/MenusUseIcons=true");
-	}
+	use_icons = gnome_config_get_bool("/Gnome/Icons/MenusUseIcons=true");
 	if (use_icons) {
 		hbox = gtk_hbox_new(FALSE, 2);
 		gtk_widget_show(hbox);
@@ -1355,7 +1433,7 @@ struct default_AccelEntry {
 	AccelEntry entry;
 };
 
-struct default_AccelEntry default_accel_hash[] = {
+static const struct default_AccelEntry default_accel_hash[] = {
 	{GNOME_STOCK_MENU_NEW, {'N', GDK_CONTROL_MASK}},
 	{GNOME_STOCK_MENU_OPEN, {'O', GDK_CONTROL_MASK}},
 	{GNOME_STOCK_MENU_CLOSE, {'W', GDK_CONTROL_MASK}},
@@ -1374,6 +1452,8 @@ struct default_AccelEntry default_accel_hash[] = {
 	{GNOME_STOCK_MENU_SEARCH, {'S', GDK_MOD1_MASK}},
 	{GNOME_STOCK_MENU_BACK, {'B', GDK_CONTROL_MASK}},
 	{GNOME_STOCK_MENU_FORWARD, {'F', GDK_CONTROL_MASK}},
+	{GNOME_STOCK_MENU_UP, {'U', GDK_CONTROL_MASK}},
+	{GNOME_STOCK_MENU_DOWN, {'D', GDK_CONTROL_MASK}},
 	{GNOME_STOCK_MENU_MAIL, {0, 0}},
 	{GNOME_STOCK_MENU_MAIL_RCV, {0, 0}},
 	{GNOME_STOCK_MENU_MAIL_SND, {0, 0}},
@@ -1382,7 +1462,7 @@ struct default_AccelEntry default_accel_hash[] = {
 
 
 static char *
-accel_to_string(AccelEntry *entry)
+accel_to_string(const AccelEntry *entry)
 {
 	static char s[30];
 
@@ -1479,12 +1559,12 @@ accel_read_rc(gpointer key, gpointer value, gpointer data)
 static GHashTable *
 accel_hash(void) {
 	static GHashTable *hash = NULL;
-	struct default_AccelEntry *p;
+	const struct default_AccelEntry *p;
 
 	if (!hash) {
 		hash = g_hash_table_new(g_str_hash, g_str_equal);
 		for (p = default_accel_hash; p->type; p++)
-			g_hash_table_insert(hash, p->type, &p->entry);
+			g_hash_table_insert(hash, p->type, (gpointer) &p->entry);
 		g_hash_table_foreach(hash, accel_read_rc,
 				     "/Gnome/Accelerators/");
 	}
@@ -1583,7 +1663,7 @@ accel_dlg_select_ok(GtkWidget *widget, GtkWindow *window)
 		if (check->active)
 			entry.mod |= GDK_MOD1_MASK;
 	}
-	row = (int)gtk_object_get_data(GTK_OBJECT(window), "row");
+	row = GPOINTER_TO_INT (gtk_object_get_data(GTK_OBJECT(window), "row"));
 	gtk_clist_get_text(GTK_CLIST(gtk_object_get_data(GTK_OBJECT(window), "clist")),
 			   row, 1, &s);
 	if (!s) s = "";
@@ -1600,7 +1680,7 @@ accel_dlg_select_ok(GtkWidget *widget, GtkWindow *window)
 
 
 static void
-accel_dlg_select(GtkCList *widget, int row, int col, GdkEventButton *event)
+accel_dlg_select(GtkCList *widget, gint row, gint col, GdkEventButton *event)
 {
 	AccelEntry entry;
 	GtkTable *table;
@@ -1619,8 +1699,8 @@ accel_dlg_select(GtkCList *widget, int row, int col, GdkEventButton *event)
 	window = gtk_dialog_new();
 	gtk_window_set_title(GTK_WINDOW(window), "Menu Accelerator");
 	gtk_object_set_data(GTK_OBJECT(window), "clist", widget);
-	gtk_object_set_data(GTK_OBJECT(window), "row", (gpointer)row);
-	gtk_object_set_data(GTK_OBJECT(window), "col", (gpointer)col);
+	gtk_object_set_data(GTK_OBJECT(window), "row", GINT_TO_POINTER (row));
+	gtk_object_set_data(GTK_OBJECT(window), "col", GINT_TO_POINTER (col));
 	gtk_object_set_data(GTK_OBJECT(window), "box",
 			    gtk_object_get_data(GTK_OBJECT(widget), "box"));
 
@@ -1699,7 +1779,7 @@ gnome_stock_menu_accel_dlg(char *section)
 {
 	GnomePropertyBox *box;
 	GtkWidget *w, *label;
-	struct default_AccelEntry *p;
+	const struct default_AccelEntry *p;
 	char *titles[2];
 	char *row_data[2];
 
@@ -1810,9 +1890,59 @@ gnome_stock_transparent_window (const char *icon, const char *subtype)
 	/* Render the image, return it */
 	gdk_imlib_render (im, im->rgb_width, im->rgb_height);
 	gdk_window_set_back_pixmap (window->window, gdk_imlib_move_image (im), FALSE);
-	gdk_window_shape_combine_mask (window->window, gdk_imlib_move_mask (im), 0, 0);
+	gtk_widget_shape_combine_mask (window, gdk_imlib_move_mask (im), 0, 0);
 
 	gdk_imlib_destroy_image (im);
 	
 	return window;
 }
+
+void 
+gnome_stock_pixmap_gdk (const char   *icon,
+			const char   *subtype,
+			GdkPixmap    **pixmap,
+			GdkPixmap    **mask)
+{
+	static GdkImlibColor shape_color = { 0xff, 0, 0xff, 0 };
+	GnomeStockPixmapEntry *entry;
+	GdkImlibImage *im;
+	
+	g_return_if_fail(icon != NULL);
+	g_return_if_fail(pixmap != NULL);
+	g_return_if_fail(mask != NULL);
+
+	/* subtype can be NULL, so not checked */
+	entry = lookup(icon, subtype, TRUE);
+
+	g_return_if_fail(entry != NULL);
+
+	switch (entry->type) {
+	case GNOME_STOCK_PIXMAP_TYPE_DATA:
+		im = gdk_imlib_create_image_from_xpm_data (entry->data.xpm_data);
+		break;
+	case GNOME_STOCK_PIXMAP_TYPE_PATH:
+		im = gdk_imlib_load_image (entry->path.pathname);
+		break;
+	case GNOME_STOCK_PIXMAP_TYPE_IMLIB_SCALED:
+	case GNOME_STOCK_PIXMAP_TYPE_IMLIB:
+		im = gdk_imlib_create_image_from_data ((gchar *)entry->imlib.rgb_data, NULL,
+						       entry->imlib.width, entry->imlib.height);
+		break;
+	 default:
+		im = NULL;
+		break;
+	}
+	
+	g_return_if_fail(im != NULL);
+	
+	/* The imlib images use a color to encode the shape, use it */
+	gdk_imlib_set_image_shape (im, &shape_color);
+
+	/* Render the image, return it */
+	gdk_imlib_render (im, im->rgb_width, im->rgb_height);
+
+	*pixmap = gdk_imlib_move_image (im);
+	*mask = gdk_imlib_move_mask (im);
+
+	gdk_imlib_destroy_image (im);
+}
