@@ -56,7 +56,7 @@
  *  calling gnome_stock_pixmap_change. An application should check (by
  *  calling gnome_stock_pixmap_checkfor), if the current theme supports
  *  an uncommon icon, before using it. The only icons an app can rely
- *  on, are those defined in this haeder file.
+ *  on, are those defined in this header file.
  *
  *  We now have stock buttons too. To use them, just replace any
  *  gtk_button_new{_with_label} with
@@ -142,6 +142,24 @@ BEGIN_GNOME_DECLS
 #define GNOME_STOCK_PIXMAP_NOT         "Not"
 #define GNOME_STOCK_PIXMAP_CONVERT     "Convert"
 #define GNOME_STOCK_PIXMAP_JUMP_TO     "Jump To"
+#define GNOME_STOCK_PIXMAP_UP          "Up"
+#define GNOME_STOCK_PIXMAP_DOWN        "Down"
+#define GNOME_STOCK_PIXMAP_TOP         "Top"
+#define GNOME_STOCK_PIXMAP_BOTTOM      "Bottom"
+#define GNOME_STOCK_PIXMAP_ATTACH      "Attach"
+#define GNOME_STOCK_PIXMAP_INDEX       "Index"
+#define GNOME_STOCK_PIXMAP_FONT        "Font"
+#define GNOME_STOCK_PIXMAP_EXEC        "Exec"
+
+#define GNOME_STOCK_PIXMAP_ALIGN_LEFT  "Left"
+#define GNOME_STOCK_PIXMAP_ALIGN_RIGHT "Right"
+#define GNOME_STOCK_PIXMAP_ALIGN_CENTER "Center"
+#define GNOME_STOCK_PIXMAP_ALIGN_JUSTIFY "Justify"
+
+#define GNOME_STOCK_PIXMAP_TEXT_BOLD    "Bold"
+#define GNOME_STOCK_PIXMAP_TEXT_ITALIC  "Italic"
+#define GNOME_STOCK_PIXMAP_TEXT_UNDERLINE "Underline"
+#define GNOME_STOCK_PIXMAP_TEXT_STRIKEOUT "Strikeout"
 
 #define GNOME_STOCK_PIXMAP_EXIT        GNOME_STOCK_PIXMAP_QUIT
 
@@ -312,6 +330,7 @@ struct _GnomeStock {
 	GnomePixmap *regular, *disabled, *focused; /* pixmap cache */
 	GnomePixmap *current;
 	char *icon;
+	guint32 c_regular, c_disabled, c_focused;
 };
 
 struct _GnomeStockClass {
@@ -339,9 +358,20 @@ GnomePixmap           *gnome_stock_pixmap          (GtkWidget *window,
 #endif
 
 /* just fetch a GnomeStock(PixmapWidget) */
+/* It is possible to specify a filename instead of an icon name. Gnome stock
+ * will use gnome_pixmap_file to find the pixmap and return a GnomeStock widget
+ * from that file. */
 GtkWidget             *gnome_stock_pixmap_widget   (GtkWidget *window,
                                                     const char *icon);
 
+/* This function loads that file scaled to the specified size. Unlike
+ * gnome_pixmap_new_from_file_at_size this function uses antializing and stuff
+ * to scale the pixmap */
+GtkWidget             *gnome_stock_pixmap_widget_at_size(GtkWidget *window,
+							 const char *icon,
+				  			 guint width,
+							 guint height);
+
 /* change the icon/look of a GnomeStockPixmapWidget */
 #if USE_NEW_GNOME_STOCK
 void gnome_stock_pixmap_widget_set_icon(GnomeStock *widget,
@@ -387,6 +417,9 @@ void		      gnome_button_can_default    (GtkButton *button,
 #define GNOME_STOCK_BUTTON_HELP   "Button_Help"
 #define GNOME_STOCK_BUTTON_NEXT   "Button_Next"
 #define GNOME_STOCK_BUTTON_PREV   "Button_Prev"
+#define GNOME_STOCK_BUTTON_UP     "Button_Up"
+#define GNOME_STOCK_BUTTON_DOWN   "Button_Down"
+#define GNOME_STOCK_BUTTON_FONT   "Button_Font"
 
 /* returns a default button widget for dialogs */
 GtkWidget             *gnome_stock_button          (const char *type);
@@ -449,6 +482,24 @@ GtkWidget             *gnome_stock_or_ordinary_button (const char *type);
 #define GNOME_STOCK_MENU_BOOK_OPEN    "Menu_Book Open"
 #define GNOME_STOCK_MENU_CONVERT      "Menu_Convert"
 #define GNOME_STOCK_MENU_JUMP_TO      "Menu_Jump To"
+#define GNOME_STOCK_MENU_UP           "Menu_Up"
+#define GNOME_STOCK_MENU_DOWN         "Menu_Down"
+#define GNOME_STOCK_MENU_TOP          "Menu_Top"
+#define GNOME_STOCK_MENU_BOTTOM       "Menu_Bottom"
+#define GNOME_STOCK_MENU_ATTACH       "Menu_Attach"
+#define GNOME_STOCK_MENU_INDEX        "Menu_Index"
+#define GNOME_STOCK_MENU_FONT         "Menu_Font"
+#define GNOME_STOCK_MENU_EXEC         "Menu_Exec"
+
+#define GNOME_STOCK_MENU_ALIGN_LEFT     "Menu_Left"
+#define GNOME_STOCK_MENU_ALIGN_RIGHT    "MEnu_Right"
+#define GNOME_STOCK_MENU_ALIGN_CENTER   "Menu_Center"
+#define GNOME_STOCK_MENU_ALIGN_JUSTIFY  "Menu_Justify"
+
+#define GNOME_STOCK_MENU_TEXT_BOLD      "Menu_Bold"
+#define GNOME_STOCK_MENU_TEXT_ITALIC    "Menu_Italic"
+#define GNOME_STOCK_MENU_TEXT_UNDERLINE "Menu_Underline"
+#define GNOME_STOCK_MENU_TEXT_STRIKEOUT "Menu_Strikeout"
 
 #define GNOME_STOCK_MENU_EXIT     GNOME_STOCK_MENU_QUIT
 
@@ -489,6 +540,14 @@ void                   gnome_stock_menu_accel_parse(const char *section);
  */
 GtkWidget *gnome_stock_transparent_window (const char *icon, const char *subtype);
 
+/*
+ * Return a GdkPixmap and GdkMask for a stock pixmap
+ */
+void gnome_stock_pixmap_gdk (const char *icon,
+			     const char *subtype,
+			     GdkPixmap **pixmap,
+			     GdkPixmap **mask);
+
 END_GNOME_DECLS
 
 #endif /* GNOME_STOCK_H */
