@@ -70,18 +70,14 @@ gtk_ted_class_init (GtkTedClass *class)
 	object_class->destroy = gtk_ted_destroy;
 }
 
-static GtkWidget *
+static void
 gtk_ted_init (GtkTed *ted)
 {
-	GtkWidget *table;
-
 	ted->dialog_name = NULL;
 	ted->widget_box  = NULL;
 	ted->need_gui = 0;
 	ted->in_gui   = 0;
 	ted->widgets = g_hash_table_new (g_str_hash, g_str_equal);
-
-	return table;
 }
 
 guint
@@ -323,7 +319,7 @@ gtk_ted_prepare_editable_widget (struct ted_widget_info *wi, GtkWidget *ted_tabl
 {
 	GtkWidget *w = wi->widget;
 	GtkWidget *window;
-	int tag;
+	gint tag;
 	
 	/* window is the actual widget, as it is packed inside a GtkAdjustment */
 	window = GTK_WIDGET (GTK_BIN (w)->child);
@@ -332,7 +328,7 @@ gtk_ted_prepare_editable_widget (struct ted_widget_info *wi, GtkWidget *ted_tabl
 	 * a gmc-specific hack;  the right fix is to find a way to disconnect the
 	 * "clicked" event
 	 */
-	tag = (int) gtk_object_get_data (GTK_OBJECT (window), "click-signal-tag");
+	tag = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (window), "click-signal-tag"));
 	if (tag)
 		gtk_signal_disconnect (GTK_OBJECT (window), tag);
 	
@@ -392,7 +388,7 @@ gtk_ted_add_control_col_at (GtkTed *t, int col)
 	GtkWidget *l;
 	char buf [40];
 
-	sprintf (buf, "%d", col);
+	g_snprintf (buf, sizeof(buf), "%d", col);
 	l = gtk_label_new (buf);
 	gtk_table_attach (GTK_TABLE (t), l,
 			  MAP_WIDGET_COL (col), MAP_WIDGET_COL (col)+1,
@@ -407,7 +403,7 @@ gtk_ted_add_control_row_at (GtkTed *t, int row)
 	GtkWidget *l;
 	char buf [40];
 
-	sprintf (buf, "%d", row);
+	g_snprintf (buf, sizeof(buf), "%d", row);
 	l = gtk_label_new (buf);
 	gtk_table_attach (GTK_TABLE (t), l,
 			  0, 1,
@@ -575,7 +571,7 @@ gtk_ted_add_frame (GtkWidget *widget, GtkTed *ted)
 	gtk_widget_show (f);
 	gtk_container_add (GTK_CONTAINER (e), f);
 
-	sprintf (my_name, "Frame-%d", frame_count++);
+	g_snprintf (my_name, sizeof(my_name), "Frame-%d", frame_count++);
 	wi = gtk_ted_widget_info_new (e, my_name, 0, 0);
 	wi->type = frame_widget;
 	gtk_object_set_data (GTK_OBJECT (e), "ted_widget_info", wi);
@@ -601,7 +597,7 @@ gtk_ted_add_label (GtkWidget *widget, GtkTed *ted)
 	gtk_widget_show (f);
 	gtk_container_add (GTK_CONTAINER (e), f);
 
-	sprintf (my_name, "Label-%d", label_count++);
+	g_snprintf (my_name, sizeof(my_name), "Label-%d", label_count++);
 	wi = gtk_ted_widget_info_new (e, my_name, 0, 0);
 	g_hash_table_insert (ted->widgets, wi->name, wi);
 	wi->type = label_widget;
@@ -623,13 +619,13 @@ gtk_ted_add_separator (GtkWidget *widget, GtkTed *ted)
 	gtk_widget_show (s);
 	gtk_container_add (GTK_CONTAINER (e), s);
 
-	sprintf (my_name, "Separator-%d", sep_count++);
+	g_snprintf (my_name, sizeof(my_name), "Separator-%d", sep_count++);
 	wi = gtk_ted_widget_info_new (e, my_name, 0, 0);
 	g_hash_table_insert (ted->widgets, wi->name, wi);
 	wi->type = sep_widget;
 	gtk_object_set_data (GTK_OBJECT (e), "ted_widget_info", wi);
 	
-	gtk_container_border_width (GTK_CONTAINER (e), 2);
+	gtk_container_set_border_width (GTK_CONTAINER (e), 2);
 	gtk_ted_add (ted, e, my_name);
 }
 
@@ -734,7 +730,9 @@ gtk_ted_save (GtkWidget *widget, GtkTed *ted)
 
 		key = g_copy_strings ("=", filename, "=/", ted->dialog_name, "-", kind, "-", wi->name, "/", NULL);
 		gnome_config_push_prefix (key);
-		sprintf (buffer, "%d,%d,%d,%d", wi->start_col, wi->start_row, wi->col_span, wi->row_span);
+		g_snprintf (buffer, sizeof(buffer), "%d,%d,%d,%d",
+			    wi->start_col, wi->start_row,
+			    wi->col_span, wi->row_span);
 		gnome_config_set_string ("geometry", buffer);
 #if 0
 		gnome_config_set_string ("flags_x", gtk_ted_render_flags (wi->flags_x, wi->lcr));
@@ -995,9 +993,9 @@ gtk_ted_update_position (struct ted_widget_info *wi)
 		gtk_ted_attach (ted, wi->widget, wi);
 		gtk_widget_unref (wi->widget);
 
-		sprintf (buf, "%d", wi->col_span);
+		g_snprintf (buf, sizeof(buf), "%d", wi->col_span);
 		gtk_label_set (GTK_LABEL (wi->label_span_x), buf);
-		sprintf (buf, "%d", wi->row_span);
+		g_snprintf (buf, sizeof(buf), "%d", wi->row_span);
 		gtk_label_set (GTK_LABEL (wi->label_span_y), buf);
 	}
 }
@@ -1098,7 +1096,7 @@ gtk_ted_orient_cb (GtkWidget *w, void *data)
 }
 
 static void
-gtk_ted_setup_radio (GtkWidget *vbox, GtkWidget *widget, char *str, struct ted_widget_info *wi, int idx, int active)
+gtk_ted_setup_radio (GtkWidget *vbox, GtkWidget *widget, gchar *str, struct ted_widget_info *wi, gint idx, gint active)
 {
 	if (active)
 		gtk_widget_set_state (GTK_WIDGET (widget), GTK_STATE_ACTIVE);
@@ -1108,7 +1106,7 @@ gtk_ted_setup_radio (GtkWidget *vbox, GtkWidget *widget, char *str, struct ted_w
 	gtk_signal_connect (GTK_OBJECT (widget), "toggled",
 			    GTK_SIGNAL_FUNC (gtk_ted_orient_cb), str);
 	gtk_object_set_data (GTK_OBJECT (widget), "ted_wi", wi);
-	gtk_object_set_data (GTK_OBJECT (widget), "index", (void *) idx);
+	gtk_object_set_data (GTK_OBJECT (widget), "index", GINT_TO_POINTER (idx));
 			     
 	gtk_box_pack_start_defaults (GTK_BOX (vbox), widget);
 	gtk_widget_show (widget);
@@ -1151,13 +1149,13 @@ gtk_ted_span_control (char *str, struct ted_widget_info *wi, int is_y)
 	gtk_container_add (GTK_CONTAINER (vbox), frame);
 	
 	
-	gtk_container_border_width (GTK_CONTAINER (frame), 2);
-	gtk_container_border_width (GTK_CONTAINER (hbox), 2);
+	gtk_container_set_border_width (GTK_CONTAINER (frame), 2);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
 	more  = gtk_button_new_with_label ("+");
 	gtk_widget_show (more);
 	gtk_box_pack_end_defaults (GTK_BOX (hbox), more);
 
-	sprintf (buf, "%d", is_y ? wi->row_span : wi->col_span);
+	g_snprintf (buf, sizeof(buf), "%d", is_y ? wi->row_span : wi->col_span);
 	label = gtk_label_new (buf);
 	gtk_box_pack_end_defaults (GTK_BOX (hbox), label);
 	gtk_widget_show (label);
@@ -1181,14 +1179,14 @@ gtk_ted_span_control (char *str, struct ted_widget_info *wi, int is_y)
 static void
 gtk_ted_pos_toggle (GtkWidget *widget, struct ted_widget_info *wi)
 {
-	int val = (int) gtk_object_get_data (GTK_OBJECT (widget), "value");
+	int val = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget), "value"));
 
 	wi->sticky ^= val;
 	gtk_ted_update_position (wi);
 }
 
 static void
-gtk_ted_pos_prep (GtkWidget *box, char *str, int val, struct ted_widget_info *wi)
+gtk_ted_pos_prep (GtkWidget *box, gchar *str, gint val, struct ted_widget_info *wi)
 {
 	GtkWidget *w;
 	
@@ -1197,7 +1195,7 @@ gtk_ted_pos_prep (GtkWidget *box, char *str, int val, struct ted_widget_info *wi
 	gtk_box_pack_start_defaults (GTK_BOX (box), w);
 	gtk_widget_show (w);
 	gtk_signal_connect (GTK_OBJECT (w), "toggled", GTK_SIGNAL_FUNC (gtk_ted_pos_toggle), wi);
-	gtk_object_set_data (GTK_OBJECT (w), "value", (void *) val);
+	gtk_object_set_data (GTK_OBJECT (w), "value", GINT_TO_POINTER (val));
 }
 
 static GtkWidget *
@@ -1438,7 +1436,7 @@ gtk_ted_new_layout (char *name, char *layout)
 	ted = gtk_type_new (gtk_ted_get_type ());
 	ted->dialog_name = g_strdup (name);
 	gtk_ted_load_layout (ted, layout);
-	gtk_container_border_width (GTK_CONTAINER (ted), 6);
+	gtk_container_set_border_width (GTK_CONTAINER (ted), 6);
 	
 	return GTK_WIDGET (ted);
 }
