@@ -30,7 +30,7 @@ message_dlg(GtkWidget *widget, gpointer data)
 	  gtk_signal_connect (GTK_OBJECT (box), "clicked",
 			      GTK_SIGNAL_FUNC (message_dlg_clicked), NULL);
 
-	  gnome_dialog_set_modal (GNOME_DIALOG(box));
+	  gtk_window_set_modal (GTK_WINDOW(box),TRUE);
 	  gnome_dialog_close_hides(GNOME_DIALOG(box), TRUE);
 	}
 	gtk_widget_show (box);
@@ -49,7 +49,7 @@ prop_apply(GtkWidget *box, int n)
 	char s[256];
 
 	if (n != -1) {
-		sprintf(s, "Applied changed on page #%d", n + 1);
+		g_snprintf(s, sizeof(s), "Applied changed on page #%d", n + 1);
 		gtk_widget_show(gnome_message_box_new(s, "info",
 				GNOME_STOCK_BUTTON_OK, NULL));
 	}
@@ -406,29 +406,13 @@ toggle_button(GtkWidget *button, GnomeStockPixmapWidget *w)
 
 
 
-#undef USE_LAMP
 #undef USE_BUTTON
 
-#ifdef USE_LAMP
-static void
-lamp_value_changed(GtkAdjustment *adj, GnomeLamp *lamp)
-{
-	GdkColor c;
-
-	c.red = (adj->value / 100.0) * 0xf000;
-	c.green = (1.0 - (adj->value / 100.0)) * 0xf000;
-	c.blue = (c.red < 0x8000) ? c.red + 0x8000 : c.green + 0x8000;
-	gnome_lamp_set_color(lamp, &c);
-}
-#endif /* USE_LAMP */
 
 static void
 fill_table(GtkWidget *window, GtkTable *table)
 {
 	GtkWidget *w, *button;
-#ifdef USE_LAMP
-	GtkWidget *lamp;
-#endif
 	gint row, column;
 
 	row = column = 0;
@@ -569,6 +553,107 @@ fill_table(GtkWidget *window, GtkTable *table)
 	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
 
 	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_UP));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_UP));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Up");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_DOWN));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_DOWN));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Down");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_TOP));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_TOP));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Top");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_BOTTOM));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_BOTTOM));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Bottom");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_EXEC));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_EXEC));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Exec");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
+	row += 3;
+	column = 0;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_ALIGN_LEFT));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_ALIGN_LEFT));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Left");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_ALIGN_RIGHT));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_ALIGN_RIGHT));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Right");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_ALIGN_CENTER));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_ALIGN_CENTER));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Center");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_ALIGN_JUSTIFY));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_ALIGN_JUSTIFY));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Justify");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
+	row += 3;
+	column = 0;
 	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_HOME));
 	gtk_widget_show(w);
 	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
@@ -601,6 +686,17 @@ fill_table(GtkWidget *window, GtkTable *table)
 	gtk_widget_show(w);
 	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
 
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_INDEX));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_INDEX));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Index");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
 	column++;
 	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_MIC));
 	gtk_widget_show(w);
@@ -634,6 +730,17 @@ fill_table(GtkWidget *window, GtkTable *table)
 	gtk_widget_show(w);
 	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
 
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_FONT));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_FONT));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Font");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
 	column = 0; row += 3;
 	button = gtk_button_new();
 	gtk_widget_show(button);
@@ -719,6 +826,17 @@ fill_table(GtkWidget *window, GtkTable *table)
 	gtk_widget_show(w);
 	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
 
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_ATTACH));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_MENU_ATTACH));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+	w = gtk_label_new("Attach");
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
+
 	column++;
 	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, GNOME_STOCK_PIXMAP_CONVERT));
 	gtk_widget_show(w);
@@ -829,6 +947,70 @@ fill_table(GtkWidget *window, GtkTable *table)
 	gtk_widget_show(w);
 	gtk_table_attach_defaults(table, w, column, column + 1, row + 2, row + 3);
 
+#undef COMPARE_TO_IMLIB
+#undef TRY_ENLARGE
+
+#define FILENAME "gnome-audio2.png"
+#ifndef COMPARE_TO_IMLIB
+	column++;
+#else
+	row += 3;
+	column = 0;
+#endif
+	w = GTK_WIDGET(gnome_stock_pixmap_widget(window, FILENAME));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 2);
+	w = gtk_label_new(FILENAME);
+	gtk_widget_show(w);
+#ifdef TRY_ENLARGE
+	gtk_table_attach_defaults(table, w, column, column + 3, row + 2, row + 3);
+#else
+	gtk_table_attach_defaults(table, w, column, column + 2, row + 2, row + 3);
+#endif
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget_at_size(window, FILENAME, 24, 24));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_stock_pixmap_widget_at_size(window, FILENAME, 16, 16));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+#ifdef TRY_ENLARGE
+	/* XXX. yikes, that might crash */
+	column++;
+	w = GTK_WIDGET(gnome_stock_pixmap_widget_at_size(window, FILENAME, 64, 64));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 2);
+#endif
+#ifdef COMPARE_TO_IMLIB
+#define FILENAME2 (gnome_pixmap_file(FILENAME))
+	column++;
+	w = GTK_WIDGET(gnome_pixmap_new_from_file(FILENAME2));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 2);
+	w = gtk_label_new(FILENAME " via Imlib");
+	gtk_widget_show(w);
+#ifdef TRY_ENLARGE
+	gtk_table_attach_defaults(table, w, column, column + 3, row + 2, row + 3);
+#else
+	gtk_table_attach_defaults(table, w, column, column + 2, row + 2, row + 3);
+#endif
+	column++;
+	w = GTK_WIDGET(gnome_pixmap_new_from_file_at_size(FILENAME2, 24, 24));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
+	w = GTK_WIDGET(gnome_pixmap_new_from_file_at_size(FILENAME2, 16, 16));
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
+#ifdef TRY_ENLARGE
+	column++;
+	w = gnome_pixmap_new_from_file_at_size(FILENAME2, 64, 64);
+	gtk_widget_show(w);
+	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 2);
+#endif
+#undef FILENAME2
+#endif /* COMPARE_TO_IMLIB */
+#undef FILENAME
+
 #ifdef USE_BUTTON
 	column++;
 	w = gnome_pixmap_new_from_file(gnome_pixmap_file("gnome-unknown.png"));
@@ -842,26 +1024,6 @@ fill_table(GtkWidget *window, GtkTable *table)
 	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
 #endif /* USE_BUTTON */
 
-#ifdef USE_LAMP
-	column++;
-	lamp = gnome_lamp_new();
-	gtk_widget_show(lamp);
-	gtk_table_attach_defaults(table, lamp, column, column + 1, row, row + 1);
-	w = (GtkWidget *)gtk_adjustment_new(0.0, 0.0, 110.0, 1.0, 10.0, 10.0);
-	gtk_signal_connect(GTK_OBJECT(w), "value_changed",
-			   (GtkSignalFunc)lamp_value_changed, lamp);
-	lamp_value_changed(GTK_ADJUSTMENT(w), GNOME_LAMP(lamp));
-	w = gtk_hscale_new(GTK_ADJUSTMENT(w));
-	gtk_widget_show(w);
-	gtk_table_attach_defaults(table, w, column, column + 1, row + 1, row + 2);
-
-	column++;
-	w = gnome_lamp_new();
-	gnome_lamp_set_sequence(GNOME_LAMP(w), "RPBAGABPRYGABP");
-	gtk_widget_show(w);
-	gtk_table_attach_defaults(table, w, column, column + 1, row, row + 1);
-#endif /* USE_LAMP */
-
 	gtk_table_set_col_spacings(table, 10);
 	gtk_table_set_row_spacing(table, 2, 10);
 	gtk_table_set_row_spacing(table, 5, 10);
@@ -875,7 +1037,8 @@ main(int argc, char **argv)
 {
 	GtkWidget *window, *hbox, *vbox, *table, *w;
 
-	gnome_init("stock_demo", NULL, argc, argv, 0, NULL);
+	gnome_init("stock_demo", VERSION, argc, argv);
+
 	textdomain(PACKAGE);
 
 	window = gnome_app_new("Gnome Stock Test", "Gnome Stock Test");
@@ -907,59 +1070,65 @@ main(int argc, char **argv)
 	gtk_container_add(GTK_CONTAINER(frame), table);
 	fill_table(window, GTK_TABLE(table));
 
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_container_border_width(GTK_CONTAINER(hbox), 5);
+	hbox = gtk_hbutton_box_new();
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
+	gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
 	gtk_widget_show(hbox);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
 
-	w = gnome_stock_button(GNOME_STOCK_BUTTON_NEXT);
+	w = gnome_stock_button(GNOME_STOCK_BUTTON_YES);
 	gtk_widget_show(w);
-	gtk_box_pack_end(GTK_BOX(hbox), w, FALSE, FALSE, 3);
-
-	w = gnome_stock_button(GNOME_STOCK_BUTTON_PREV);
-	gtk_widget_show(w);
-	gtk_box_pack_end(GTK_BOX(hbox), w, FALSE, FALSE, 3);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 3);
 
 	w = gnome_stock_button(GNOME_STOCK_BUTTON_NO);
 	gtk_widget_show(w);
-	gtk_box_pack_end(GTK_BOX(hbox), w, FALSE, FALSE, 3);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 3);
 
-	w = gnome_stock_button(GNOME_STOCK_BUTTON_YES);
+	w = gnome_stock_button(GNOME_STOCK_BUTTON_PREV);
 	gtk_widget_show(w);
-	gtk_box_pack_end(GTK_BOX(hbox), w, FALSE, FALSE, 3);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 3);
 
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_container_border_width(GTK_CONTAINER(hbox), 5);
+	w = gnome_stock_button(GNOME_STOCK_BUTTON_NEXT);
+	gtk_widget_show(w);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 3);
+
+	w = gnome_stock_button(GNOME_STOCK_BUTTON_FONT);
+	gtk_widget_show(w);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 3);
+
+	hbox = gtk_hbutton_box_new();
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
+	gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
 	gtk_widget_show(hbox);
 	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
 
-	w = gnome_stock_button(GNOME_STOCK_BUTTON_CLOSE);
+	w = gnome_stock_button(GNOME_STOCK_BUTTON_OK);
 	gtk_widget_show(w);
-	gtk_box_pack_end(GTK_BOX(hbox), w, FALSE, FALSE, 3);
-	gtk_signal_connect_object(GTK_OBJECT(w), "clicked",
-				  (GtkSignalFunc)message_dlg,
-				  NULL);
-
-	w = gnome_stock_button(GNOME_STOCK_BUTTON_HELP);
-	gtk_widget_show(w);
-	gtk_box_pack_end(GTK_BOX(hbox), w, FALSE, FALSE, 3);
-
-	w = gnome_stock_button(GNOME_STOCK_BUTTON_CANCEL);
-	gtk_widget_show(w);
-	gtk_box_pack_end(GTK_BOX(hbox), w, FALSE, FALSE, 3);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 3);
 	gtk_signal_connect(GTK_OBJECT(w), "clicked",
-			   (GtkSignalFunc)tb_insens, NULL);
+			   (GtkSignalFunc)tb_sens, NULL);
 
 	w = gnome_stock_button(GNOME_STOCK_BUTTON_APPLY);
 	gtk_widget_show(w);
-	gtk_box_pack_end(GTK_BOX(hbox), w, FALSE, FALSE, 3);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 3);
 	gtk_widget_set_sensitive(w, FALSE);
 
-	w = gnome_stock_button(GNOME_STOCK_BUTTON_OK);
+	w = gnome_stock_button(GNOME_STOCK_BUTTON_CANCEL);
 	gtk_widget_show(w);
-	gtk_box_pack_end(GTK_BOX(hbox), w, FALSE, FALSE, 3);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 3);
 	gtk_signal_connect(GTK_OBJECT(w), "clicked",
-			   (GtkSignalFunc)tb_sens, NULL);
+			   (GtkSignalFunc)tb_insens, NULL);
+
+	w = gnome_stock_button(GNOME_STOCK_BUTTON_HELP);
+	gtk_widget_show(w);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 3);
+
+	w = gnome_stock_button(GNOME_STOCK_BUTTON_CLOSE);
+	gtk_widget_show(w);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 3);
+	gtk_signal_connect_object(GTK_OBJECT(w), "clicked",
+				  (GtkSignalFunc)message_dlg,
+				  NULL);
 
 	gnome_app_set_contents(GNOME_APP(window), vbox);
 	gnome_app_set_menus(GNOME_APP(window),
