@@ -32,6 +32,9 @@
 static const gchar *authors[] = {
 	"Richard Hestilow",
 	"Federico Mena",
+	"Eckehard Berns",
+	"Havoc Pennington",
+	"Miguel de Icaza",
 	NULL
 };
 
@@ -93,8 +96,6 @@ static GnomeUIInfo file_menu[] = {
 };
 
 static GnomeUIInfo help_menu[] = {
-        { GNOME_APP_UI_HELP, NULL, NULL, NULL, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
         { GNOME_APP_UI_ITEM, "About...", NULL, create_about, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0,
 	  NULL },
@@ -122,22 +123,9 @@ create_newwin(gboolean normal, gchar *appname, gchar *title)
         };
 
 	gnome_app_create_menus_with_data (GNOME_APP(app), main_menu, app);
-	gtk_menu_item_right_justify(GTK_MENU_ITEM(main_menu[1].widget));
 	return GTK_WIDGET(app);
 }
 
-static void
-create_calendar(void)
-{
-	GtkWidget *app;
-	GtkWidget *cal;
-	app=create_newwin(TRUE,"testGNOME","calendar");
-	cal=gtk_calendar_new();
-	gnome_app_set_contents(GNOME_APP(app),cal);
-	gtk_widget_show(cal);
-	gtk_widget_show(app);
-}
-
 static void
 create_calc(void)
 {
@@ -244,38 +232,107 @@ create_color_picker (void)
 	gtk_widget_show (app);
 }
 
-static void
-color_changed_cb( GnomeColorSelector *widget, gchar **color )
+static void guru_finish(GnomeGuru* guru, GtkWidget* destroyme)
 {
-        char *tmp;
-        int r,g,b;
-
-        tmp = malloc(24);
-        if( !tmp )
-        {
-                g_warning( "Can't allocate memory for color\n" );
-                return;
-        }
-        gnome_color_selector_get_color_int(
-                widget, &r, &g, &b, 255 );
-
-        sprintf( tmp, "#%02x%02x%02x", r, g, b );
-        *color = tmp;
+  gnome_ok_dialog("Guru successfully completed, emitted 'finished'");
+  gtk_widget_destroy(destroyme);
 }
 
-static void
-create_colorsel(void)
+static void guru_cancel(GnomeGuru* guru, GtkWidget* destroyme)
 {
-	GnomeColorSelector *colorsel;
-	GtkWidget *colbutton;
+  gnome_ok_dialog("Guru cancelled");
+  gtk_widget_destroy(destroyme);
+}
+
+static void 
+create_guru(void)
+{
+  /* We want to test both in a dialog, and without a dialog. */
+  GtkWidget* guru;
+  GtkWidget* page;
+  GtkWidget* dialog;
+  GtkWidget* graphic;
+  GtkWidget* window;
+  gchar* pix;
+
+  pix = gnome_pixmap_file("gnome-logo-large.png");
+  graphic = gnome_pixmap_new_from_file_at_size(pix, 100, 300);
+  g_free(pix);
+
+  guru = gnome_guru_new("Test Gnome Guru (without dialog)",
+			graphic,
+			NULL);
+  
+  window = create_newwin(TRUE, "testGNOME", "Gnome Guru Test Window");
+  
+  gnome_app_set_contents(GNOME_APP(window), guru);
+
+  page = gtk_label_new("Page one");
+  gnome_guru_append_page(GNOME_GURU(guru), "First page", page);
+  page = gtk_label_new("Page two");
+  gnome_guru_append_page(GNOME_GURU(guru), "Second page", page);
+  page = gtk_label_new("Page three");
+  gnome_guru_append_page(GNOME_GURU(guru), "Third page", page);  
+  page = gtk_label_new("Page four");
+  gnome_guru_append_page(GNOME_GURU(guru), "Fourth page", page);  
+
+  gtk_signal_connect(GTK_OBJECT(guru), "finished", 
+		     GTK_SIGNAL_FUNC(guru_finish),
+		     window);
+
+  gtk_signal_connect(GTK_OBJECT(guru), "cancelled", 
+		     GTK_SIGNAL_FUNC(guru_cancel),
+		     window);
+
+  gtk_widget_show(guru);
+  gtk_widget_show(window);
+
+  dialog = gnome_dialog_new(_("Guru-in-a-dialog test"),
+			    NULL);
+ 
+  pix = gnome_pixmap_file("gnome-logo-large.png");
+  graphic = gnome_pixmap_new_from_file_at_size(pix, 100, 300);
+  g_free(pix);
+
+  guru = gnome_guru_new("Test Gnome Guru (without dialog)",
+			graphic,
+			GNOME_DIALOG(dialog));
+
+  page = gtk_label_new("Page one");
+  gnome_guru_append_page(GNOME_GURU(guru), "First page", page);
+  page = gtk_label_new("Page two");
+  gnome_guru_append_page(GNOME_GURU(guru), "Second page", page);
+  page = gtk_label_new("Page three");
+  gnome_guru_append_page(GNOME_GURU(guru), "Third page", page);  
+  page = gtk_label_new("Page four");
+  gnome_guru_append_page(GNOME_GURU(guru), "Fourth page", page);  
+
+  gtk_signal_connect(GTK_OBJECT(guru), "finished", 
+		     GTK_SIGNAL_FUNC(guru_finish),
+		     dialog);
+
+  gtk_signal_connect(GTK_OBJECT(guru), "cancelled", 
+		     GTK_SIGNAL_FUNC(guru_cancel),
+		     dialog);
+  
+  gtk_widget_show(guru);
+  gtk_widget_show(dialog); 
+}
+
+/* 
+ * GnomePaperSelector
+ */
+
+static void
+create_papersel(void)
+{
+	GtkWidget *papersel;
 	GtkWidget *app;
-	gchar *dummy;
-	app = create_newwin(TRUE,"testGNOME","Color Selection");
-	colorsel = gnome_color_selector_new( (SetColorFunc)color_changed_cb,
-					     &dummy);
-	colbutton = gnome_color_selector_get_button( colorsel );
-	gnome_app_set_contents(GNOME_APP(app),colbutton);
-	gtk_widget_show(colbutton);
+
+	app = create_newwin(TRUE,"testGNOME","Paper Selection");
+	papersel = gnome_paper_selector_new( );
+	gnome_app_set_contents(GNOME_APP(app),papersel);
+	gtk_widget_show(papersel);
 	gtk_widget_show(app);
 }
 
@@ -312,11 +369,11 @@ set_to_null(GtkWidget * ignore, GnomeDialog ** d)
 }
 
 static void
-create_test_dialog (GtkWidget * ignored, gboolean * settings)
+create_test_dialog (GtkWidget * button, gboolean * settings)
 {
   static GnomeDialog * dialog = NULL;
   GtkWidget * entry;
-  GtkWidget * button;
+  GtkWidget * app;
 
   if (dialog) {
     g_print("Previous dialog was not destroyed, destroying...\n");
@@ -324,10 +381,17 @@ create_test_dialog (GtkWidget * ignored, gboolean * settings)
     dialog = NULL;
   }
 
+  app = gtk_object_get_user_data(GTK_OBJECT(button));
+
+  g_return_if_fail(app != NULL);
+  g_return_if_fail(GTK_IS_WINDOW(app));
+
   dialog = GNOME_DIALOG(gnome_dialog_new( "A Test Dialog", 
 					  GNOME_STOCK_BUTTON_OK,
 					  "Not a stock button",
 					  GNOME_STOCK_BUTTON_CANCEL, NULL ));
+
+  gnome_dialog_set_parent(dialog,GTK_WINDOW(app));
   
   entry = gtk_entry_new();
   button = gtk_button_new_with_label("gnome_dialog_run");
@@ -345,7 +409,7 @@ create_test_dialog (GtkWidget * ignored, gboolean * settings)
   
   if (settings[modal]) {
     g_print("Modal... ");
-    gnome_dialog_set_modal(dialog);
+    gtk_window_set_modal(GTK_WINDOW (dialog), TRUE);
   }
   if (settings[just_hide]) {
     g_print("Close hides... ");
@@ -407,6 +471,9 @@ create_dialog(void)
   gtk_box_pack_start(GTK_BOX(hbox), toggle, FALSE, FALSE, GNOME_PAD);
 
   button = gtk_button_new_with_label("Create the dialog");
+
+  gtk_object_set_user_data(GTK_OBJECT(button),app);
+
   gtk_signal_connect(GTK_OBJECT(button), "clicked", 
 		     GTK_SIGNAL_FUNC(create_test_dialog), 
 		     &settings[0]);
@@ -416,14 +483,107 @@ create_dialog(void)
   gtk_widget_show_all(app);
 }
 
+static void
+file_entry_update_files(GtkWidget *w, GnomeFileEntry *fentry)
+{
+	char *p;
+	char *pp;
+
+	GtkLabel *l1 = gtk_object_get_data(GTK_OBJECT(w),"l1");
+	GtkLabel *l2 = gtk_object_get_data(GTK_OBJECT(w),"l2");
+	
+	p = gnome_file_entry_get_full_path(fentry,FALSE);
+	pp = g_strconcat("File name: ",p,NULL);
+	gtk_label_set(l1,pp);
+	g_free(pp);
+	if(p) g_free(p);
+
+	p = gnome_file_entry_get_full_path(fentry,TRUE);
+	pp = g_strconcat("File name(if exists only): ",p,NULL);
+	gtk_label_set(l2,pp);
+	g_free(pp);
+	if(p) g_free(p);
+}
+
+static void
+file_entry_modal_toggle(GtkWidget *w, GnomeFileEntry *fentry)
+{
+	gnome_file_entry_set_modal(fentry,GTK_TOGGLE_BUTTON(w)->active);
+}
+
+static void
+file_entry_directory_toggle(GtkWidget *w, GnomeFileEntry *fentry)
+{
+	gnome_file_entry_set_directory(fentry,GTK_TOGGLE_BUTTON(w)->active);
+}
+
 static void
 create_file_entry(void)
 {
 	GtkWidget *app;
 	GtkWidget *entry;
+	GtkWidget *l1,*l2;
+	GtkWidget *but;
+	GtkWidget *box;
 
 	app = create_newwin(TRUE,"testGNOME","File Entry");
+
+	box = gtk_vbox_new(FALSE,5);
+
 	entry = gnome_file_entry_new("Foo","Bar");
+	gtk_box_pack_start(GTK_BOX(box),entry,FALSE,FALSE,0);
+	
+	l1 = gtk_label_new("File name: ");
+	gtk_box_pack_start(GTK_BOX(box),l1,FALSE,FALSE,0);
+	
+	l2 = gtk_label_new("File name(if exists only): ");
+	gtk_box_pack_start(GTK_BOX(box),l2,FALSE,FALSE,0);
+	
+	but = gtk_button_new_with_label("Update file labels");
+	gtk_object_set_data(GTK_OBJECT(but),"l1",l1);
+	gtk_object_set_data(GTK_OBJECT(but),"l2",l2);
+	gtk_signal_connect(GTK_OBJECT(but),"clicked",
+			   GTK_SIGNAL_FUNC(file_entry_update_files),
+			   entry);
+	gtk_box_pack_start(GTK_BOX(box),but,FALSE,FALSE,0);
+	
+	but = gtk_toggle_button_new_with_label("Make browse dialog modal");
+	gtk_signal_connect(GTK_OBJECT(but),"toggled",
+			   GTK_SIGNAL_FUNC(file_entry_modal_toggle),
+			   entry);
+	gtk_box_pack_start(GTK_BOX(box),but,FALSE,FALSE,0);
+
+	but = gtk_toggle_button_new_with_label("Directory only picker");
+	gtk_signal_connect(GTK_OBJECT(but),"toggled",
+			   GTK_SIGNAL_FUNC(file_entry_directory_toggle),
+			   entry);
+	gtk_box_pack_start(GTK_BOX(box),but,FALSE,FALSE,0);
+
+	gnome_app_set_contents(GNOME_APP(app),box);
+	gtk_widget_show_all(app);
+}
+
+static void
+create_pixmap_entry(void)
+{
+	GtkWidget *app;
+	GtkWidget *entry;
+
+	app = create_newwin(TRUE,"testGNOME","Pixmap Entry");
+	entry = gnome_pixmap_entry_new("Foo","Pixmap",TRUE);
+	gnome_app_set_contents(GNOME_APP(app),entry);
+	gtk_widget_show(entry);
+	gtk_widget_show(app);
+}
+
+static void
+create_icon_entry(void)
+{
+	GtkWidget *app;
+	GtkWidget *entry;
+
+	app = create_newwin(TRUE,"testGNOME","Icon Entry");
+	entry = gnome_icon_entry_new("Foo","Icon");
 	gnome_app_set_contents(GNOME_APP(app),entry);
 	gtk_widget_show(entry);
 	gtk_widget_show(app);
@@ -442,85 +602,193 @@ create_number_entry(void)
 	gtk_widget_show(app);
 }
 
-static void
-create_font_sel(void)
+/* CREATE FONT PICKER
+ */
+static void cfp_ck_UseFont(GtkWidget *widget,GnomeFontPicker *gfp)
 {
-	GtkWidget *fontsel;
+    gboolean show;
+    gint size;
+    
+    show=!gfp->use_font_in_label;
+    size=gfp->use_font_in_label_size;
+    
+    gnome_font_picker_fi_set_use_font_in_label(gfp,show,size);
 
-	fontsel = gnome_font_selector_new();
-	gtk_widget_show(fontsel);
+}
+static void cfp_sp_value_changed(GtkAdjustment *adj,GnomeFontPicker *gfp)
+{
+    gboolean show;
+    gint size;
+    
+    show=gfp->use_font_in_label;
+    size=(gint)adj->value;
+    
+    gnome_font_picker_fi_set_use_font_in_label(gfp,show,size);
+
+}
+
+static void cfp_ck_ShowSize(GtkWidget *widget,GnomeFontPicker *gfp)
+{
+    GtkToggleButton *tb;
+
+    tb=GTK_TOGGLE_BUTTON(widget);
+
+    gnome_font_picker_fi_set_show_size(gfp,tb->active);
+}
+
+static void cfp_set_font(GnomeFontPicker *gfp, gchar *font_name, GtkLabel *label)
+{
+    g_print("Font name: %s\n",font_name);
+    gtk_label_set(label,font_name);
+}
+
+
+static void
+create_font_picker (void)
+{
+        GtkWidget *fontpicker1,*fontpicker2,*fontpicker3;
+        
+        GtkWidget *app;
+        GtkWidget *vbox,*vbox1,*vbox2,*vbox3;
+        GtkWidget *hbox1,*hbox3;
+        GtkWidget *frPixmap,*frFontInfo,*frUser;
+        GtkWidget *lbPixmap,*lbFontInfo,*lbUser;
+        GtkWidget *ckUseFont,*spUseFont,*ckShowSize;
+        GtkAdjustment *adj;
+
+        app = create_newwin(TRUE,"testGNOME","Font Picker");
+
+
+        vbox=gtk_vbox_new(FALSE,5);
+        gtk_container_border_width(GTK_CONTAINER(vbox),5);
+        gnome_app_set_contents(GNOME_APP(app),vbox);
+
+        /* Pixmap */
+        frPixmap=gtk_frame_new(_("Default Pixmap"));
+        gtk_box_pack_start(GTK_BOX(vbox),frPixmap,TRUE,TRUE,0);        
+        vbox1=gtk_vbox_new(FALSE,FALSE);
+        gtk_container_add(GTK_CONTAINER(frPixmap),vbox1);
+        /* GnomeFontPicker with pixmap */
+        fontpicker1 = gnome_font_picker_new();
+        gtk_container_border_width(GTK_CONTAINER(fontpicker1),5);
+        gtk_box_pack_start(GTK_BOX(vbox1),fontpicker1,TRUE,TRUE,0);
+        lbPixmap=gtk_label_new(_("If you choose a font it will appear here"));
+        gtk_box_pack_start(GTK_BOX(vbox1),lbPixmap,TRUE,TRUE,5);
+
+        gtk_signal_connect(GTK_OBJECT(fontpicker1),"font_set",
+                           GTK_SIGNAL_FUNC(cfp_set_font),lbPixmap);
+        
+        /* Font_Info */
+        frFontInfo=gtk_frame_new(_("Font Info"));
+        gtk_box_pack_start(GTK_BOX(vbox),frFontInfo,TRUE,TRUE,0);
+        vbox2=gtk_vbox_new(FALSE,FALSE);
+        gtk_container_border_width(GTK_CONTAINER(vbox2),5);        
+        gtk_container_add(GTK_CONTAINER(frFontInfo),vbox2);
+        
+        /* GnomeFontPicker with fontinfo */
+        hbox1=gtk_hbox_new(FALSE,5);
+        gtk_box_pack_start(GTK_BOX(vbox2),hbox1,FALSE,FALSE,0);
+        
+        ckUseFont=gtk_check_button_new_with_label(_("Use Font in button with size"));
+        gtk_box_pack_start(GTK_BOX(hbox1),ckUseFont,TRUE,TRUE,0);
+
+        adj=GTK_ADJUSTMENT(gtk_adjustment_new(14,5,150,1,1,1));
+        spUseFont=gtk_spin_button_new(adj,1,0);
+        gtk_box_pack_start(GTK_BOX(hbox1),spUseFont,FALSE,FALSE,0);
+
+        ckShowSize=gtk_check_button_new_with_label(_("Show font size"));
+        gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(ckShowSize),TRUE);
+        gtk_box_pack_start(GTK_BOX(vbox2),ckShowSize,FALSE,FALSE,5);
+        
+        fontpicker2 = gnome_font_picker_new();
+        gnome_font_picker_set_mode(GNOME_FONT_PICKER(fontpicker2),GNOME_FONT_PICKER_MODE_FONT_INFO);
+        gtk_box_pack_start(GTK_BOX(vbox2),fontpicker2,TRUE,TRUE,0);
+
+        gtk_signal_connect(GTK_OBJECT(ckUseFont),"toggled",
+                           (GtkSignalFunc)cfp_ck_UseFont,fontpicker2);
+
+        gtk_signal_connect(GTK_OBJECT(ckShowSize),"toggled",
+                           (GtkSignalFunc)cfp_ck_ShowSize,fontpicker2);
+        
+        gtk_signal_connect(GTK_OBJECT(adj),"value_changed",
+                           (GtkSignalFunc)cfp_sp_value_changed,fontpicker2);
+        
+        lbFontInfo=gtk_label_new(_("If you choose a font it will appear here"));
+        gtk_box_pack_start(GTK_BOX(vbox2),lbFontInfo,TRUE,TRUE,5);
+
+        gtk_signal_connect(GTK_OBJECT(fontpicker2),"font_set",
+                           GTK_SIGNAL_FUNC(cfp_set_font),lbFontInfo);
+        
+        
+        /* User Widget */
+        frUser=gtk_frame_new("User Widget");
+        gtk_box_pack_start(GTK_BOX(vbox),frUser,TRUE,TRUE,0);
+        vbox3=gtk_vbox_new(FALSE,FALSE);
+        gtk_container_add(GTK_CONTAINER(frUser),vbox3);
+        /* GnomeFontPicker with User Widget */
+        fontpicker3 = gnome_font_picker_new();
+        gnome_font_picker_set_mode(GNOME_FONT_PICKER(fontpicker3),GNOME_FONT_PICKER_MODE_USER_WIDGET);
+
+        hbox3=gtk_hbox_new(FALSE,0);
+        gtk_box_pack_start(GTK_BOX(hbox3),gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_SPELLCHECK),
+                           FALSE,FALSE,5);
+        gtk_box_pack_start(GTK_BOX(hbox3),gtk_label_new(_("This is an hbox with pixmap and text")),
+            FALSE,FALSE,5);
+        gnome_font_picker_uw_set_widget(GNOME_FONT_PICKER(fontpicker3),hbox3);
+        gtk_container_border_width(GTK_CONTAINER(fontpicker3),5);
+        gtk_box_pack_start(GTK_BOX(vbox3),fontpicker3,TRUE,TRUE,0);
+
+        lbUser=gtk_label_new(_("If you choose a font it will appear here"));
+        gtk_box_pack_start(GTK_BOX(vbox3),lbUser,TRUE,TRUE,5);
+
+        gtk_signal_connect(GTK_OBJECT(fontpicker3),"font_set",
+                           GTK_SIGNAL_FUNC(cfp_set_font),lbUser);
+        
+        gtk_widget_show_all(app);
+    
 }
 
 static void
 create_icon_list(void)
 {
 	GtkWidget *app;
+	GtkWidget *sw;
 	GtkWidget *iconlist;
 	GdkImlibImage *pix;
+	int i;
+	
 	app = create_newwin(TRUE,"testGNOME","Icon List");
 
-	iconlist = gnome_icon_list_new();
+	sw = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gnome_app_set_contents (GNOME_APP (app), sw);
+	gtk_widget_set_usize (sw, 200, 200);
+	gtk_widget_show (sw);
 
+	iconlist = gnome_icon_list_new (80, NULL, TRUE);
+	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), iconlist);
+	
+	GTK_WIDGET_SET_FLAGS(iconlist, GTK_CAN_FOCUS);
 	pix = gdk_imlib_create_image_from_xpm_data((gchar **)bomb_xpm);
 	gdk_imlib_render (pix, pix->rgb_width, pix->rgb_height);
 
-	gnome_icon_list_append_imlib(GNOME_ICON_LIST(iconlist), pix, "Foo");
-	gnome_icon_list_append_imlib(GNOME_ICON_LIST(iconlist), pix, "Bar");
-	gnome_icon_list_append_imlib(GNOME_ICON_LIST(iconlist), pix, "LaLa");
-
-	gnome_app_set_contents(GNOME_APP(app),iconlist);
-	gtk_widget_show(iconlist);
-	gtk_widget_show(app);
-}
-
-
-static void
-create_lamp_update(GtkAdjustment *adj, GnomeLamp *lamp)
-{
-	GdkColor c;
+	gtk_widget_grab_focus (iconlist);
 	
-	c.red = (adj->value / 100) * 0xf000;
-	c.green = (1.0 - (adj->value / 100)) * 0xf000;
-	c.blue = (c.red < 0x8000) ? c.red + 0x8000 : c.green + 0x8000;
-	gnome_lamp_set_color(lamp, &c);
-}
+	gnome_icon_list_freeze (GNOME_ICON_LIST (iconlist));
 
-static void
-create_lamp(void)
-{
-	GtkWidget *w, *lamp, *vbox, *hbox, *scale, *app;
-	GtkAdjustment *adj;
-
-	app = create_newwin(TRUE, "testGNOME", "Lamp/Beacon");
-	vbox = gtk_vbox_new(0, FALSE);
-	hbox = gtk_hbox_new(0, FALSE);
-	gtk_widget_show(hbox);
-	gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
-	lamp = gnome_lamp_new();
-	gtk_box_pack_start_defaults(GTK_BOX(hbox), lamp);
-	gtk_widget_show(lamp);
-	w = gnome_lamp_new_with_type(GNOME_LAMP_INPUT);
-	gtk_widget_show(w);
-	gtk_box_pack_start_defaults(GTK_BOX(hbox), w);
-	w = gnome_lamp_new();
-	gnome_lamp_set_sequence(GNOME_LAMP(w), "RRRYYY");
-	gtk_widget_show(w);
-	gtk_box_pack_start_defaults(GTK_BOX(hbox), w);
-	adj = (GtkAdjustment *)gtk_adjustment_new(0.0, 0.0, 110.0,
-						  1.0, 10.0, 10.0);
-	create_lamp_update(adj, GNOME_LAMP(lamp));
-	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
-			   (GtkSignalFunc)create_lamp_update, lamp);
-	scale = gtk_hscale_new(adj);
-	gtk_scale_set_draw_value(GTK_SCALE(scale), FALSE);
-	gtk_box_pack_start_defaults(GTK_BOX(vbox), scale);
-	gtk_widget_show(scale);
-	gtk_widget_show(vbox);
-	gnome_app_set_contents(GNOME_APP(app), vbox);
-	gnome_lamp_set_window_type(GTK_WINDOW(app), GNOME_LAMP_INPUT);
+	for (i = 0; i < 30; i++) {
+		gnome_icon_list_append_imlib(GNOME_ICON_LIST(iconlist), pix, "Foo");
+		gnome_icon_list_append_imlib(GNOME_ICON_LIST(iconlist), pix, "Bar");
+		gnome_icon_list_append_imlib(GNOME_ICON_LIST(iconlist), pix, "LaLa");
+	}
+	
+	gnome_icon_list_set_selection_mode (GNOME_ICON_LIST (iconlist), GTK_SELECTION_MULTIPLE);
+	gnome_icon_list_thaw (GNOME_ICON_LIST (iconlist));
+	gtk_widget_show (iconlist);
 	gtk_widget_show(app);
 }
 
+
 static void
 create_less(void)
 {
@@ -604,7 +872,6 @@ make_button_hbox(GtkBox * box, gchar * buttontext,
 		 GtkSignalFunc callback, gpointer data)
 {
   GtkWidget * hbox;
-  GtkWidget * entry;
   GtkWidget * button;
 
   hbox = gtk_hbox_new(TRUE, GNOME_PAD);
@@ -926,7 +1193,7 @@ create_app_util(void)
   gtk_container_set_focus_vadjustment (GTK_CONTAINER (vbox),
 				       gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW(sw)));
 
-  gtk_container_add(GTK_CONTAINER(sw), GTK_WIDGET(vbox));
+  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), GTK_WIDGET(vbox));
 
   gnome_app_set_contents(app, sw);
 
@@ -1200,6 +1467,16 @@ insert_blue (GtkWidget *widget, gpointer data)
 
 /* Shared popup menu definition for the GnomeAppHelper test */
 
+static GnomeUIInfo helper_shared_popup_dup[] = {
+	{ GNOME_APP_UI_ITEM, "Insert a _red book", NULL, insert_red, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_RED, 0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, "Insert a _green book", NULL, insert_green, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_GREEN, 0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, "Insert a _blue book", NULL, insert_blue, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_BLUE, 0, 0, NULL },
+	GNOMEUIINFO_END
+};
+
 static GnomeUIInfo helper_shared_popup[] = {
 	{ GNOME_APP_UI_ITEM, "Insert a _red book", NULL, insert_red, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_RED, 0, 0, NULL },
@@ -1207,6 +1484,7 @@ static GnomeUIInfo helper_shared_popup[] = {
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_GREEN, 0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, "Insert a _blue book", NULL, insert_blue, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_BLUE, 0, 0, NULL },
+        GNOMEUIINFO_SUBTREE("Subtree", helper_shared_popup_dup),
 	GNOMEUIINFO_END
 };
 
@@ -1250,6 +1528,17 @@ set_yellow (GtkWidget *widget, gpointer data)
 
 /* Explicit popup menu definition for the GnomeAppHelper test */
 
+
+static GnomeUIInfo helper_explicit_popup_dup[] = {
+	{ GNOME_APP_UI_ITEM, "Set color to _cyan", NULL, set_cyan, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, "Set color to _magenta", NULL, set_magenta, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, "Set color to _yellow", NULL, set_yellow, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
+	GNOMEUIINFO_END
+};
+
 static GnomeUIInfo helper_explicit_popup[] = {
 	{ GNOME_APP_UI_ITEM, "Set color to _cyan", NULL, set_cyan, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
@@ -1257,6 +1546,7 @@ static GnomeUIInfo helper_explicit_popup[] = {
 	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, "Set color to _yellow", NULL, set_yellow, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
+        GNOMEUIINFO_SUBTREE("Subtree", helper_explicit_popup_dup),
 	GNOMEUIINFO_END
 };
 
@@ -1283,12 +1573,18 @@ create_app_helper (GtkWidget *widget, gpointer data)
 	GtkWidget *vbox2;
 	GtkWidget *w;
 	GtkWidget *popup;
+        GnomeAppBar *bar;
 	GnomeCanvasItem *item;
 
 	app = gnome_app_new ("testGNOME", "GnomeAppHelper test");
 	gnome_app_create_menus (GNOME_APP (app), helper_main_menu);
 	gnome_app_create_toolbar (GNOME_APP (app), helper_toolbar);
 
+        bar = GNOME_APPBAR(gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_USER));
+        gnome_app_set_statusbar(GNOME_APP(app), GTK_WIDGET(bar));  
+
+        gnome_app_install_appbar_menu_hints(GNOME_APPBAR(bar), helper_main_menu);
+
 	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
 	gtk_container_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
 
@@ -1324,7 +1620,7 @@ create_app_helper (GtkWidget *widget, gpointer data)
 	gtk_widget_show (frame);
 
 	w = gnome_canvas_new ();
-	gnome_canvas_set_size (GNOME_CANVAS (w), 200, 100);
+	gtk_widget_set_usize ((w), 200, 100);
 	gnome_canvas_set_scroll_region (GNOME_CANVAS (w), 0.0, 0.0, 200.0, 100.0);
 	gtk_container_add (GTK_CONTAINER (frame), w);
 	gtk_widget_show (w);
@@ -1363,6 +1659,77 @@ create_app_helper (GtkWidget *widget, gpointer data)
 	gtk_widget_show (app);
 }
 
+/*test for dentry edit*/
+static void
+create_dentry_edit(void)
+{
+	GtkWidget *app;
+	GtkWidget *edit;
+	GtkWidget *nbook;
+	
+	nbook = gtk_notebook_new();
+
+	app = create_newwin(TRUE,"testGNOME","DEntry Entry");
+	edit = GTK_WIDGET (gnome_dentry_edit_new_notebook(GTK_NOTEBOOK(nbook)));
+	gnome_app_set_contents(GNOME_APP(app),nbook);
+	gtk_widget_show_all(app);
+}
+
+static void
+href_cb(GtkObject *button)
+{
+	GtkWidget *href = gtk_object_get_data(button, "href");
+	GtkWidget *url_ent = gtk_object_get_data(button, "url");
+	GtkWidget *label_ent = gtk_object_get_data(button, "label");
+	gchar *url, *label;
+
+	url = gtk_entry_get_text(GTK_ENTRY(url_ent));
+	label = gtk_entry_get_text(GTK_ENTRY(label_ent));
+	if (!label || ! label[0])
+		label = url;
+	gnome_href_set_url(GNOME_HREF(href), url);
+	gnome_href_set_label(GNOME_HREF(href), label);
+}
+
+static void
+create_href(void)
+{
+	GtkWidget *app, *vbox, *href, *ent1, *ent2, *wid;
+
+	app = create_newwin(TRUE,"testGNOME","HRef test");
+	vbox = gtk_vbox_new(FALSE, 5);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
+	gnome_app_set_contents(GNOME_APP(app), vbox);
+
+	href = gnome_href_new("http://www.gnome.org/", "Gnome Website");
+	gtk_box_pack_start(GTK_BOX(vbox), href, FALSE, FALSE, 0);
+
+	wid = gtk_hseparator_new();
+	gtk_box_pack_start(GTK_BOX(vbox), wid, TRUE, FALSE, 0);
+
+	wid = gtk_label_new("The launch behaviour of the\n"
+			    "configured with the control center");
+	gtk_box_pack_start(GTK_BOX(vbox), wid, TRUE, FALSE, 0);
+
+	ent1 = gtk_entry_new();
+	gtk_entry_set_text(GTK_ENTRY(ent1), "http://www.gnome.org/");
+	gtk_box_pack_start(GTK_BOX(vbox), ent1, TRUE, TRUE, 0);
+	
+	ent2 = gtk_entry_new();
+	gtk_entry_set_text(GTK_ENTRY(ent2), "Gnome Website");
+	gtk_box_pack_start(GTK_BOX(vbox), ent2, TRUE, TRUE, 0);
+
+	wid = gtk_button_new_with_label("set href props");
+	gtk_object_set_data(GTK_OBJECT(wid), "href", href);
+	gtk_object_set_data(GTK_OBJECT(wid), "url", ent1);
+	gtk_object_set_data(GTK_OBJECT(wid), "label", ent2);
+	gtk_signal_connect(GTK_OBJECT(wid), "clicked",
+			   GTK_SIGNAL_FUNC(href_cb), NULL);
+	gtk_box_pack_start(GTK_BOX(vbox), wid, TRUE, TRUE, 0);
+
+	gtk_widget_show_all(app);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1373,21 +1740,24 @@ main (int argc, char *argv[])
 	  {
 		  { "app-util/appbar/dialog-util", create_app_util },
 		  { "app-helper", create_app_helper },
-		  { "calendar", create_calendar },
 		  { "calculator", create_calc },
 		  { "canvas", create_canvas },
 		  { "clock",	create_clock },
 		  { "color picker", create_color_picker },
-		  { "color-sel", create_colorsel },
+		  { "guru", create_guru },
+		  { "paper-sel", create_papersel },
 		  { "date edit", create_date_edit },
 		  { "dialog", create_dialog },
 		  { "file entry", create_file_entry },
-		  { "number entry", create_number_entry },
-		  { "font sel", create_font_sel },
+                  { "pixmap entry", create_pixmap_entry },
+                  { "icon entry", create_icon_entry },
+                  { "number entry", create_number_entry },
+                  { "font picker", create_font_picker },                  
 		  { "icon list", create_icon_list }, 
-		  { "lamp", create_lamp },
 		  { "less", create_less },
 		  { "pixmap", create_pixmap },
+		  { "dentry edit", create_dentry_edit },
+		  { "href", create_href },
 		  { "(Reload preferences)", gnome_preferences_load },
 /*	{ "prop box", create_property_box }, */
 	  };
@@ -1398,8 +1768,8 @@ main (int argc, char *argv[])
 	GtkWidget *button;
 	GtkWidget *scrolled_window;
 	int i;
-	
-	gnome_init ("testGNOME", NULL, argc, argv, 0, NULL);
+
+	gnome_init ("testGNOME", VERSION, argc, argv);
 
 	app = create_newwin(FALSE,"testGNOME", "testGNOME");
 	gtk_widget_set_usize (app, 200,300);
@@ -1417,7 +1787,7 @@ main (int argc, char *argv[])
 	gtk_widget_show (scrolled_window);
 	box2 = gtk_vbox_new (FALSE, 0);
 	gtk_container_border_width (GTK_CONTAINER (box2), 10);
-	gtk_container_add (GTK_CONTAINER (scrolled_window), box2);
+	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window), box2);
 	gtk_container_set_focus_vadjustment (GTK_CONTAINER (box2),gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW(scrolled_window)));
 	gtk_widget_show (box2);
 	for (i = 0; i < nbuttons; i++)
