@@ -3,18 +3,18 @@
  *
  *  A virtual terminal emulator.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  GNU Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
@@ -25,13 +25,18 @@
   It also handles selections, and making them visible, etc.
 */
 
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
 
 #include "lists.h"
 #include "memory.h"
@@ -40,74 +45,6 @@
 
 #define d(x)
 
-/*
-  invert the reverse attribute if the text is selected
-  and then draw as usual.
-
-  this is unecessarily complicated by the fact selstarty not guaranteed
-  to be < selendy (and same for selstartx/selendx)
-*/
-void vt_draw_text_select(struct _vtx *vx, int col, int row, char *text, int len, int attr)
-{
-  int newattr;
-  int startlen,midlen,endlen;	/* start/mid/end sections */
-  int sx, ex;
-
-  if (vx->selected &&
-      (((row >= (vx->selstarty - vx->vt.scrollbackoffset)) && /* normal order select */
-      (row <= (vx->selendy - vx->vt.scrollbackoffset))) ||
-      ((row <= (vx->selstarty - vx->vt.scrollbackoffset)) && /* start<end */
-      (row >= (vx->selendy - vx->vt.scrollbackoffset)))) ) {
-
-    d(printf("drawing text from %d for %d\n", col, len));
-
-    newattr = attr ^ VTATTR_REVERSE;
-    sx = 0;
-    ex = col+len;
-
-    if (vx->selstarty<=vx->selendy) {
-      if (row == (vx->selstarty-vx->vt.scrollbackoffset))
-	sx = vx->selstartx;
-      if (row == (vx->selendy-vx->vt.scrollbackoffset))
-	ex = vx->selendx;
-    } else {
-      if (row == (vx->selendy-vx->vt.scrollbackoffset))
-	sx = vx->selendx;
-      if (row == (vx->selstarty-vx->vt.scrollbackoffset)) {
-	ex = vx->selstartx;
-      }
-    }
-
-    /* check startx<endx, if on the same line */
-    if ( (sx>ex) &&
-	 (row == vx->selstarty-vx->vt.scrollbackoffset) &&
-	 (row == vx->selendy-vx->vt.scrollbackoffset) ) {
-      startlen=sx; sx=ex; ex=startlen; /* use startlen as a temp */
-    }
-
-    /* now calculate which bits of the line are selected/otherwise */
-    startlen = (sx-col)<0?0:(sx-col);
-    endlen = (col+len-ex)<0?0:(col+len-ex);
-    if (startlen>len)
-      startlen = len;
-    if (endlen>len)
-      endlen = len;
-    midlen = (len-startlen-endlen);
-
-    d(printf("startx = %d, endx = %d\n", sx, ex));
-    d(printf("drawing text <%d> normal <%d> selected <%d> normal\n", startlen, midlen, endlen));
-
-    if (startlen)
-      vt_draw_text(vx->user_data, col, row, text, startlen, attr);
-    if (midlen)
-      vt_draw_text(vx->user_data, col+startlen, row, text+startlen, midlen, newattr);
-    if (endlen)
-      vt_draw_text(vx->user_data, col+startlen+midlen, row, text+startlen+midlen, endlen, attr);
-  } else {
-    vt_draw_text(vx->user_data, col, row, text, len, attr);
-  }
-}
-
 /*
   update line 'line' (node 'l') of the vt
 
@@ -121,10 +58,11 @@ static void vt_line_update(struct _vtx *vx, struct vt_line *l, int line, int alw
   int run;
   int runstart;
   char *runbuffer, *p;
-  int attr;
+  uint32 attr, newattr;
   uint32 c;
+  int ch;
   struct vt_line *bl;
-  int noclear;			/* clear before writing text? */
+  int sx, ex;			/* start/end selection */
 
   d(printf("updating line %d: ", line));
   d(fwrite(l->data, l->width, 1, stdout));
@@ -132,6 +70,31 @@ static void vt_line_update(struct _vtx *vx, struct vt_line *l, int line, int alw
 
   d(printf("updating line from (%d-%d) ->", start, end));
 
+  /* for 'scrollback' where the window has changed size -
+     create a proper-length line (this is easier than special case code below */
+  bl = (struct vt_line *)vt_list_index(&vx->vt.lines_back, line);
+
+  /* some sanity checks */
+  g_return_if_fail (bl != NULL);
+  g_return_if_fail (bl->next != NULL);
+
+  if (bl->width > l->width) {
+    struct vt_line *newline;
+
+    d(printf("line size mismatch %d != %d\n", bl->width, l->width));
+
+    /* create a new temp line, and set it to a clear
+       last-character (attributes) of the new line */
+    newline = alloca(sizeof(*l)+bl->width*sizeof(bl->data[0]));
+    memcpy(newline, l, sizeof(*l)+l->width*sizeof(l->data[0]));
+    c = l->data[l->width-1]&VTATTR_MASK;
+    for (i=l->width;i<bl->width;i++) {
+      newline->data[i] = c;
+    }
+    /* over-write the 'l' pointer */
+    l = newline;
+  }
+
   /* check range conditions */
   if (end>l->width) {
     end = l->width;
@@ -142,44 +105,85 @@ static void vt_line_update(struct _vtx *vx, struct vt_line *l, int line, int alw
 
   runbuffer = alloca(vx->vt.width * sizeof(char));
 
-  bl = (struct vt_line *)vt_list_index(&vx->vt.lines_back, line);
+  /* work out if selections are being rendered */
+  if (vx->selected &&
+      (((line >= (vx->selstarty - vx->vt.scrollbackoffset)) && /* normal order select */
+      (line <= (vx->selendy - vx->vt.scrollbackoffset))) ||
+      ((line <= (vx->selstarty - vx->vt.scrollbackoffset)) && /* start<end */
+      (line >= (vx->selendy - vx->vt.scrollbackoffset)))) ) {
+    
+    /* work out range of selections */
+    sx = 0;
+    ex = l->width;
+    
+    if (vx->selstarty<=vx->selendy) {
+      if (line == (vx->selstarty-vx->vt.scrollbackoffset))
+	sx = vx->selstartx;
+      if (line == (vx->selendy-vx->vt.scrollbackoffset))
+	ex = vx->selendx;
+    } else {
+      if (line == (vx->selendy-vx->vt.scrollbackoffset))
+	sx = vx->selendx;
+      if (line == (vx->selstarty-vx->vt.scrollbackoffset)) {
+	ex = vx->selstartx;
+      }
+    }
 
-  /* some sanity checks */
-  g_return_if_fail (bl != NULL);
-  g_return_if_fail (runbuffer != NULL);
-  g_return_if_fail (bl->width == l->width);
+    /* check startx<endx, if on the same line, swap if so */
+    if ( (sx>ex) &&
+	 (line == vx->selstarty-vx->vt.scrollbackoffset) &&
+	 (line == vx->selendy-vx->vt.scrollbackoffset) ) {
+      int tmp;
+      tmp=sx; sx=ex; ex=tmp;
+    }
+  } else {
+    sx = -1;
+    ex = -1;
+  }
+  /* ^^ at this point sx -- ex needs to be inverted (it is selected) */
 
   /* work out the minimum range of change */
   if (!always) {
     
     while (start<end) {
-      if ((l->data[start]&0x7fffffff) != (bl->data[start]&0x7fffffff))
+      c = l->data[start];
+      if (start>=sx && start<ex)
+	c ^= VTATTR_REVERSE;
+      if (c != bl->data[start])
 	break;
       start++;
     }
     
     while (end>start) {
-      if ((l->data[end-1]&0x7fffffff) != (bl->data[end-1]&0x7fffffff))
+      c = l->data[end-1];
+      if (end>=sx && end<ex)
+	c ^= VTATTR_REVERSE;
+      if (c != bl->data[end-1])
 	break;
       end--;
     }
   }  /* else, update everything */
 
   /* see if on-screen is all background ... */
+  /* (this is messy, and probably slow?) */
   vx->back_match=1;
   for(i=start;i<end;i++) {
-    if ((bl->data[i] & ~VTATTR_CHANGED) != (vx->vt.attr&~VTATTR_CHANGED)) {
+    /* check for clear of the same background colour */
+    c = bl->data[i];
+    ch = c & VTATTR_DATAMASK;
+    if (i>=sx && i<ex)
+      c ^= VTATTR_REVERSE;
+    if ((ch!=0 && ch!=9 && ch!=' ')
+	|| ((c & (VTATTR_MASK ^ VTATTR_FORECOLOURM))
+	    != (l->data[i] & (VTATTR_MASK ^ VTATTR_FORECOLOURM)))) {
+      d(printf("bl->data[i] = %08x != %08x, c=%d (back now=%d not %d)\n", bl->data[i], l->data[i], c, (l->data[i]&VTATTR_BACKCOLOURM)>>VTATTR_BACKCOLOURB, (bl->data[i]&VTATTR_BACKCOLOURM)>>VTATTR_BACKCOLOURB));
       vx->back_match=0;
       break;
+    } else {
+      d(printf("bl->data[i] == %08x != %08x, c=%08x (back now=%d not %d)\n", bl->data[i], l->data[i], c, (l->data[i]&VTATTR_BACKCOLOURM)>>VTATTR_BACKCOLOURB, (bl->data[i]&VTATTR_BACKCOLOURM)>>VTATTR_BACKCOLOURB));
     }
   }
 
-  /* copy changed section of line to destination line */
-  for(i=start;i<end;i++) {
-    bl->data[i]=l->data[i];
-  }
-  bl->line = line;
-
   d(printf("actually (%d-%d)?\n", start, end));
 
   run=0;
@@ -187,51 +191,46 @@ static void vt_line_update(struct _vtx *vx, struct vt_line *l, int line, int alw
   runstart = 0;
   p = 0;
   for(i=start;i<end;i++) {
-#if 0
-    if (always || ((l->data[i] & VTATTR_CHANGED)) ) {
-#endif
-      if (run==0) {
-	runstart = i;
-	p = runbuffer;
-	attr = l->data[i] & 0x7fff0000;
-      } else {
-	if (attr != (l->data[i] & 0x7fff0000)) { /* check run of same type ... */
-	  vt_draw_text_select(vx, runstart, line, runbuffer, run, attr);
-	  runstart = i;
-	  p = runbuffer;
-	  run=0;
-	  attr = l->data[i] & 0x7fff0000;
-	}
-      }
-      c = l->data[i] & 0xff;
-      /* FIXME: this is needed for alternate charsets
-	 will need to do something else to mark empty-space and tabs? */
-      if ((c==0) || (c==9))
-	c=' ';
-      /*if (c<32)*/			/* all control chars ->space */
-      /*c=' ';*/
-      *p++=c;
-      run++;
-#if 0
+    /* map on 'selected' areas, and copy to screen buffer */
+    newattr = l->data[i] & VTATTR_MASK;
+    if (i>=sx && i<ex) {
+      newattr ^= VTATTR_REVERSE;
+      bl->data[i]=l->data[i]^VTATTR_REVERSE;
     } else {
-      if (run) {
+      bl->data[i]=l->data[i];
+    }
+
+    if (run==0) {
+      runstart = i;
+      p = runbuffer;
+      attr = newattr;
+    } else {
+      if (attr != newattr) { /* check run of same type ... */
 	d(printf("found a run of %d characters from %d: '", run, runstart));
 	d(fwrite(runbuffer, run, 1, stdout));
-	vt_draw_text_select(vx, runstart, line, runbuffer, run, attr);
-	d(printf("'\n"));
+	vt_draw_text(vx->vt.user_data, runstart, line, runbuffer, run, attr);
+	runstart = i;
+	p = runbuffer;
 	run=0;
+	attr = newattr;
       }
     }
-#endif
-    l->data[i] &= ~VTATTR_CHANGED;
+    c = l->data[i] & 0xff;
+    /* FIXME: this is needed for alternate charsets
+       will need to do something else to mark empty-space and tabs? */
+    if ((c==0) || (c==9))
+      c=' ';
+    *p++=c;
+    run++;
   }
   if (run) {
     d(printf("found a run of %d characters from %d: '", run, runstart));
     d(fwrite(runbuffer, run, 1, stdout));
-    vt_draw_text_select(vx, runstart, line, runbuffer, run, attr);
+    vt_draw_text(vx->vt.user_data, runstart, line, runbuffer, run, attr);
     d(printf("'\n"));
   }
   l->modcount = 0;
+  bl->line = line;
 }
 
 int vt_get_attr_at (struct _vtx *vx, int col, int row)
@@ -245,23 +244,19 @@ int vt_get_attr_at (struct _vtx *vx, int col, int row)
 /*
   scroll/update a section of lines
   from firstline (fn), scroll count lines 'offset' lines
-
-  FIXME: check this
 */
-void vt_scroll_update(struct _vtx *vx, int firstline, struct vt_line *fn, int count, int offset)
+static void vt_scroll_update(struct _vtx *vx, int firstline, int count, int offset)
 {
-  struct vt_line *tn, *bn, *dn;	/* top/bottom of scroll area */
-  int i;
+  struct vt_line *tn, *bn, *dn, *nn;	/* top/bottom of scroll area */
+  int i, fill;
 
   d(printf("scrolling %d lines from %d, by %d\n", count, firstline, offset));
 
-  vt_scroll_area(vx->user_data, firstline, count, offset);
-
   d({
     printf("before:\n");
     dn = &vx->vt.lines_back.head;
     while(dn) {
-      printf("node %p:  n: %p  p:%p\n", dn, dn->next, dn->prev);
+      printf("node %p:  n: %p  p:%p -> %d\n", dn, dn->next, dn->prev, dn->line);
       dn = dn->next;
     }
   });
@@ -271,26 +266,23 @@ void vt_scroll_update(struct _vtx *vx, int firstline, struct vt_line *fn, int co
   if (offset>0) {
     /* grab n lines at bottom of scroll area, and move them to above it */
     tn = (struct vt_line *)vt_list_index(&vx->vt.lines_back, firstline);
+    nn = (struct vt_line *)vt_list_index(&vx->vt.lines, firstline);
     bn = (struct vt_line *)vt_list_index(&vx->vt.lines_back, firstline+offset-1);
     dn = (struct vt_line *)vt_list_index(&vx->vt.lines_back, firstline+count+offset);
   } else {
     /* grab n lines at top of scroll area, and move them to below it */
-    /* FIXME: this is wrong */
     tn = (struct vt_line *)vt_list_index(&vx->vt.lines_back, firstline+count+offset);
+    nn = (struct vt_line *)vt_list_index(&vx->vt.lines, firstline+count+offset);
     bn = (struct vt_line *)vt_list_index(&vx->vt.lines_back, firstline+count-1);
     dn = (struct vt_line *)vt_list_index(&vx->vt.lines_back, firstline+offset);
   }
 
-  /*
-    
+  /*    
     0->dn->4->tn->1->2->bn->5
-    
     0->dn->4->5
-    
     0->tn->1->2->bn->dn->4->5
   */
 
-
   /* remove the scroll segment */
   tn->prev->next = bn->next;
   bn->next->prev = tn->prev;
@@ -305,57 +297,24 @@ void vt_scroll_update(struct _vtx *vx, int firstline, struct vt_line *fn, int co
     printf("After\n");
     dn = &vx->vt.lines_back.head;
     while(dn) {
-      printf("node %p:  n: %p  p:%p\n", dn, dn->next, dn->prev);
+      printf("node %p:  n: %p  p:%p -> %d\n", dn, dn->next, dn->prev, dn->line);
       dn = dn->next;
     }
   });
 
-  /* need to clear tn->bn lines */
-  /* FIXME: maybe if we didn't 'clear' this, and we dont 'clear' the screen after
-     a scroll, it will make things faster ... but we'd have to 'copy' stuff here
-     so it matches the result of a scroll */
+  /* find out what colour the new lines is - make it match (use
+     first character as a guess), and perform the visual scroll */
+  fill = (nn->data[0] & VTATTR_BACKCOLOURM) >> VTATTR_BACKCOLOURB;
+  vt_scroll_area(vx->vt.user_data, firstline, count, offset, fill);
+
+  /* need to clear tn->bn lines.  Use attributes from first character of corresponding new line */
+  fill = nn->data[0] & VTATTR_MASK;
   do {
     d(printf("clearning line %d\n", tn->line));
     for(i=0;i<tn->width;i++) {
-      tn->data[i]=vx->vt.attr; /* not sure if this is the correct thing to 'clear' with */
+      tn->data[i]=fill;
     }
   } while ((tn!=bn) && (tn=tn->next));
-
-    /*
-1
-2
-3
-4
-5
-6
-7
-8
-9
-
-scroll(3, 4, 1);
-
-1
-2
--7
-3
-4
-5
-6
-8
-9
-
-scroll(3, 4, -1);
- 
-
-1
-3
-4
-5
-6
--2
-7
-8
-    */
 }
 
 /*
@@ -375,220 +334,213 @@ void vt_update(struct _vtx *vx, int update_state)
   int oldoffset=0;
   struct vt_line *wn, *nn, *fn;
   int firstline;
-  int count;
   int old_state;
+  int update_start=-1;	/* where to start/stop update */
+  int update_end=-1;
 
   d(printf("updating screen\n"));
 
-  old_state = vt_cursor_state(vx->user_data, 0);
+  old_state = vt_cursor_state(vx->vt.user_data, 0);
 
-  if (vx->vt.scrollbackoffset != 0) {
-    if (vx->vt.scrollbackoffset != vx->vt.scrollbackold ||
-	update_state&UPDATE_SCROLLBACK) {
-
-      /* still experimental */
-#ifdef OPT_SCROLLBACK
-      /* FIXME: this could be optimised, based on vt->scrollbackold! */
-      if (!(update_state&UPDATE_SCROLLBACK)) {
-	/*
-	  offset>0 -> scroll down
-	    scroll from line offset, (total-offset) lines, by offset lines
-	  offset<0 -> scroll up
-	    scroll from line total-offset
-	*/
-	offset = vt->scrollbackold-vt->scrollbackoffset;
-	if (offset>0) {		/* scrolled .. down */
-	  if (offset < vt->height-8) { /* otherwise just do all of it */
-	    (printf("scrolling down\n"));	/* need to add on top */
-	    vt_scroll_area(0, vt->height, -offset);
-	  }
-	} else {		/* scrolled .. up */
-	  if ((-offset) < vt->height-8) {
-	    printf("scrolling up\n"); /* need to add on bottom */
-	    vt_scroll_area(0, vt->height, -offset);
-	    if ( (-vt->scrollbackoffset) < vt->height) {
-	      printf("*** scrolls into normal screen (%d %d)\n", vt->scrollbackoffset, vt->height+vt->scrollbackoffset);
-	      count = vt->scrollbackoffset+vt->height;
-	      firstline=0;
-	      /*line=vt->height+offset;*/ /* FIXME: must also specify the number of lines to skip ... */
-	      line=-vt->scrollbackoffset;
-	    } else {
-	      count = -offset;	/* how many lines to add */
-	      firstline= (- vt->scrollbackoffset); /* how many lines to go back ... */
-	      line=0;
-	    }
-
-	    printf("updating: count=%d, firstline=%d, line=%d\n", count, firstline, line);
-
-	    goto skip_countset;
-	  }
-	}
-      }
-
-#endif
-      /* we are in scroll back mode ... */
-      /* must first print any scroll-back buffer lines that need to be displayed */
-      /* find the scrollback lines visible, and display them .. */
-      count=vx->vt.height;		/* lines left to draw */
-      firstline= (- vx->vt.scrollbackoffset); /* how many lines to go back ... */
-      line=0;
-#ifdef OPT_SCROLLBACK
-	skip_countset:
-#endif
-
-      wn = (struct vt_line *)vx->vt.scrollback.tailpred;
-      nn = wn->prev;
-      while (nn && count>0 && firstline>0) {
-	if (firstline <= vx->vt.height) { /* are we on the screen yet? */
-	  vt_line_update(vx, wn, firstline-1, 1, 0, wn->width);
-	  count--;
-	  line++;
-	}
-	firstline--;
-	wn=nn;
-	nn=nn->prev;
-      }
-      /* now, find working buffer lines visible, and display them */
-      wn = (struct vt_line *)vx->vt.lines.head;
-      nn = wn->next;
-      while (nn && count>0) {
-	vt_line_update(vx, wn, line, 1, 0, wn->width);
-	wn=nn;
-	nn=nn->next;
-	line++;
-	count--;
-      }
-      vx->vt.scrollbackold = vx->vt.scrollbackoffset;
+  /* find first line of visible screen, take into account scrollback */
+  offset = vx->vt.scrollbackoffset;
+  if (offset<0) {
+    wn = (struct vt_line *)vt_list_index(&vx->vt.scrollback, offset);
+    if (wn==0) {
+      /* check for error condition */
+      printf("LINE UNDERFLOW!\n");
+      wn = (struct vt_line *)vx->vt.scrollback.head;
     }
   } else {
-    if (update_state==0) {
-				/* perform simple whole-screen update? */
-				/* perform optimised update */
-      wn = (struct vt_line *)vx->vt.lines.head;
-      nn = wn->next;
-      firstline = 0;		/* this isn't really necessary */
-      fn = wn;
-      while (nn) {
-	d(printf("scanning line %d, offset=%d : ", line, (wn->line-line)));
-	d(printf("(%d - %d)\n", line, wn->line));
-	if ((wn->line!=-1) && ((offset = wn->line-line) >0)) {
-	  wn->line = line;
-#ifdef CHANGE_CHECK
-	  if (wn->modcount > (wn->width>>4)) { /* simple update metric, not much changed? */
-	    if (oldoffset != 0) {
-	      d(printf("scrolling (1.0)\n"));
-	      vt_scroll_update(vx, firstline, fn, line-firstline, oldoffset); /* scroll/update a line */
-	    }
-	    offset=0;
-	  } else {
-#endif
-	    if (offset == oldoffset) {
-	      /* accumulate "update" lines */
-	    } else {
-	      if (oldoffset != 0) {
-		d(printf("scrolling (1.1)\n"));
-		vt_scroll_update(vx, firstline, fn, line-firstline, oldoffset); /* scroll/update a line */
-	      }
-	      d(printf("found a scrolled line\n"));
-	      firstline = line;	/* first line to scroll */
-	      fn = wn;
-	    }
-#ifdef CHANGE_CHECK
-	  }
-#endif
+    wn=(struct vt_line *)vx->vt.lines.head;
+  }
+  
+  d({
+    struct vt_line *dn;
+    printf("INPUT before:\n");
+    dn = &vx->vt.lines;
+    while(dn) {
+      printf("  node %p:  n: %p  p:%p -> %d\n", dn, dn->next, dn->prev, dn->line);
+      dn = dn->next;
+    }
+
+    dn = wn;
+    while(dn) {
+      printf("  node %p:  n: %p  p:%p -> %d\n", dn, dn->next, dn->prev, dn->line);
+      dn = dn->next;
+    }
+
+  });
+
+  /* updated scrollback if we can ... otherwise dont */
+  if (!(update_state&UPDATE_REFRESH)) {
+
+    /* if scrollback happening, then force update of newly exposed areas - calculate here */
+    offset = vx->vt.scrollbackoffset-vx->vt.scrollbackold;
+    if (offset>0) {
+      d(printf("scrolling up, by %d\n", offset));
+      update_start = vx->vt.height-offset-1;
+      update_end = vx->vt.height;
+    } else {
+      d(printf("scrolling down, by %d\n", -offset));
+      update_start = -1;
+      update_end = -offset;
+    }
+    d(printf("forced updated from %d - %d\n", update_start, update_end));
+    
+    nn = wn->next;
+    firstline = 0;		/* this isn't really necessary (quietens compiler) */
+    fn = wn;
+    while (nn && line<vx->vt.height) {
+
+      /* enforce full update for 'scrollback' areas */
+      if (line>update_start && line<update_end) {
+	d(printf("forcing scroll update refresh (line %d)\n", line));
+	wn->line = -1;
+      }
+
+      d(printf("%p: scanning line %d, offset=%d : line = %d\n ", wn, line, (wn->line-line), wn->line));
+      d(printf("(%d - %d)\n", line, wn->line));
+      if ((wn->line!=-1) && ((offset = wn->line-line) >0)) {
+	wn->line = line;
+	if (offset == oldoffset) {
+	  /* accumulate "update" lines */
 	} else {
 	  if (oldoffset != 0) {
-	    d(printf("scrolling (1.2)\n"));
-	    vt_scroll_update(vx, firstline, fn, line-firstline, oldoffset); /* scroll/update a line */
+	    d(printf("scrolling (1.1)\n"));
+	    vt_scroll_update(vx, firstline, line-firstline, oldoffset); /* scroll/update a line */
 	  }
-	  offset=0;
+	  d(printf("found a scrolled line\n"));
+	  firstline = line;	/* first line to scroll */
+	  fn = wn;
 	}
+      } else {
+	if (oldoffset != 0) {
+	  d(printf("scrolling (1.2)\n"));
+	  vt_scroll_update(vx, firstline, line-firstline, oldoffset); /* scroll/update a line */
+	}
+	offset=0;
+      }
 
-	oldoffset = offset;
-	line ++;
+      /* goto next logical line */
+      if (wn==(struct vt_line *)vx->vt.scrollback.tailpred) {
+	d(printf("skipping to real lines at line %d\n", line));
+	wn = (struct vt_line *)vx->vt.lines.head;
+      } else {
+	d(printf("going to next line ...\n"));
 	wn = nn;
-	nn = nn->next;
-      }
-      if (oldoffset != 0) {
-	d(printf("scrolling (1.3)\n"));
-	d(printf("oldoffset = %d, must scroll\n", oldoffset));
-	vt_scroll_update(vx, firstline, fn, line-firstline, oldoffset); /* scroll/update a line */
       }
 
-      /* now scan backwards ! */
+      oldoffset = offset;
+      line ++;
+      nn = wn->next;
+    }
+    if (oldoffset != 0) {
+      d(printf("scrolling (1.3)\n"));
+      d(printf("oldoffset = %d, must scroll\n", oldoffset));
+      vt_scroll_update(vx, firstline, line-firstline, oldoffset); /* scroll/update a line */
+    }
+
+    /* now scan backwards ! */
+    d(printf("scanning backwards now\n"));
+    
+      /* does this need checking for overflow? */
+    if (wn == (struct vt_line *)vx->vt.lines.head) {
+      wn = (struct vt_line *)vx->vt.scrollback.tailpred;
+      nn = wn->prev;
+    } else {
       wn = wn->prev;
       nn = wn->prev;
-      line = vx->vt.height;
-      oldoffset = 0;
-      while (nn) {
-	line--;
-	d(printf("scanning line %d, offset=%d : ", line, (wn->line-line)));
-	if ((wn->line !=-1) && ((offset = wn->line-line) < 0)) {
-	  wn->line = line;
-#ifdef CHANGE_CHECK
-	  if (wn->modcount > (wn->width>>4)) { /* simple update metric, not much changed? */
-	    d(printf("changed too much - ignoring\n"));
-	    if (oldoffset != 0) {
-	      d(printf("scrolling (2.0)\n"));
-	      vt_scroll_update(vx, line, wn->next, firstline-line+1, oldoffset); /* scroll/update a line, from this line *down* */
-	    }
-	    offset=0;
-	  } else {
-#endif
-	    if (offset == oldoffset) {
-	      /* accumulate "update" lines */
-	    } else {
-	      if (oldoffset != 0) {
-		d(printf("scrolling (2.1)\n"));
-		vt_scroll_update(vx, line, wn->next, firstline-line+1, oldoffset); /* scroll/update a line */
-	      }
-	      d(printf("found a scrolled line\n"));
-	      firstline = line;	/* first line to scroll */
-	      fn = wn;
-	    }
-#ifdef CHANGE_CHECK
-	  }
-#endif
+    }
+
+    line = vx->vt.height;
+    oldoffset = 0;
+    while (nn && line) {
+      line--;
+      d(printf("%p: scanning line %d, offset=%d : line = %d, oldoffset=%d\n ", wn, line, (wn->line-line), wn->line, oldoffset));
+      if ((wn->line !=-1) && ((offset = wn->line-line) < 0)) {
+	wn->line = line;
+	if (offset == oldoffset) {
+	  /* accumulate "update" lines */
 	} else {
 	  if (oldoffset != 0) {
-	    d(printf("scrolling (2.2)\n"));
-	    vt_scroll_update(vx, line+1, wn->next, firstline-line, oldoffset); /* scroll/update a line */
+	    d(printf("scrolling (2.1)\n"));
+	    vt_scroll_update(vx, line, firstline-line+1, oldoffset); /* scroll/update a line */
 	  }
-	  offset=0;
+	  d(printf("found a scrolled line\n"));
+	  firstline = line;	/* first line to scroll */
+	  fn = wn;
 	}
+      } else {
+	if (oldoffset != 0) {
+	  d(printf("scrolling (2.2)\n"));
+	  vt_scroll_update(vx, line+1, firstline-line, oldoffset); /* scroll/update a line */
+	}
+	offset=0;
+      }
 
-	oldoffset = offset;
+      /* goto previous logical line */
+      if (wn==(struct vt_line *)vx->vt.lines.head) {
+	wn = (struct vt_line *)vx->vt.scrollback.tailpred;
+	d(printf("going to scrollback buffer line ...\n"));
+      } else {
+	d(printf("going to prev line ...\n"));
 	wn = nn;
-	nn = nn->prev;
-      }
-      if (oldoffset != 0) {
-	d(printf("scrolling (2.3)\n"));
-	d(printf("oldoffset = %d, must scroll 2\n", oldoffset));
-	vt_scroll_update(vx, 0, wn, firstline-line+1, oldoffset); /* scroll/update a line */
       }
+
+      nn = wn->prev;
+      oldoffset = offset;
+      d(printf("nn = %p\n", nn));
+    }
+    if (oldoffset != 0) {
+      d(printf("scrolling (2.3)\n"));
+      d(printf("oldoffset = %d, must scroll 2\n", oldoffset));
+      vt_scroll_update(vx, 0, firstline-line+1, oldoffset); /* scroll/update a line */
     }
 
-    /* now, re-scan, since all lines should be at the right position now,
-     and update as required */
-
-    wn = (struct vt_line *)vx->vt.lines.head;
-    nn = wn->next;
-    firstline = 0;		/* this isn't really necessary */
-    fn = wn;
-    line=0;
-    while (nn) {
-      if (wn->line==-1) {
-	vt_line_update(vx, wn, line, 0, 0, wn->width);
-      } else if (wn->modcount || update_state) {
-	vt_line_update(vx, wn, line, update_state, 0, wn->width);
-      }
-      wn->line = line;		/* make sure line is reset */
-      line++;
-      wn=nn;
-      nn=nn->next;
+    /* have to align the pointer properly for the last pass */
+    if (wn==(struct vt_line *)vx->vt.scrollback.tailpred) {
+      d(printf("++ skipping to real lines at line %d\n", line));
+      wn = (struct vt_line *)vx->vt.lines.head;
+    } else {
+      d(printf("++ going to next line ...\n"));
+      wn = wn->next;
     }
+
   }
 
+  /* now, re-scan, since all lines should be at the right position now,
+     and update as required */
+  d(printf("scanning from top again\n"));
+
+  nn = wn->next;
+  firstline = 0;		/* this isn't really necessary */
+  fn = wn;
+  line=0;
+  while (nn && line<vx->vt.height) {
+    d(printf("%p: scanning line %d, was %d\n", wn, line, wn->line));
+    if (wn->line==-1) {
+      vt_line_update(vx, wn, line, 0, 0, wn->width);
+    } else if (wn->modcount || update_state&UPDATE_REFRESH) {
+      vt_line_update(vx, wn, line, update_state, 0, wn->width);
+    }
+    wn->line = line;		/* make sure line is reset */
+    line++;
+
+    /* goto next logical line */
+    if (wn==(struct vt_line *)vx->vt.scrollback.tailpred) {
+      d(printf("skipping to real lines at line %d\n", line));
+      wn = (struct vt_line *)vx->vt.lines.head;
+    } else {
+      wn = nn;
+    }
+    
+    nn = wn->next;
+    d(printf("  -- wn = %p, wn->next = %p\n", wn, wn->next));
+  }
+
+  vx->vt.scrollbackold = vx->vt.scrollbackoffset;
+
   /* some debug */
 #if 0
   {
@@ -601,7 +553,7 @@ void vt_update(struct _vtx *vx, int update_state)
     while (nb) {
       printf("%d: ", wb->line);
       for (i=0;i<wb->width;i++) {
-	(printf("%c", (wb->data[i]&0xffff))); /*>=32?(wb->data[i]&0xffff):' '));*/
+	(printf("%c", (wb->data[i]&VTATTR_DATAMASK))); /*>=32?(wb->data[i]&0xffff):' '));*/
       }
       (printf("\n"));
       wb=nb;
@@ -610,7 +562,7 @@ void vt_update(struct _vtx *vx, int update_state)
   }
 #endif
 
-  vt_cursor_state(vx->user_data, old_state);
+  vt_cursor_state(vx->vt.user_data, old_state);
 }
 
 /*
@@ -625,7 +577,7 @@ void vt_update_rect(struct _vtx *vx, int csx, int csy, int cex, int cey)
   struct vt_line *wn, *nn;
   int old_state;
 
-  old_state = vt_cursor_state(vx->user_data, 0);	/* ensure cursor is really off */
+  old_state = vt_cursor_state(vx->vt.user_data, 0);	/* ensure cursor is really off */
 
   d(printf("updating (%d,%d) - (%d,%d)\n", csx, csy, cex, cey));
 
@@ -635,31 +587,49 @@ void vt_update_rect(struct _vtx *vx, int csx, int csy, int cex, int cey)
   if (csx>vx->vt.width)
     csx = vx->vt.width;
 
+  if (cey>=vx->vt.height)
+    cey = vx->vt.height-1;
+  if (csy>=vx->vt.height)
+    csy = vx->vt.height-1;
+
   lines = cey-csy;
-  wn = (struct vt_line *)vt_list_index(&vx->vt.lines, csy);
+
+  /* check scrollback for current line */
+  if ((vx->vt.scrollbackoffset+csy)<0) {
+    wn = (struct vt_line *)vt_list_index(&vx->vt.scrollback, vx->vt.scrollbackoffset+csy);
+  } else {
+    wn = (struct vt_line *)vt_list_index(&vx->vt.lines, vx->vt.scrollbackoffset+csy);
+  }
   if (wn) {
     nn = wn->next;
     while ((csy<=cey) && nn) {
       d(printf("updating line %d\n", csy));
       vt_line_update(vx, wn, csy, 1, csx, cex);
       csy++;
-      wn = nn;
-      nn = nn->next;
+
+      /* skip out of scrollback buffer if need be */
+      if (wn==(struct vt_line *)vx->vt.scrollback.tailpred) {
+	wn = (struct vt_line *)vx->vt.lines.head;
+      } else {
+	wn = nn;
+      }
+
+      nn = wn->next;
     }
   }
 
-  vt_cursor_state(vx->user_data, old_state);
+  vt_cursor_state(vx->vt.user_data, old_state);
 }
 
 /*
   returns true if 'c' is in a 'wordclass' (for selections)
 */
-int vt_in_wordclass(uint32 c)
+static int vt_in_wordclass(uint32 c)
 {
   int ch;
 
   ch = c&0xff;
-  return (isalnum(ch) || ispunct(ch));
+  return (isalnum(ch));
 }
 
 /*
@@ -716,6 +686,10 @@ void vt_fix_selection(struct _vtx *vx)
     d(printf("selecting by word\n"));
     /* scan back over word chars */
     d(printf("startx = %d %p-> \n", sx, s->data));
+
+    if (ex==sx && ex<e->width)
+      ex++;
+
     if ((s->data[sx]&0xff)==0) {
       while ((sx>0) && ((s->data[sx]&0xff) == 0))
 	sx--;
@@ -747,8 +721,17 @@ void vt_fix_selection(struct _vtx *vx)
   case VT_SELTYPE_CHAR:
   default:
     d(printf("selecting by char\n"));
-    while ((sx>0) && ((s->data[sx]&0xff) == 0))
-      sx--;
+
+    if (ex==sx && ex<e->width)
+      ex++;
+
+    if ((s->data[sx]&0xff)==0) {
+      while ((sx>0) && ((s->data[sx]&0xff) == 0))
+	sx--;
+      if (sx &&
+	  (( ((s->data[sx])&0xff)!=0x09))) /* 'compress' tabs */
+	sx++;
+    }
 
     /* special cases for tabs and 'blank' character select */
     if ( !((ex >0) && ((e->data[ex-1]&0xff) != 0)) )
@@ -770,7 +753,7 @@ void vt_fix_selection(struct _vtx *vx)
 
 /* convert columns start to column end of line l, into
    a byte array, and store into out */
-char *vt_expand_line(struct vt_line *l, int start, int end, char *out)
+static char *vt_expand_line(struct vt_line *l, int start, int end, char *out)
 {
   int i;
   char *o;
@@ -822,39 +805,33 @@ char *vt_expand_line(struct vt_line *l, int start, int end, char *out)
 }
 
 /*
-  return currently selected text
+  select block (sx,sy)-(ex,ey) from the buffer.
+
+  Line '0' is the top of the screen, negative lines are part of the
+  scrollback, positive lines are part of the visible screen.
 */
-char *vt_get_selection(struct _vtx *vx, int *len)
+char *vt_select_block(struct _vtx *vx, int sx, int sy, int ex, int ey, int *len)
 {
   struct vt_line *wn, *nn;
   int line;
-  char *out;
-  int sx, sy, ex, ey;
+  char *out, *data;
+  int tmp;
 
-  if ( ((vx->selstarty == vx->selendy) && (vx->selstartx > vx->selendx)) ||
-       (vx->selstarty > vx->selendy) ) {
-    ex = vx->selstartx;		/* swap start/end */
-    ey = vx->selstarty;
-    sx = vx->selendx;
-    sy = vx->selendy;
-  } else {			/* keep start/end same */
-    sx = vx->selstartx;      
-    sy = vx->selstarty;
-    ex = vx->selendx;
-    ey = vx->selendy;
+  if ( ((sy == ey) && (sx > ex)) ||
+       (sy > ey) ) {
+    /* swap start/end values */
+    tmp = sx; sx=ex; ex=tmp;
+    tmp = sy; sy=ey; ey=tmp;
   }
 
-  /* FIXME: assumes buffer hasn't shrunk much horizontally */
-  if (vx->selection_data)
-    free(vx->selection_data);
-
-  if ( (vx->selection_data = malloc((ey-sy+1)*(vx->vt.width+20))) == 0 ) {
-    vx->selection_size = 0;
+  /* makes a rough assumption about the buffer size needed */
+  if ( (data = malloc((ey-sy+1)*(vx->vt.width+20))) == 0 ) {
+    *len = 0;
     printf("ERROR: Cannot malloc selection buffer\n");
     return 0;
   }
 
-  out = vx->selection_data;
+  out = data;
 
   line = sy;
   wn = (struct vt_line *)vt_list_index((line<0)?(&vx->vt.scrollback):(&vx->vt.lines), line);
@@ -862,8 +839,9 @@ char *vt_get_selection(struct _vtx *vx, int *len)
   if (wn)
     nn = wn->next;
   else {
-    vx->selection_size = 0;
-    return 0;
+    *len = 0;
+    strcpy(data, "");
+    return data;
   }
 
   /* FIXME: check 'wn' exists ... */
@@ -895,14 +873,26 @@ char *vt_get_selection(struct _vtx *vx, int *len)
       out = vt_expand_line(wn, 0, ex, out);
   }
 
-  vx->selection_size = out-vx->selection_data;
-  if (len)
-    *len = vx->selection_size;
+  *len = out-data;
 
   d(printf("selected text = \n");
-    fwrite(vx->selection_data, vx->selection_size, 1, stdout);
+    fwrite(data, out-data, 1, stdout);
     printf("\n"));
 
+  return data;
+}
+
+/*
+  return currently selected text
+*/
+char *vt_get_selection(struct _vtx *vx, int *len)
+{
+  if (vx->selection_data)
+    free(vx->selection_data);
+
+  vx->selection_data = vt_select_block(vx, vx->selstartx, vx->selstarty,
+				      vx->selendx, vx->selendy, len);
+  vx->selection_size = *len;
   return vx->selection_data;
 }
 
@@ -921,7 +911,7 @@ void vt_clear_selection(struct _vtx *vx)
 
   Called by vt_draw_selection
 */
-void vt_draw_selection_part(struct _vtx *vx, int sx, int sy, int ex, int ey)
+static void vt_draw_selection_part(struct _vtx *vx, int sx, int sy, int ex, int ey)
 {
   int tmp;
   struct vt_line *l;
@@ -988,8 +978,8 @@ void vt_draw_cursor(struct _vtx *vx, int state)
   unsigned char c;
   uint32 attr;
 
-  if (vx->vt.scrollbackoffset == 0) {
-    attr = vx->vt.this->data[vx->vt.cursorx];
+  if (vx->vt.scrollbackold == 0) {
+    attr = vx->vt.this_line->data[vx->vt.cursorx];
     c = attr & 0xff;
     if (c==9 || c==0)			/* remap tab */
       c=' ';
@@ -999,7 +989,7 @@ void vt_draw_cursor(struct _vtx *vx, int state)
       | ( attr & ~(VTATTR_FORECOLOURM|VTATTR_BACKCOLOURM));
     }
     vx->back_match=0;		/* forces re-draw? */
-    vt_draw_text(vx->user_data, vx->vt.cursorx, vx->vt.cursory, &c, 1, attr);
+    vt_draw_text(vx->vt.user_data, vx->vt.cursorx, vx->vt.cursory, &c, 1, attr);
   }
 }
 
@@ -1019,7 +1009,7 @@ struct _vtx *vtx_new(void *user_data)
 
   /* other parameters initialised to 0 by calloc */
 
-  vx->user_data = user_data;
+  vx->vt.user_data = user_data;
 
   return vx;
 }
