@@ -3,18 +3,18 @@
  *
  *  A virtual terminal emulator.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  GNU Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
@@ -30,14 +30,18 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <signal.h>
+#include <errno.h>
+#include <string.h>
 
 #include <termios.h>
 #include <sys/ioctl.h>
 
-#include "subshell.h"
+#include <glib.h>
+
 #include "lists.h"
 #include "memory.h"
 #include "vt.h"
+#include "subshell.h"
 
 #define SCROLLBACK_BUFFER	/* use scrollback buffer? */
 #define DOUBLE_BUFFER		/* use back buffer to represent screen
@@ -62,7 +66,25 @@ void vt_clear_line_portion(struct vt_em *vt, int start_col, int end_col);
 
 static unsigned char vt_remap_dec[256];
 
-void vt_dump(struct vt_em *vt)
+#ifdef DEBUG
+static void
+dump_scrollback(struct vt_em *vt)
+{
+  struct vt_line *wn, *nn;
+
+  printf("****** scrollback list:\n");
+  
+  wn=&vt->scrollback.head;
+  wn=&vt->lines.head;
+  while (wn) {
+    printf(" %p: p<: %p, n>:%p\n", wn, wn->prev, wn->next);
+    wn = wn->next;
+  }
+}
+#endif
+
+static void
+vt_dump(struct vt_em *vt)
 {
   struct vt_line *wn, *nn;
   int i;
@@ -72,7 +94,7 @@ void vt_dump(struct vt_em *vt)
   (printf("dumping state of vt buffer:\n"));
   while (nn) {
     for (i=0;i<wn->width;i++) {
-      (printf("%c", wn->data[i]&0xffff));
+      (printf("%c", wn->data[i]&VTATTR_DATAMASK));
     }
     (printf("\n"));
     wn=nn;
@@ -82,14 +104,14 @@ void vt_dump(struct vt_em *vt)
 }
 
 /***********************************************************************
- Update functions
-*/
+ * Update functions
+ */
 
 /*
-  set the screen, either
-    screen=0 for main screen
-          =1 for alternate screen
-*/
+ *set the screen, either
+ *  screen = 0 for main screen
+ *         = 1 for alternate screen
+ */
 
 static void vt_set_screen(struct vt_em *vt, int screen)
 {
@@ -147,17 +169,23 @@ static void vt_set_screen(struct vt_em *vt, int screen)
     if (screen)
       vt_clear_lines(vt, 0, vt->height);
 
-    vt->this = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
+    vt->this_line = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
     vt->mode = (vt->mode&~VTMODE_ALTSCREEN) | (screen?VTMODE_ALTSCREEN:0);
   }
 }  
 
 #ifdef SCROLLBACK_BUFFER
 
-/*
-  set the scrollback buffer size
-*/
-void vt_scrollback_set(struct vt_em *vt, int lines)
+/**
+ * vt_scrollback_set:
+ * @vt: the vt
+ * @lines: number of lines to set the scrollback to
+ *
+ * Sets the scrollback buffer size to @lines lines.  This will
+ * truncate the scrollback buffer if necessary.
+ */
+void
+vt_scrollback_set(struct vt_em *vt, int lines)
 {
   struct vt_line *ln;
 
@@ -172,14 +200,15 @@ void vt_scrollback_set(struct vt_em *vt, int lines)
 }
 
 /*
-  clone the line 'line' and add it to the bottom of
-  the scrollback buffer.
-
-  if the scrollback buffer is full, discard the oldest line
-
-  it is up-to the caller to free or 'discard' the old line
-*/
-void vt_scrollback_add(struct vt_em *vt, struct vt_line *wn)
+ * clone the line 'line' and add it to the bottom of
+ * the scrollback buffer.
+ *
+ * if the scrollback buffer is full, discard the oldest line
+ *
+ * it is up-to the caller to free or 'discard' the old line
+ */
+static void
+vt_scrollback_add(struct vt_em *vt, struct vt_line *wn)
 {
   struct vt_line *ln;
 
@@ -189,8 +218,10 @@ void vt_scrollback_add(struct vt_em *vt, struct vt_line *wn)
   ln = vt_mem_get(&vt->mem_list, sizeof(struct vt_line) + sizeof(uint32)*wn->width);/* always allocates 1 more 'char' */
   ln->width = wn->width;
   memcpy(ln->data, wn->data, wn->width*sizeof(uint32));
+
   /* add it to the scrollback buffer */
   vt_list_addtail(&vt->scrollback, (struct vt_listnode *)ln);
+  ln->line = ln->prev->line+1;
   
   d(printf("scrollback grow\n"));
   
@@ -198,15 +229,34 @@ void vt_scrollback_add(struct vt_em *vt, struct vt_line *wn)
   if (vt->scrollbacklines >= vt->scrollbackmax) {
     ln = (struct vt_line *)vt_list_remhead(&vt->scrollback); /* remove the top of list line */
     vt_mem_push(&vt->mem_list, ln, sizeof(struct vt_line)+sizeof(uint32)*ln->width);
+    /* need to track changes to this, even if they're not 'real' */
+    if (vt->scrollbackoffset) {
+      vt->scrollbackold--;
+      if ((-vt->scrollbackoffset) < vt->scrollbackmax)
+	vt->scrollbackoffset--;
+    }
   } else {
     vt->scrollbacklines++;
+    /* we've effectively moved the 'old' scrollback position */
+    if (vt->scrollbackoffset) {
+      vt->scrollbackold--;
+      vt->scrollbackoffset--;
+    }
   }
 }
 
 #endif /* SCROLLBACK_BUFFER */
 
-/* normal scroll */
-void vt_scroll_up(struct vt_em *vt, int count)
+/**
+ * vt_scroll_up:
+ * @vt: An initialised &vt_em.
+ * @count: Number of lines to scroll.
+ *
+ * Scrolls the emulator @vt up by @count lines.  The scrolling
+ * window is honoured.
+ */
+void
+vt_scroll_up(struct vt_em *vt, int count)
 {
   struct vt_line *wn, *nn;
   int i;
@@ -231,7 +281,7 @@ void vt_scroll_up(struct vt_em *vt, int count)
 #endif /* SCROLLBACK_BUFFER */
 
     for (i=0;i<wn->width;i++)
-      wn->data[i] = vt->attr|VTATTR_CHANGED;
+      wn->data[i] = vt->attr;
 
     if (wn->line == -1) {
       wn->modcount = wn->width;	/* make sure a wrap-scrolled line isn't marked clean */
@@ -250,8 +300,17 @@ void vt_scroll_up(struct vt_em *vt, int count)
   d(printf("vt_scroll_up() done\n"));
 }
 
-/* reverse scroll */
-void vt_scroll_down(struct vt_em *vt, int count)
+
+/**
+ * vt_scroll_down:
+ * @vt: An initialised &vt_em.
+ * @count: Number of lines to scroll.
+ *
+ * Scrolls the emulator @vt down by @count lines.  The scrolling
+ * window is honoured.
+ */
+void
+vt_scroll_down(struct vt_em *vt, int count)
 {
   struct vt_line *wn, *nn;
   int i;
@@ -267,7 +326,7 @@ void vt_scroll_down(struct vt_em *vt, int count)
     
     /* clear it */
     for (i=0;i<wn->width;i++) {
-      wn->data[i] = vt->attr|VTATTR_CHANGED;
+      wn->data[i] = vt->attr;
     }
     wn->modcount=0;
     wn->line = -1;		/* flag new line */
@@ -280,44 +339,53 @@ void vt_scroll_down(struct vt_em *vt, int count)
   }
 }
 
-void vt_insert_chars(struct vt_em *vt, int count)
+/**
+ * vt_insert_chars:
+ * @vt: An initialised &vt_em.
+ * @count: Number of character to insert.
+ *
+ * Inserts @count blank characters at the current cursor position.
+ */
+void
+vt_insert_chars(struct vt_em *vt, int count)
 {
   int i, j;
   struct vt_line *l;		/* FIXME: kludge */
 
   d(printf("vt_insert_chars(%d)\n", count));
-  l=vt->this;
+  l=vt->this_line;
 
   /* scroll data over count bytes */
   j = (l->width-count)-vt->cursorx;
   for (i=l->width-1;j>0;i--,j--) {
-    l->data[i] = l->data[i-count]|VTATTR_CHANGED;    
+    l->data[i] = l->data[i-count];
   }
 
   /* clear the rest of the line */
   for (i=vt->cursorx;i<vt->cursorx+count;i++) {
-    l->data[i] = vt->attr|VTATTR_CHANGED;
+    l->data[i] = vt->attr;
   }
   l->modcount+=count;
 }
 
-void vt_delete_chars(struct vt_em *vt, int count)
+void
+vt_delete_chars(struct vt_em *vt, int count)
 {
   int i, j;
   struct vt_line *l;
 
   d(printf("vt_delete_chars(%d)\n", count));
-  l=vt->this;
+  l=vt->this_line;
 
   /* scroll data over count bytes */
   j = (l->width-count)-vt->cursorx;
   for (i=vt->cursorx;j>0;i++,j--) {
-    l->data[i] = l->data[i+count]|VTATTR_CHANGED;
+    l->data[i] = l->data[i+count];
   }
 
   /* clear the rest of the line */
   for (i=l->width-count;i<l->width;i++) {
-    l->data[i] = vt->attr|VTATTR_CHANGED;
+    l->data[i] = vt->attr;
   }
   l->modcount+=count;
 
@@ -339,12 +407,7 @@ void vt_insert_lines(struct vt_em *vt, int count)
     
     /* clear it */
     for (i=0;i<wn->width;i++) {
-      if ((wn->data[i]&0xff) && ((wn->data[i] & 0xff) != ' ')) {
-	wn->data[i] = vt->attr|VTATTR_CHANGED; /* FIXME: include attributes */
-      }
-				/* NOTE: This should not have
-				   ATTR_CHANGED - taken care of by
-				   scroll update */
+      wn->data[i] = vt->attr;
     }
     wn->modcount=0;		/* set as 'unchanged' so the scroll
 				   routine can update it.
@@ -361,7 +424,7 @@ void vt_insert_lines(struct vt_em *vt, int count)
     count--;
   }
 
-  vt->this = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
+  vt->this_line = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
 }
 
 void vt_delete_lines(struct vt_em *vt, int count)
@@ -379,10 +442,7 @@ void vt_delete_lines(struct vt_em *vt, int count)
     
     /* clear it */
     for (i=0;i<wn->width;i++) {
-      if ((wn->data[i]&0xff) && (((wn->data[i] & 0xff) != ' ') || 
-		((wn->data[i] & (0x7ffff)) != vt->attr))) {
-	wn->data[i] = vt->attr|VTATTR_CHANGED; /* FIXME: include attributes */
-      }
+      wn->data[i] = vt->attr;
     }
     wn->modcount=0;
     /*wn->line=vt->scrollbottom;*/
@@ -394,7 +454,7 @@ void vt_delete_lines(struct vt_em *vt, int count)
     
     count--;
   }
-  vt->this = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
+  vt->this_line = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
 }
 
 void vt_clear_lines(struct vt_em *vt, int top, int count)
@@ -408,7 +468,7 @@ void vt_clear_lines(struct vt_em *vt, int top, int count)
   nn=wn->next;
   while(nn && count>=0) {
     for(i=0;i<wn->width;i++) {
-      wn->data[i] = vt->attr|VTATTR_CHANGED;
+      wn->data[i] = vt->attr;
     }
     wn->modcount = wn->width;
     count--;
@@ -419,16 +479,16 @@ void vt_clear_lines(struct vt_em *vt, int top, int count)
 
 void vt_clear_line_portion(struct vt_em *vt, int start_col, int end_col)
 {
-  struct vt_line *this;
+  struct vt_line *this_line;
   int i;
 
   d(printf("vt_clear_line_portion()\n"));
 
-  this = vt->this;
+  this_line = vt->this_line;
   for(i=start_col;i<end_col;i++) {
-    this->data[i] = vt->attr | VTATTR_CHANGED;
+    this_line->data[i] = vt->attr;
   }
-  this->modcount+=(this->width-vt->cursorx);
+  this_line->modcount+=(this_line->width-vt->cursorx);
 }
 
 
@@ -439,6 +499,8 @@ void vt_clear_line_portion(struct vt_em *vt, int start_col, int end_col)
 \********************************************************************************/
 static void vt_bell(struct vt_em *vt)
 {
+  if (vt->ring_my_bell)
+    vt->ring_my_bell(vt->user_data);
   d(printf("bell\n"));
 }
 
@@ -456,11 +518,11 @@ static void vt_lf(struct vt_em *vt)
   if (vt->cursory >= vt->scrollbottom) {
     d(printf("must scroll\n"));
     vt_scroll_up(vt, 1);
-    vt->this = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
+    vt->this_line = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
   } else {
     vt->cursory++;
     d(printf("new ypos = %d\n", vt->cursory));
-    vt->this = vt->this->next;
+    vt->this_line = vt->this_line->next;
   }
 }
 
@@ -468,28 +530,21 @@ static void vt_lf(struct vt_em *vt)
 static void vt_tab(struct vt_em *vt)
 {
   int nx;
+  unsigned char c;
+  struct vt_line *l;
+  int i;
 
   d(printf("tab\n"));
-  {				/* FIXME: neaten this */
-    unsigned char c;
-    struct vt_line *l;
-    int i;
 
-    l = vt->this;
-    if (((c=l->data[vt->cursorx]&0xff)==0) || (c==' ')) {
-      l->data[vt->cursorx] = 9|vt->attr|VTATTR_CHANGED; /* store 'tab' if we can ... helps cut/paste */
-    }
-				/* expand data after tab to be 'empty' */
-    nx = (vt->cursorx+8) & ~7;
-    for(i=vt->cursorx+1;i<nx;i++) {
-      if ( ((c=l->data[i]&0xff)==' ')) {
-	l->data[i] = vt->attr|VTATTR_CHANGED;
-      }
-    }
+  l = vt->this_line;
+  c=l->data[vt->cursorx]&VTATTR_DATAMASK;
+  /* dont store tab over a space - will affect attributes */
+  if (c==0) {
+    l->data[vt->cursorx] = 9|vt->attr; /* store 'tab' if we can ... helps cut/paste */
   }
 
-  /*nx = (vt->cursorx+8) & ~7;*/
-  vt->cursorx = (vt->cursorx + 8) & 0xfff8;
+  /* move cursor to new tab position */
+  vt->cursorx = (vt->cursorx + 8) & (~7);
   if (vt->cursorx >= vt->width) {
     vt->cursorx = 0;
     vt_lf(vt);			/* goto next line */
@@ -544,7 +599,8 @@ static void vt_delete_char(struct vt_em *vt)
 }
 
 /* insert lines and scroll down */
-static void vt_insert_line(struct vt_em *vt)
+static void
+vt_insert_line(struct vt_em *vt)
 {
   d(printf("insert line(s)\n"));
   if (vt->argcnt==0) {
@@ -557,7 +613,8 @@ static void vt_insert_line(struct vt_em *vt)
 }
 
 /* delete lines and scroll up */
-static void vt_delete_line(struct vt_em *vt)
+static void
+vt_delete_line(struct vt_em *vt)
 {
   d(printf("delete line(s)\n"));
   if (vt->argcnt==0) {
@@ -571,44 +628,63 @@ static void vt_delete_line(struct vt_em *vt)
   } else {
     d(printf("delete characters got >1 parameters\n"));
   }
-  vt->this = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
+  vt->this_line = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
 }
 
 /* clear from current cursor position to end of screen */
-static void vt_cleareos(struct vt_em *vt)
+static void
+vt_cleareos(struct vt_em *vt)
 {
+  int arg=0;
+
   d(printf("clear screen/end of screen\n"));
-  vt_clear_line_portion(vt, vt->cursorx, vt->this->width);
-  if (vt->cursory<vt->height) {
+
+  if (vt->argcnt)
+    arg = atoi(vt->args[0]);
+
+  switch(arg) {
+  case 0:			/* clear here to end of screen */
+    vt_clear_line_portion(vt, vt->cursorx, vt->this_line->width);
     vt_clear_lines(vt, vt->cursory+1, vt->height);
+    break;
+  case 1:			/* clear top of screen to here */
+    vt_clear_line_portion(vt, 0, vt->cursorx);
+    vt_clear_lines(vt, 0, vt->cursory);
+    break;
+  case 2:			/* clear the whole damn thing */
+    vt_clear_lines(vt, 0, vt->height);
+    break;
   }
 }
 
-static void vt_clear_lineportion(struct vt_em *vt)
+static void
+vt_clear_lineportion(struct vt_em *vt)
 {
   d(printf("Clear part of line\n"));
   if (vt->argcnt>1) {
     /* eat the command */
   } else if (vt->argcnt==0 || *vt->args[0] == '0') {
-    vt_clear_line_portion(vt, vt->cursorx, vt->this->width);
+    vt_clear_line_portion(vt, vt->cursorx, vt->this_line->width);
   } else if (*vt->args[0] == '1') {
     vt_clear_line_portion(vt, 0, vt->cursorx + 1);
   } else if (*vt->args[0] == '2') {
-    vt_clear_line_portion(vt, 0, vt->this->width);
+    vt_clear_line_portion(vt, 0, vt->this_line->width);
   }
   /* eat bad parameters */
 }
 
 
 /* cursor save/restore */
-static void vt_save_cursor(struct vt_em *vt)
+static void
+vt_save_cursor(struct vt_em *vt)
 {
   d(printf("save cursor\n"));
   vt->savex = vt->cursorx;
   vt->savey = vt->cursory;
 }
 
-static void vt_restore_cursor(struct vt_em *vt)
+static void
+vt_restore_cursor(struct vt_em *vt)
 {
   d(printf("restore cursor\n"));
   vt->cursorx = vt->savex;
@@ -620,13 +696,14 @@ static void vt_restore_cursor(struct vt_em *vt)
   if (vt->cursory >= vt->height)
     vt->cursory = vt->height-1;
 
-  vt->this = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
-  d(printf("found line %d, %p\n", vt->cursory, vt->this));
+  vt->this_line = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
+  d(printf("found line %d, %p\n", vt->cursory, vt->this_line));
 }
 
 /* cursor movement */
 /* in app-cursor mode, this will cause the screen to scroll ? */
-static void vt_up(struct vt_em *vt)
+static void
+vt_up(struct vt_em *vt)
 {
   int count=1;
 
@@ -641,10 +718,11 @@ static void vt_up(struct vt_em *vt)
     vt->cursory=vt->scrolltop;
   }
 
-  vt->this = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
+  vt->this_line = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
 }
 
-static void vt_down(struct vt_em *vt)
+static void
+vt_down(struct vt_em *vt)
 {
   int count=1;
 
@@ -653,13 +731,14 @@ static void vt_down(struct vt_em *vt)
     count=atoi(vt->args[0]);
 
   vt->cursory+=count;
-  if (vt->cursory>=vt->scrollbottom)
-    vt->cursory=vt->scrollbottom-1;
+  if (vt->cursory>vt->scrollbottom)
+    vt->cursory=vt->scrollbottom;
 
-  vt->this = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
+  vt->this_line = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
 }
 
-static void vt_right(struct vt_em *vt)
+static void
+vt_right(struct vt_em *vt)
 {
   int count=1;
 
@@ -672,7 +751,8 @@ static void vt_right(struct vt_em *vt)
     vt->cursorx=vt->width-1;
 }
 
-static void vt_left(struct vt_em *vt)
+static void
+vt_left(struct vt_em *vt)
 {
   int count=1;
 
@@ -686,21 +766,22 @@ static void vt_left(struct vt_em *vt)
 }
 
 /* jump to a cursor position */
-static void vt_goto(struct vt_em *vt)
+static void
+vt_goto(struct vt_em *vt)
 {
   d(printf("goto position\n"));
   if (vt->argcnt==0) {
     vt->cursorx=0;
     vt->cursory=0;
-    /*vt->this = vt->first;*/
+    /*vt->this_line = vt->first;*/
   } else if (vt->argcnt==1) {
     vt->cursory = atoi(vt->args[0])-1;
     vt->cursorx=0;
-    /*vt->this = vt_list_index(&vt->first, vt->cursory);*/
+    /*vt->this_line = vt_list_index(&vt->first, vt->cursory);*/
   } else if (vt->argcnt==2) {
     vt->cursory = atoi(vt->args[0])-1;
     vt->cursorx = atoi(vt->args[1])-1;
-    /*vt->this = vt_list_index(&vt->first, vt->cursory);*/
+    /*vt->this_line = vt_list_index(&vt->first, vt->cursory);*/
   } else {
     d(printf("position had too many parameters\n"));
   }
@@ -714,11 +795,12 @@ static void vt_goto(struct vt_em *vt)
     vt->cursory = vt->height-1;
   d(printf("pos = %d %d\n", vt->cursory, vt->cursorx));
 
-  vt->this = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
+  vt->this_line = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
 }
 
 /* various mode stuff */
-static void vt_modeh(struct vt_em *vt)
+static void
+vt_modeh(struct vt_em *vt)
 {
   d(printf("mode h set\n"));
   if (vt->argcnt==1) {
@@ -747,7 +829,8 @@ static void vt_modeh(struct vt_em *vt)
 }
 
 
-static void vt_model(struct vt_em *vt)
+static void
+vt_model(struct vt_em *vt)
 {
 d(printf("mode l set\n"));
   if (vt->argcnt==1) {
@@ -774,7 +857,8 @@ d(printf("mode l set\n"));
   }
 }
 
-static void vt_modek(struct vt_em *vt)
+static void
+vt_modek(struct vt_em *vt)
 {
   d(printf("mode k set\n"));
   if (vt->argcnt==1) {
@@ -790,7 +874,8 @@ static void vt_modek(struct vt_em *vt)
 }
 
 
-static void vt_mode(struct vt_em *vt)
+static void
+vt_mode(struct vt_em *vt)
 {
   int i, j;
   int mode_map[] = {0, VTATTR_BOLD, 0, 0,
@@ -818,9 +903,10 @@ static void vt_mode(struct vt_em *vt)
 }
 
 /*
-  set g0-g3 charset
-*/
-static void vt_gx_set(struct vt_em *vt)
+ *  sets g0-g3 charset
+ */
+static void
+vt_gx_set(struct vt_em *vt)
 {
   int index;
   unsigned char *table;
@@ -848,17 +934,19 @@ static void vt_gx_set(struct vt_em *vt)
 /*
   device status report
 */
-static void vt_dsr(struct vt_em *vt)
+static void
+vt_dsr(struct vt_em *vt)
 {
   char status[16];
 
   if (vt->argcnt==1) {
     switch(*(vt->args[0])) {
     case '5':			/* report 'ok' status */
-      sprintf(status, "\033[0n");
+      g_snprintf(status, sizeof(status), "\033[0n");
       break;
     case '6':			/* report cursor position */
-      sprintf(status, "\033[%d;%dR", vt->cursory+1, vt->cursorx+1);
+      g_snprintf(status, sizeof(status), "\033[%d;%dR",
+      		 vt->cursory+1, vt->cursorx+1);
       break;
     default:
       status[0]=0;
@@ -867,7 +955,8 @@ static void vt_dsr(struct vt_em *vt)
   }
 }
 
-static void vt_scroll(struct vt_em *vt)
+static void
+vt_scroll(struct vt_em *vt)
 {
   d(printf("set scroll region called\n"));
   if (vt->argcnt==0) {
@@ -889,18 +978,19 @@ static void vt_scroll(struct vt_em *vt)
 }
 
 /*
-  master soft reset ^[c
-*/
-static void vt_reset(struct vt_em *vt)
+ * master soft reset ^[c
+ */
+static void
+vt_reset(struct vt_em *vt)
 {
   vt->cursorx=0;
   vt->cursory=0;
-  vt->this = (struct vt_line *)vt->lines.head;
+  vt->this_line = (struct vt_line *)vt->lines.head;
   vt->attr=VTATTR_CLEAR;	/* reset attributes */
   vt->remaptable = 0;		/* no character remapping */
   vt_set_screen(vt, 0);
   vt->mode = 0;
-  vt_cleareos(vt);
+  vt_clear_lines(vt, 0, vt->height);
 
   vt->Gx=0;			/* reset all fonts */
   vt->G[0]=0;
@@ -910,7 +1000,8 @@ static void vt_reset(struct vt_em *vt)
 }
 
 /* function keys */
-static void vt_func(struct vt_em *vt)
+static void
+vt_func(struct vt_em *vt)
 {
   int i;
 
@@ -937,6 +1028,45 @@ static void vt_func(struct vt_em *vt)
   }
 }
 
+/*
+  'set text' called, call a callback if we have one
+
+  The callback is called with VTTTILE_* constants and the string
+  to set.
+*/
+static void
+vt_set_text(struct vt_em *vt)
+{
+  char *p;
+  int i;
+
+  if (vt->change_my_name) {
+    p = strchr(vt->argptr[0], ';');
+    if (p) {
+      *p=0;
+      p++;
+      i = atoi(vt->argptr[0]);
+      switch(i) {
+      case 0:
+	i = VTTITLE_WINDOWICON;
+	break;
+      case 1:
+	i = VTTITLE_ICON;
+	break;
+      case 2:
+	i = VTTITLE_WINDOW;
+	break;
+      case 46:			/* log file .. disabled */
+      case 50:			/* set font .. disabled too */
+	return;
+      default:			/* dont care, piss off */
+	return;
+      }
+      vt->change_my_name(vt->user_data, i, p);
+    }
+  }
+}
+
 struct vt_jump {
   void (*process)(struct vt_em *vt);	/* process function */
   int modes;			/* modes appropriate */
@@ -989,33 +1119,41 @@ struct vt_jump vtjumps[] = {
   {0,VT_LIT}, {0,VT_LIT}, {vt_func,VT_EBL}, {0,VT_LIT},	/* |}~? */
 };
 
-  /* parse vt commands:
-     states:
-     0: normal escape mode
-     1: escape mode.  switch on next character.
-     2: '[' escape mode (keep finding numbers until a command code found)
-     3: 'O' escape mode.  switch on next character.
-     4: ']' escape mode.  swallows until following bell (or newline).
-     needs a little work on '[' mode (with parameter grabbing)
-  */
-void parse_vt(struct vt_em *vt, char *ptr, int length)
+/**
+ * vt_parse_vt:
+ * @vt: An initialised &vt_em.
+ * @ptr: Text to parse.
+ * @length: Length of text available at @ptr
+ *
+ * Parses a sequence of characters using an xterm compatible
+ * terminal state machine.  All state and terminal output is
+ * stored in @vt.
+ */
+void
+vt_parse_vt (struct vt_em *vt, char *ptr, int length)
 {
   register int c;
   register int state;
   register int mode;
-  int handled;
   struct vt_jump *modes = vtjumps;
   char *ptrend;
   void (*process)(struct vt_em *vt);	/* process function */
 
+  /* states:
+   *  0: normal escape mode
+   *   1: escape mode.  switch on next character.
+   *   2: '[' escape mode (keep finding numbers until a command code found)
+   *   3: 'O' escape mode.  switch on next character.
+   *   4: ']' escape mode.  swallows until following bell (or newline).
+   *   needs a little work on '[' mode (with parameter grabbing)
+   */
+
   state = vt->state;
   ptrend = ptr+length;
   while (ptr<ptrend) {
     c=(*ptr++) & 0xff;		/* convert to unsigned byte */
     mode = modes[c & 0x7f].modes;
     process = modes[c & 0x7f].process;
-    do {
-      handled = 1;
       switch(state) {
       case 0:
 	if (mode & VT_LIT) {
@@ -1034,8 +1172,8 @@ void parse_vt(struct vt_em *vt, char *ptr, int length)
 	  }
 
 	  /* output character */
-	  vt->this->data[vt->cursorx] = ((vt->attr | VTATTR_CHANGED) & 0xffff0000) | c;
-	  vt->this->modcount++;
+	  vt->this_line->data[vt->cursorx] = ((vt->attr) & VTATTR_MASK) | c;
+	  vt->this_line->modcount++;
 	  d(printf("literal %c\n", c));
 	  vt->cursorx++;
 	} else if (mode & VT_CON) {
@@ -1063,7 +1201,7 @@ void parse_vt(struct vt_em *vt, char *ptr, int length)
 	  vt->outptr = vt->argptr[0];
 	  vt->outend = vt->outptr+VTPARAM_ARGMAX*VTPARAM_MAXARGS-1;
 	} else if (mode & VT_EXA) {
-	  vt->args[0][0]=c;
+	  vt->args[0][0]=c & 0x7f;
 	  state = 5;
 	} else {
 	  state = 0;		/* dont understand input */
@@ -1103,6 +1241,7 @@ void parse_vt(struct vt_em *vt, char *ptr, int length)
 	if (c==0x07) {
 	  /* handle output */
 	  *(vt->outptr)=0;
+	  vt_set_text(vt);
 	  d(printf("received text mode: %s\n", vt->argptr[0]));
 	  state = 0;
 	} else if (c==0x0a) {
@@ -1114,25 +1253,23 @@ void parse_vt(struct vt_em *vt, char *ptr, int length)
 	break;
       case 5:			/* \E?x */
 	vt->args[0][1]=c;
-	if (vt->args[0][0] < 0x7f) {
-	  process = modes[vt->args[0][0]].process;
-	  if (process) {		/* check, if doesn't exist, then ignore and lose */
-	    process(vt);
-	  }
-	} else {
-	  handled = 0;
-	}
+	modes[vt->args[0][0]].process(vt);
 	state=0;
 	break;
       }
-    } while (!handled);
   }
   vt->state = state;
 }
 
-/*
-  Allocate a new line
-*/
+/**
+ * vt_newline:
+ * @vt: An initialised &vt_em.
+ *
+ * Allocates a new line using malloc().  The line matches the
+ * current terminal size stored in @vt.
+ *
+ * Return value: A pointer to the newly allocated &vt_line structure.
+ */
 struct vt_line *vt_newline(struct vt_em *vt)
 {
   struct vt_line *n;
@@ -1145,19 +1282,26 @@ struct vt_line *vt_newline(struct vt_em *vt)
     n->width = vt->width;
     n->modcount = vt->width;
     for (i=0;i<len+1;i++) {
-      n->data[i] = vt->attr | VTATTR_CHANGED;
+      n->data[i] = vt->attr;
     }
     n->line = j++;
   }
   return n;
 }
 
-/*
-  Setup a new VT terminal
-
-  FIXME: check allocations
-*/
-struct vt_em *vt_init(struct vt_em *vt, int width, int height)
+/**
+ * vt_init:
+ * @vt: Pointer to an uninitialised &vt_em structure.
+ * @width: Desired width of the terminal, in characters.
+ * @height: Desired height of the terminal, in characters.
+ *
+ * Initialise a new terminal emulator.
+ *
+ * Return value: Returns a pointer to the intialised terminal.
+ * This will be the same as the value of @vt, passed in.
+ */
+struct vt_em *
+vt_init(struct vt_em *vt, int width, int height)
 {
   struct vt_line *vl;
   int i;
@@ -1200,7 +1344,7 @@ struct vt_em *vt_init(struct vt_em *vt, int width, int height)
     vt->args[i]=&vt->args_mem[i*VTPARAM_ARGMAX];
   }
 
-  vt->this = (struct vt_line *)vt->lines.head;
+  vt->this_line = (struct vt_line *)vt->lines.head;
 
   vt->scrollbacklines=0;
   vt->scrollbackoffset=0;
@@ -1219,78 +1363,139 @@ struct vt_em *vt_init(struct vt_em *vt, int width, int height)
 
   vt_mem_init(&vt->mem_list);
 
+  vt->ring_my_bell = 0L;
+  vt->change_my_name = 0L;
+
+  vt->user_data = 0;
   return vt;
 }
 
-/*
-  start a child process running in the VT emulator
-
-   fork the child process
-
-*/
-int vt_forkpty(struct vt_em *vt)
+/**
+ * vt_forkpty:
+ * @vt: An initialised &vt_em.
+ * @do_uwtmp_log: If %TRUE, updates utmp/wtmp records.
+ *
+ * Start a child process running in the VT emulator using fork, and
+ * giving it a pty to communicate through.
+ *
+ * Return value: The process id of the child process, or -1 on error.
+ * See Also: fork(2)
+ */
+pid_t
+vt_forkpty (struct vt_em *vt, int do_uwtmp_log)
 {
   char ttyname[256];
 
-  vt->childpid = init_subshell(&vt->childfd, ttyname, &vt->msgfd);
-  if (vt->childpid>0) {
-    fcntl(vt->childfd, F_SETFL, O_NONBLOCK);
-    resize_subshell(vt->childfd, vt->width, vt->height, vt->width*8, vt->height*8); /* FIXME: approx sizes only */
+  if (zvt_init_subshell(vt, ttyname, do_uwtmp_log) == -1)
+	  return -1;
+  
+  if (vt->childpid > 0){
+	  fcntl(vt->childfd, F_SETFL, O_NONBLOCK);
+
+	  /* FIXME: approx sizes only */
+	  zvt_resize_subshell (vt->childfd, vt->width, vt->height, vt->width*8, vt->height*8); 
   }
 
   d(fprintf(stderr, "program started on pid %d, on tty %s\n", vt->childpid, ttyname));
   return vt->childpid;
 }
 
-/*
-  write to the child process
-*/
-int vt_writechild(struct vt_em *vt, char *buffer, int len)
+/**
+ * vt_writechild:
+ * @vt: An initialised &vt_em, which has had a subprocess started using vt_forkpty().
+ * @buffer: Data to write.
+ * @len: Length of valid data pointed to by @buffer.
+ *
+ * Write characters to the child process started using vt_forkpty().
+ *
+ * Return value: The number of characters successfully written.
+ * See Also: write(2)
+ */
+int
+vt_writechild(struct vt_em *vt, char *buffer, int len)
 {
   return write(vt->childfd, buffer, len);
 }
 
-/*
-  read from the child process
-*/
-int vt_readchild(struct vt_em *vt, char *buffer, int len)
+/**
+ * vt_readchild:
+ * @vt: An initialised &vt_em, which has had a subprocess started using vt_forkpty().
+ * @buffer: Buffer to store data.
+ * @len: Maximum number of characters to store in @buffer.
+ *
+ * Read upto @len characters from the child process into @buffer.
+ *
+ * Return value: The number of characters read into @buffer, or -1 on error.
+ * See Also: read(2).
+ */
+int
+vt_readchild(struct vt_em *vt, char *buffer, int len)
 {
   return read(vt->childfd, buffer, len);
 }
 
-/*
-  signal the child process
-*/
-int vt_killchild(struct vt_em *vt, int signal)
+/**
+ * vt_killchild:
+ * @vt: An initialised &vt_em, which has had a subprocess started using vt_forkpty().
+ * @signal: Signal number to send to child process.
+ *
+ * Send a signal to the child proccess created using vt_forkpty().
+ *
+ * Return value: Zero if successful, otherwise returns -1 and
+ * sets errno.
+ * See Also: kill(2), signal(5).
+ */
+int
+vt_killchild(struct vt_em *vt, int signal)
 {
   if (vt->childpid!=-1) {
     return kill(vt->childpid, signal);
   }
+  errno = ENOENT;
   return -1;
 }
 
-/*
-  close the child connection pty, and invalidates it.
-*/
-int vt_closepty(struct vt_em *vt)
+/**
+ * vt_closepty:
+ * @vt: An initialised &vt_em, which has had a subprocess started using vt_forkpty().
+ *
+ * Close the child connection pty, and invalidates it.  After calling this
+ * function, do not call functions which operate on the child process.
+ *
+ * Return value: Returns zero on success, or -1 on error, and sets errno.
+ * See Also: close(2)
+ */
+int
+vt_closepty(struct vt_em *vt)
 {
   int ret;
 
   d(printf("vt_closepty called\n"));
 
   if (vt->childfd != -1){
+	  zvt_shutdown_subshell (vt);
 	  ret = close(vt->childfd);
-	  close_msgfd(vt->childpid);
+	  zvt_close_msgfd(vt->childpid);
 	  vt->msgfd = vt->childfd = -1;
   } else
 	  ret = 0;
   return ret;
 }
 
-/*
-  destroy all data associated with a given 'vt' structure
-*/
-void vt_destroy(struct vt_em *vt)
+/**
+ * vt_destroy:
+ * @vt: An initialised &vt_em.
+ *
+ * Destroy all data associated with a given &vt_em structure.
+ *
+ * It is safe to call this function if a child process has been created
+ * using vt_forkpty().
+ *
+ * After this function returns, @vt is no longer a valid emulator structure
+ * and may only be passed to vt_init() for re-initialisation.
+ */
+void
+vt_destroy(struct vt_em *vt)
 {
   struct vt_line *wn;
 
@@ -1321,16 +1526,24 @@ void vt_destroy(struct vt_em *vt)
   /* done */
 }
 
-/*
-  resize the window to a new window size
-
-  handles tty re-size too.
-*/
+/**
+ * vt_resize:
+ * @vt: An initalised &vt_em.
+ * @width: New terminal width, in characters.
+ * @height: New terminal height, in characters.
+ * @pixwidth: New terminal width, in pixels.
+ * @pixheight: New terminal height, in pixels.
+ *
+ * Resize the emulator pages to the new size.  The pty is resized too.
+ * The scrollback buffer takes part in the resize, but scrollback lines
+ * are NOT resized.
+ */
 void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixheight)
 {
   int count;
   struct vt_line *wn, *nn;
   int i;
+  uint32 c;
 #ifdef DOUBLE_BUFFER
   int pass;
 #endif
@@ -1381,17 +1594,51 @@ void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixhei
       d(printf("scroll top too big, reset\n"));
     }
   } else if (height>vt->height) {
-    /* if we just got bigger, add new lines to the bottom */
-    /* FIXME: this should look in the scrollback buffer if it exists 
-      no - would upset the memory allocation of the scrollback buffer */
+    /* if we just got bigger, try and recover scrollback lines into the top
+       of the buffer, or failing that, add new lines to the bottom */
     count = (height-vt->height);
     d(printf("adding %d lines to buffer window\n", count));
     for(i=0;i<count;i++) {
-      vt_list_addtail(&vt->lines, (struct vt_listnode *)vt_newline(vt));
+#ifdef SCROLLBACK_BUFFER
+      /* if scrollback exists, remove it from the buffer and add it to the
+	 top of the screen */
+      if (vt->scrollbacklines>0) {
+	int len;
+	int j;
+
+	d(printf("removing scrollback -> top of screen\n"));
+
+	nn = vt_newline(vt);
+	wn = (struct vt_line *)vt_list_remtail(&vt->scrollback);
+
+	len = nn->width<wn->width?nn->width:wn->width;
+	memcpy(nn->data, wn->data, len*sizeof(nn->data[0]));
+	/* clear rest of screen (if it exists) with blanks of the
+	   same attributes as the last character */
+	c = nn->data[len-1]&VTATTR_MASK;
+	for(j=wn->width;j<nn->width;j++) {
+	  nn->data[j]=c;
+	}
+	vt_mem_unget(&vt->mem_list, wn);
+	vt_list_addhead(&vt->lines, (struct vt_listnode *)nn);
+	vt_list_addhead(&vt->lines_alt, (struct vt_listnode *)vt_newline(vt));
 #ifdef DOUBLE_BUFFER
-      vt_list_addtail(&vt->lines_back, (struct vt_listnode *)vt_newline(vt));
+	vt_list_addhead(&vt->lines_back, (struct vt_listnode *)vt_newline(vt));
+#endif
+	vt->scrollbacklines--;	/* since we just nuked one */
+	vt->cursory++;		/* fix up cursor */
+      } else {
+#endif /* SCROLLBACK_BUFFER */
+	d(printf("adding line to bottom of screen\n"));
+	/* otherwise just add blank lines to the bottom */
+	vt_list_addtail(&vt->lines, (struct vt_listnode *)vt_newline(vt));
+#ifdef DOUBLE_BUFFER
+	vt_list_addtail(&vt->lines_back, (struct vt_listnode *)vt_newline(vt));
+#endif
+	vt_list_addtail(&vt->lines_alt, (struct vt_listnode *)vt_newline(vt));
+#ifdef SCROLLBACK_BUFFER
+      } /* if scrollbacklines */
 #endif
-      vt_list_addtail(&vt->lines_alt, (struct vt_listnode *)vt_newline(vt));
     }
   } /* otherwise width may have changed? */
 
@@ -1421,8 +1668,12 @@ void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixhei
 	}
 	wn->next->prev = wn;	/* re-link line into linked list */
 	wn->prev->next = wn;
+	if (wn->width)
+	  c = wn->data[wn->width-1]&VTATTR_MASK;
+	else
+	  c = vt->attr;
 	for(i=wn->width;i<width;i++) { /* if the line got bigger, fix it up */
-	  wn->data[i]=vt->attr | VTATTR_CHANGED;
+	  wn->data[i]=c;
 	  wn->modcount++;		/* FIXME: optimise? */
 	}
 	wn->width = width;
@@ -1439,25 +1690,27 @@ void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixhei
 #endif
 
   /* re-fix 'this line' pointer */
-  vt->this = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
+  vt->this_line = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
 
-  resize_subshell(vt->childfd, width, height, pixwidth, pixheight);
+  zvt_resize_subshell(vt->childfd, width, height, pixwidth, pixheight);
 
-  d(printf("resized to %d,%d, this = %p\n", vt->width, vt->height, vt->this));
+  d(printf("resized to %d,%d, this = %p\n", vt->width, vt->height, vt->this_line));
 }
 
-/*
-  reports a button click to the terminal, (if it is asking
-  for them at the moment).
-
-  button = 1, 2, 3.  If button=0, then signifies button up
-  qual = qualifiers (shift/control/etc)
-      bit 
-
-  returns true if the button event should be swallowed
-
-  FIXME: handle qualifiers
-*/
+/**
+ * vt_report_button:
+ * @vt: An initialised &vt_em.
+ * @button: Mouse button, from 1 to 3.  0 signifies button-up.
+ * @qual: Button event qualifiers.  Currently unused.
+ * @x: The horizontal character position of the mouse event.
+ * @y: The vertical character position of the mouse event.
+ *
+ * Reports a button click to the terminal, if it has requested
+ * them to be sent.  The terminal coordinates begin at 0, starting
+ * from the top-left corner of the terminal.
+ *
+ * Return value: Returns %TRUE if the button event was sent.
+ */
 int vt_report_button(struct vt_em *vt, int button, int qual, int x, int y)
 {
   char mouse_info[16];
@@ -1466,7 +1719,7 @@ int vt_report_button(struct vt_em *vt, int button, int qual, int x, int y)
        (y>=0) /*&&
 		!(event->state & GDK_SHIFT_MASK)*/
        ) {
-    sprintf(mouse_info, "\033[M%c%c%c",
+    g_snprintf(mouse_info, sizeof(mouse_info), "\033[M%c%c%c",
 	    ' ' + ((button - 1)&3) /*+ 
 				     ((event->state&GDK_SHIFT_MASK)?4:0) +
 				     ((event->state&GDK_MOD1_MASK)?8:0) +
