@@ -3,18 +3,18 @@
  *
  *  Screen definitions and structures
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  GNU Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
@@ -27,6 +27,10 @@
 /* include the toolkit */
 #include <gtk/gtk.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 /* defines for screen update routine */
 #define UPDATE_CHANGES 0x00	/* only update changed areas */
 #define UPDATE_REFRESH 0x01	/* just refersh all */
@@ -64,9 +68,6 @@ struct _vtx {
   int selstartxold, selstartyold;
   int selendxold, selendyold;
 
-  /* needed to pass args to render functions */
-  void *user_data;
-
 #ifdef STAND_ALONE
   /* all tool-kit specific data goes here ... */
   GtkWidget *drawingarea;	/* rendering area */
@@ -83,8 +84,6 @@ struct _vtx {
 };
 
 
-void vt_draw_text_select(struct _vtx *vx, int col, int row, char *text, int len, int attr);
-
 /* from update.c */
 char *vt_get_selection   (struct _vtx *vx, int *len);
 void vt_clear_selection  (struct _vtx *vx);
@@ -100,10 +99,16 @@ void vtx_set_fontsize    (struct _vtx *vx, int width, int height);
 			 
 /* defined by caller */	 
 void vt_draw_text        (void *user_data, int col, int row, char *text, int len, int attr);
-void vt_scroll_area      (void *user_data, int firstrow, int count, int offset);
+void vt_scroll_area      (void *user_data, int firstrow, int count, int offset, int fill);
 int  vt_cursor_state     (void *user_data, int state);
 void vt_hightlight_block (void *user_data, int col, int row, int width, int height);
 int  vt_get_attr_at      (struct _vtx *vx, int col, int row);
 
+
+#ifdef __cplusplus
+	   }
+#endif /* __cplusplus */
+
+
 #endif
 
