@@ -33,11 +33,25 @@
 
 #include "zvtterm.h"
 
-void child_died_event(ZvtTerm *term)
+GtkWindow *window;
+
+static void child_died_event(ZvtTerm *term)
 {
   exit(0);
 }
 
+static void title_changed_event(ZvtTerm *term, VTTITLE_TYPE type, char *newtitle)
+{
+  switch(type) {
+  case VTTITLE_WINDOW:
+  case VTTITLE_WINDOWICON:
+    gtk_window_set_title(window, newtitle);
+    break;
+  default:
+    break;
+  }
+}
+
 /*
   main routine
 
@@ -47,9 +61,9 @@ gint main (gint argc, gchar *argv[])
 {
   int c;
   int cmdindex;
+  int login_shell = 0;
   int scrollbacklines;
   struct passwd *pw;
-  GtkWindow *window;
   ZvtTerm *term;
   GtkWidget *table;
   GtkWidget *scrollbar;
@@ -59,7 +73,7 @@ gint main (gint argc, gchar *argv[])
   /* process arguments */
   cmdindex = 0;
   scrollbacklines = 50;
-  while ( (cmdindex==0) && (c=getopt(argc, argv, "e:s:")) != EOF ) {
+  while ( (cmdindex==0) && (c=getopt(argc, argv, "le:s:")) != EOF ) {
     switch(c) {
     case 'e':
       cmdindex = optind-1;	/* index of argv array to pass to exec */
@@ -67,6 +81,9 @@ gint main (gint argc, gchar *argv[])
     case 's':
       scrollbacklines = atoi(optarg);
       break;
+    case 'l':
+      login_shell = 1;
+      break;
     }
   }
 
@@ -84,6 +101,9 @@ gint main (gint argc, gchar *argv[])
   gtk_signal_connect (GTK_OBJECT (term), "child_died",
                       (GtkSignalFunc) child_died_event, NULL);
 
+  gtk_signal_connect (GTK_OBJECT (term), "title_changed",
+                      (GtkSignalFunc) title_changed_event, NULL);
+
   scrollbar = gtk_vscrollbar_new (GTK_ADJUSTMENT (term->adjustment));
   GTK_WIDGET_UNSET_FLAGS (scrollbar, GTK_CAN_FOCUS);
 
@@ -102,8 +122,9 @@ gint main (gint argc, gchar *argv[])
   gtk_widget_show (table);
   gtk_widget_show (GTK_WIDGET(window));
 
+  gdk_window_set_hints (((GtkWidget *)window)->window, 0, 0, 50, 50, 0, 0, GDK_HINT_MIN_SIZE);
 
-  switch (zvt_term_forkpty(term)) {
+  switch (zvt_term_forkpty(term, TRUE)) {
   case -1:
     perror("ERROR: unable to fork:");
     exit(1);
@@ -112,12 +133,25 @@ gint main (gint argc, gchar *argv[])
     if (cmdindex) {
       execvp(argv[cmdindex], &argv[cmdindex]);
     } else {
+      char basename [BUFSIZ];
+
 				/* get shell from passwd */
-      pw = getpwuid(getpid());
-      if (pw) {
-	execl(pw->pw_shell, rindex(pw->pw_shell, '/'), 0);
+      pw = getpwuid(getuid());
+      if (login_shell) {
+	if (pw) {
+	  chdir(pw->pw_dir);
+	  g_snprintf(basename, sizeof(basename), "-%s",
+	  	     rindex(pw->pw_shell, '/'));
+	  execl(pw->pw_shell, basename, NULL);
+	} else {
+	  execl("/bin/bash", "-bash", NULL);
+	}
       } else {
-	execl("/bin/bash", "bash", 0);
+	if (pw) {
+	  execl(pw->pw_shell, strrchr(pw->pw_shell, '/'), NULL);
+	} else {
+	  execl("/bin/bash", "bash", NULL);
+	}
       }
     }
     perror("ERROR: Cannot exec command:");
